# This script generates a graph which shows the structure of the parameter data
proc pdbGraph { {pdbParrent "all"} {dispVal "0"} } {
if { $pdbParrent=="all" } {
	set pdbOutput [pdbDump]				;# pass the entire pdb
} else {
	set pdbOutput [pdbDump $pdbParrent]	;# pass the specified pdb section
}

set fileId [open tempFile.txt "w"]		;# make a temporary text file to hold the pdb data
puts -nonewline $fileId $pdbOutput
close $fileId
set fp [open tempFile.txt "r"]

set subParrentLevel 2
set oldSubParrentLevel 2

set graph "digraph pdb_print { " 		;# graph is the string that will contain all the information used to make the plot
set nodeNum 1 ;# to remain unique each node must have a different number
while { [gets $fp line]!=-1 } { ;# goes through each line of the tempFil

	if { ([llength $line] == 1) && ([regexp {\s\s\s\s} $line] == 0) } { ;# checks if this is a parent and not a sub parrent, "regexp {\s\s\s\s} $line" looks for instances of tabs
		
		set ParrentLevel [regexp -all {\s\s\s\s} $line]
		set parrent [lindex $line 0]
		set parrentNode $nodeNum
		incr nodeNum
		
		append graph "$parrentNode \[label=\"$parrent\"\]; "
	} elseif { ([llength $line] == 1) && ([regexp {\s\s\s\s} $line] >= 1) } { ;# We are now looking at a sub parrent
		set subParrentLevel [regexp -all {\s\s\s\s} $line]
		set subParrent [lindex $line 0] ;# line indexing makes it so that the space characters are not also passed
		
		if { $subParrentLevel > $oldSubParrentLevel } {
			set subParrentNode $nodeNum
			incr nodeNum
			
			append graph "$subParrentNode \[label=\"$subParrent\"\]; "
			append graph "$oldsubParrentNode -> $subParrentNode; "

			set oldSubParrentLevel $subParrentLevel
			set oldsubParrentNode $subParrentNode
		} else {
			set subParrentNode $nodeNum
			incr nodeNum

			append graph "$subParrentNode \[label=\"$subParrent\"\]; "
			append graph "$parrentNode -> $subParrentNode; "

			set oldSubParrentLevel $subParrentLevel
			set oldsubParrentNode $subParrentNode
		}

	} elseif { [llength $line] != 1} { ;# We are looking at a child
		set child [lindex $line 0]
		set childLevel [regexp -all {\s\s\s\s} $line]

		set childNode $nodeNum
		incr nodeNum
		append graph "$childNode \[label=\"$child\"\]; "

		if { $dispVal=="val" } {
			set childVal [lindex $line 1]
			set childValNode $nodeNum
			incr nodeNum
			append graph "$childValNode \[label=\"$childVal\"\]; "

			if { $childLevel>$subParrentLevel } {
				append graph "$subParrentNode -> $childNode -> $childValNode; "
			} else {
				append graph "$parrentNode -> $childNode -> $childValNode; "
			}
		} else {
			if { $childLevel>$subParrentLevel } {
			append graph "$subParrentNode -> $childNode; "
			} else {
			append graph "$parrentNode -> $childNode; "
			}
		}
	}
}
append graph "}"
close $fp
exec echo $graph | dot -Tsvg > pdbGraph$pdbParrent.svg
exec rm tempFile.txt ;# removes the test file after use

return 0
}

# example call
#pdbGraph all val
#pdbGraph Silicon val