# pdb constants go here
	# Assume every metal atom donates its electron
	# 8.75 g/cm^3 / (92.90638 g/ mol) * 6.022e23 mol^-1
	set Atoms [expr 8.75 /92.90638 * 6.022e23]
	set T 300.0
	set k 1.38066e-23
	set q 1.619e-19
	set enb [expr 10.0 * 8.85418e-14]
	set eox [expr 22.34 * 8.85418e-14]
	set Conductivity [ expr 7.66e6 / 100. ]

#pdbSetDouble FundamentalConstants eox [expr 22.34 * 8.85418e-14]
	pdbSetDouble FundamentalConstants k 1.38066e-23									;# Boltzman constant [J/K]
	pdbSetDouble FundamentalConstants hbar 6.582119569e-16							;# Planck’s Constant [?]
	pdbSetDouble FundamentalConstants q 1.619e-19									;# Charge of electron [C]
	pdbSetDouble FundamentalConstants eps 8.85418e-14								;# Permittivity of free space
	pdbSetDouble FundamentalConstants small 1.0e-10									;# not sure if this one should be here
#DynamicConstants
	pdbSetDouble DynamicConstants m [expr 1.08*9.1093837015e-31]					;# Effective mass of an electron
	pdbSetDouble DynamicConstants gap 1.42e-3										;# ?? Gap width in nm? [um] this must be confirmed later
	pdbSetDouble DynamicConstants T 300.0											;# Temperature [K]
	pdbSetDouble DynamicConstants Vt [expr "$k*$T/$q"]								;# Terminal voltage [V]
#Niob 
	pdbSetDouble Niobium ernb 10													;# Relative permeability of niobium
	pdbSetDouble Niobium enb [expr 10.0 * 8.85418e-14]								;# Permittivity of oxide
	pdbSetDouble Niobium enbq [expr $enb / $q]										;# Permittivity of niobium per q, divide by q for Poisson's equation
	pdbSetDouble Niobium Atoms [expr 8.75 /92.90638 * 6.022e23]						;#!!! Atomic density of niobium [1/cm^3]
	pdbSetDouble Niobium Conductivity [ expr 7.66e6 / 100. ]						;# conductivity of niobium [1/(cm ohm)] Where do we get this number from?
	pdbSetDouble Niobium mob [ expr $Conductivity/($Atoms*$q)]						;# electron mobility of niobium [?]
#Oxide
	pdbSetDouble Oxide erox 22.34												;# Relative permeability of oxide
	pdbSetDouble Oxide eox [expr 22.34 * 8.85418e-14]								;# Permittivity of oxide
	pdbSetDouble Oxide eoxq [expr $eox / $q]											;# Permittivity of oxide per q, divide by q for Poisson's equation


#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! this one use niobium parameters rather than the oxide
proc PoissonOxide { DevPsi } {
	set eps [pdbGetDouble FundamentalConstants eps]
	set q 	[pdbGetDouble FundamentalConstants q]
	set Atoms [pdbGetDouble Niobium Atoms];#!!!!!!!! all of these rely on nioium rather than the oxide parameters, should be updated in the future
	set ernb  [pdbGetDouble Niobium ernb];#!!!!!!!!!

	set eoxq [expr $eps*$ernb/$q];#!!!!!!!!!!!!!!!!!!!!!!! Permittivity of oxide per q, divide by q for Poisson's equation. Consider renaming
	#set eox [pdbGetDouble Niobium enb]

	return "$eoxq * grad($DevPsi)"
}
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

proc Poisson { DevPsi Elec } {
	set eps [pdbGetDouble FundamentalConstants eps]
	set q 	[pdbGetDouble FundamentalConstants q]
	set Atoms [pdbGetDouble Niobium Atoms]
	set ernb  [pdbGetDouble Niobium ernb]
	
	set enbq [expr $eps*$ernb/$q] ;# Permittivity of niobium per q, divide by q for Poisson's equation. Consider renaming!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	#set enb [pdbGetDouble Niobium enb]

	sel z= $Atoms name= Doping
	return "$enbq * grad($DevPsi) + Doping - $Elec"
}
# example
#puts [Poisson DevPsi Elec]
# pdbSetString Niob1 DevPsi Equation [Poisson DevPsi Elec]

proc ContinuityTunnel { F Conductivity } {
	set Atoms [pdbGetDouble Niobium Atoms]
	set q [pdbGetDouble FundamentalConstants q]
	return "$Conductivity/$q * grad($F)"
}

proc ContinuityTunnelElec { F Elec Conductivity } {
	set Atoms [pdbGetDouble Niobium Atoms]
	set q [pdbGetDouble FundamentalConstants q]
	set mob $Conductivity/($Atoms*$q) 

	return "ddt($Elec) + $mob * $Elec * grad($F)"
}

proc Continuity { F Elec } {
	set mob [pdbGetDouble Niobium mob]

	return "ddt($Elec) + $mob * $Elec * grad($F)"
}
# example
# puts [Continuity F Elec]
# pdbSetString Niob1 F    Equation [Continuity F Elec]

proc CarrierRelationship { DevPsi F } {
	set ni [pdbGetDouble Niobium Atoms]
	set Vt [pdbGetDouble DynamicConstants Vt]

	return "$ni*(($DevPsi-$F)/$Vt + 1)"
}


proc F12CarrierRelationship { DevPsi F } {
	set ni [pdbGetDouble Niobium Atoms]
	set Vt [pdbGetDouble DynamicConstants Vt]

	return "$ni*F12(($DevPsi-$F)/$Vt)"
}


# Returns a Poisson interface equation
# RegionA is the region of the equation
# RegionB is the region next to the equation
# thickness is in cm
proc PoissonInterface  { DevPsi RegionA RegionB thickness } {
	set eps  [pdbGetDouble FundamentalConstants eps]
	set q 	 [pdbGetDouble FundamentalConstants q]
	set erox [pdbGetDouble Oxide erox]
	
	set eoxq [expr $eps*$erox/$q]
	return "($DevPsi\($RegionB)-$DevPsi\($RegionA))/($thickness/$eoxq)" 
}

# example
#The average barrier thickness is 0.9 nm, m to cm 
# puts [PoissonInterface DevPsi Niob2 Niob1 [ expr 0.9e-9 * 100. ]]
# puts [PoissonInterface DevPsi Niob1 Niob2 [ expr 0.9e-9 * 100. ]]
# pdbSet Niob1_Niob2 DevPsi Niob2 Equation [PoissonInterface DevPsi Niob2 Niob1 [ expr 0.9e-9 * 100. ]]
# pdbSet Niob1_Niob2 DevPsi Niob1 Equation [PoissonInterface DevPsi Niob1 Niob2 [ expr 0.9e-9 * 100. ]]

proc Contact {Contact F DevPsi Elec} {
	#set Vt [dict get $c Vt]
	#set ni [dict get $c ni]
    pdbSet $Contact $F      Flux 1
    pdbSet $Contact $DevPsi Flux 1
    pdbSet $Contact $F      Fixed 1
    pdbSet $Contact $DevPsi Fixed 1
    pdbSet $Contact $F Flux.Scale 1.619e-19
    pdbSet $Contact $F Equation "$F-$Contact"
    pdbSet $Contact $DevPsi Equation "Doping - $Elec - CP"
    #pdbSetString $Contact Equation "1.619e-19 * (Flux_Hole - Flux_Elec)"
}

proc AvgBarrierThickness { G G0 t0 } {
	set lambda -0.069
	# Convert Jc to G
	return [ expr log10($G/$G0)*$lambda+$t0 ]
}

# Jc units are mA/um^2
proc ConstConductance { Jc } {
	set gap [pdbGetDouble DynamicConstants gap]
	# Convert Jc to G
	return [ expr $Jc*1e-3/2./$gap ]
}
puts [ConstConductance 0.1]
# Returns a Continuity interface equation
# RegionA is the region of the equation
# RegionB is the region next to the equation
proc SimpTunnelInterface  { F RegionA RegionB Conductance Y Z } {
	set q [pdbGetDouble FundamentalConstants q]
	set gap [pdbGetDouble DynamicConstants gap]
	set CurRatio 0.1
	set Diff ($F\($RegionA)-$F\($RegionB))
	return "$F\($RegionA)-$Conductance/(1e-4*1e-4*$Z*$Y)/$q*((abs($Diff) > [expr 2*$gap]) ? $Diff : [expr $CurRatio*1e-3])"
} 
# puts [SimpTunnelInterface F Niob2 Niob1 [ConstConductance 100e-6] 1 1]
# pdbSet Niob1_Niob2 F Niob2 Equation [SimpTunnelInterface F Niob2 Niob1 [ConstConductance 100e-6] 1 1]
# pdbSet Niob1_Niob2 F Niob1 Equation [SimpTunnelInterface F Niob1 Niob2 [ConstConductance 100e-6] 1 1]

proc SmallVoltExpTunnel { BarrierHeight } {
	set c [constants]
	set h [dict get $c h] ;# in eV
	set q [dict get $c q]
	set m0 [dict get $c m0]

	set C [expr 3.0*sqrt(2.0*$m0*$BarrierHeight*$q) / 2.0 * (1.0 / $h) ** 2.0 / 100.0 ]
	set lambda [ expr $q*$h / 4.0 / acos(-1.0) / sqrt(2*$m0*$BarrierHeight*$q) * 100 ]
	return "($C*exp(- AlOx_Thickness/$lambda))"
}

