namespace eval AlN {
#set low field electron mobility via analytical expression from Farahmand for low field mobility
#paramters for AlN for low field mobility from Farahmand
proc ElecMobility args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=Temp"
  lappend usage "The temperature (T) defaults to an alagator expression of Temp"
  set returns "AlN electron mobility alagator string"
  set description {Computes the temperature dependent electron mobility for AlN}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set T [__GetValue $args T ]
  if {$T == ""} { set T Temp }
  set T ($T)

  set ANmumin 297.8
  set ANmumax 683.8
  set ANlowalpha 1.16
  set ANbeta1 -1.82
  set ANbeta2 -3.43
  set ANbeta3 3.78
  set ANbeta4 0.86
  set ANNref 1e17
  #build equation for low field mobility
  set ANseg1 "($ANmumin)*(exp(log(Temp/300)*($ANbeta1)))"
  set ANseg2 "($ANmumax-$ANmumin)*(exp(log(Temp/300)*($ANbeta2)))"
  set ANseg3 "$ANNref*(exp(log(Temp/300)*($ANbeta3)))"
  set ANseg4 "abs((Doping+1)/$ANseg3)"
  set ANseg5 "$ANlowalpha*(exp(log(Temp/300)*($ANbeta4)))"
  set ANseg6 "exp(log($ANseg4)*($ANseg5))"
  set ANseg7 "1+$ANseg6"
  set ANlowfldmob ($ANseg1+(($ANseg2)/($ANseg7)))
  set ANhighalpha 8.7253
  set ANn1 17.3681
  set ANn2 0.8554
  set ANEcmob 447033.9
  set ANvsat 2.167e7
  set ANEfield "(sqrt(dot(DevPsi,DevPsi)+1.0e2))"
  set ANEfield_EcRatio "($ANEfield)/($ANEcmob)"
  set ANEn1 "exp(log($ANEfield_EcRatio)*($ANn1-1))"
  set ANEn2 "exp(log($ANEfield_EcRatio)*($ANn2))"
  set ANnum "($ANlowfldmob+($ANvsat*($ANEn1)/($ANEcmob)))"
  set ANden "(1.0+($ANhighalpha*($ANEn2))+($ANEn1*$ANEfield_EcRatio))"
  return ($ANlowfldmob)
}
#set hole mobility as constant
proc HoleMobility args {
  set name [lindex [info level 0] 0]
  lappend usage "$name"
  set returns "AlN hole mobility alagator string"
  set description {Gives a rough AlN hole mobility}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  return 0.20
}
}

