namespace eval GaN {
#set low field electron mobility via analytical expression from Farahmand for low field mobility
#paramters for GaN for low field mobility from Farahmand
proc ElecMobility args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=Temp"
  lappend usage "The temperature (T) defaults to an alagator expression of Temp"
  set returns "GaN electron mobility alagator string"
  set description {Computes the temperature dependent electron mobility for GaN}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set T [__GetValue $args T ]
  if {$T == ""} { set T Temp }
  set T ($T)


  set Gmumin 295.0
  set Gmumax 1460.7
  set Glowalpha 0.66
  set Gbeta1 -1.02
  set Gbeta2 -3.84
  set Gbeta3 3.02
  set Gbeta4 0.81
  set GNref 1e17
  
  #build equation for low field mobility
  set Gseg1 "$Gmumin*(exp(log($T/300)*($Gbeta1)))"
  set Gseg2 "($Gmumax-$Gmumin)*(1*exp(log($T/300)*($Gbeta2)))"
  set Gseg3 "$GNref*(1*exp(log($T/300)*($Gbeta3)))"
  set Gseg4 "abs((abs(Doping)+Donor+Acceptor+1)/$Gseg3)";     #modified to include ionized donor or acceptor traps
  set Gseg5 "$Glowalpha*(1*exp(log($T/300)*($Gbeta4)))"
  set Gseg6 "1*exp(log($Gseg4)*($Gseg5))"
  set Gseg7 "1+$Gseg6"
  set Gseg8 "($Gseg2)/($Gseg7)"
  return "(($Gseg1)+($Gseg8))"
}
#set hole mobility as constant
proc HoleMobility args {
  set name [lindex [info level 0] 0]
  lappend usage "$name"
  set returns "GaN hole mobility alagator string"
  set description {Gives a rough GaN hole mobility}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  return 100
}
}

