
proc Gate args {
  set name [lindex [info level 0] 0]
  lappend usage "$name name=?contact_name? wfn=0"
  lappend usage "Contact name (name) is set in the pdb with the equations"
  lappend usage "The work function (wfn) is assumed to be zero unless otherwise specified"
  set returns "nothing"
  set description {Convienence wrapper for gate contacts.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set contact_name [__GetValue $args name "$name Error: No contact name (name) specified"]
  set wfn [__GetValue $args wfn]
  if { $wfn == "" } { 
    puts "Warning using default work function for contact $contact_name"
    set wfn 4.6 
  }
  if { ![string is double -strict $wfn] } {
    if { [string length $wfn] > 1 } {
      set sym [string toupper [string index $wfn 0]][string index $wfn 1]
    } else {
      set sym [string toupper [string index $wfn 0]]
    }
    if { $sym == "Tu" } { set sym W }
    set wfn [MetalWorkFunction sym=$sym]
  }

  pdbSet $contact_name DevPsi Fixed 1
  pdbSet $contact_name DevPsi Flux 1
  set q 1.602176634e-19
  pdbSet $contact_name DevPsi Flux.Scale $q
  pdbSet $contact_name DevPsi Equation "($wfn+DevPsi) - $contact_name"; # Efm+WFN=El=-DevPsi, Efm=-$Contact
}

proc GateCap args {
  set name [lindex [info level 0] 0]
  lappend usage "$name name=?contact_name? freq=1e5 Vsmall=0.1"
  lappend usage "Contact name (name) is set in the pdb with the equations"
  lappend usage "The AC small signal frequency (freq) is assumed to be 1e5 Hz"
  lappend usage "The small signal voltage (Vsmall) is assumed to be 1 Volt"
  set returns "Runs a small signal device simulation and returns the correct scaled gate contact capacitance"
  set description {Convienence wrapper for gate contacts.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set contact_name [__GetValue $args name "$name Error: No contact name (name) specified"]
  set freq [__GetValue $args freq]
  if { $freq == "" } { set freq 1e5 }
  set Vsmall [__GetValue $args Vsmall]
  if { $Vsmall == "" } { set Vsmall 1 }
  contact name=$contact_name voltage supply= $Vsmall acreal
  contact name=$contact_name voltage supply=0.0 acimag
  device freq= $freq
  return [expr (abs([contact name=$contact_name sol=DevPsi flux acreal])/($Vsmall))]
}

