
proc Poisson args {
  set name [lindex [info level 0] 0]
  lappend usage "$name er= potential=DevPsi Doping=Doping Elec=Elec Hole=Hole"
  lappend usage {The relative permitivity (er) is unitless.}
  lappend usage {The doping concentration (Doping) is optional and defaults to Doping.}
  lappend usage {Hole concentration (Hole) is optional and defaults to Hole.}
  lappend usage {Electron concentration (Elec) is optional and defaults to Elec.}
  set returns {Poisson's equation tunneling alagator string}
  set description {Simple Poisson's equation $\epsilon_r\nabla\Phi + N - n + p = 0$}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set er [__GetValue $args er "$name Error: No relative permitivity (er) is given"]
  set potential [__GetValue $args potential ]
  if {$potential == ""} { set potential DevPsi }
  set Doping [__GetValue $args Doping ]
  if {$Doping == ""} { set Doping Doping }
  set Hole [__GetValue $args Hole ]
  if {$Hole == ""} { set Hole Hole }
  set Elec [__GetValue $args Elec ]
  if {$Elec == ""} { set Elec Elec }

  set q 1.602e-19
  set eps [expr $er * 8.85418e-14 / $q]

  return "$eps * grad($potential) + $Doping - $Elec + $Hole"
}

