
proc CurrentMag args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Qf=? Conc=? Mob=?"
  lappend usage "The quasi fermi level (Qf) must be specified"
  lappend usage "The mobility must be specified (Mob) must be specified"
  lappend usage "The carrier concentration must be specified (Conc) must be specified"
  set returns "Current magnitude equation"
  set description {}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set q 1.602176634e-19

  set Qf [__GetValue $args Qf "$name error: the quasi-Fermi level (Qf) must be specified"]
  set Mob [__GetValue $args Mob "$name error: the mobility (Mob) must be specified"]
  set Conc [__GetValue $args Conc "$name error: the carrier concentration (Conc) must be specified"]

  return ($q*(MagGrad($Qf))*($Mob)*($Conc)+1e-10)

}

proc ElectricFieldMag args {
  set name [lindex [info level 0] 0]
  lappend usage "$name DevPsi=DevPsi"
  lappend usage "The electrostatic potential (DevPsi) is assumed to be DevPsi"
  set returns "Electric field magnitude equation"
  set description {}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set DevPsi [__GetValue $args DevPsi ]
  if {$DevPsi == ""} { set DevPsi DevPsi }

  return (MagGrad($DevPsi)+1)

}
