namespace eval Germanium {
# Masetti Mobility Model from Sentaurus, P at end of parameter is Phospherous, B at end of parameter is Boron
# See paper titled Electrical TCAD Simulations of a Germanium pMOSFET Technology
proc Masetti args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T= Accept=Accept Donor=Donor"
  lappend usage {Temperature (T) is in Kelvin.}
  lappend usage {Donor concentration (Donor) is assumed to be the alagator field Donor and dopant type As.}
  lappend usage {Accept concentration (Accept) is assumed to be the alagator field Accept.}
  set returns {A list of two alagator expressions, the first item is the electron mobility and the second is the hole mobility }
  set description {Masetti majority carrier mobility model. Great to use for many applications where the majority carrier is the only concern.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set expand [__Contains $args expand]
  set Accept [__GetValue $args Accept ]
  if {$Accept == ""} { set Accept Accept }
  set Donor [__GetValue $args Donor ]
  if {$Donor == ""} { set Donor Donor }
  
  set umin1E  60
  set umin1H  60
  set umin2E  0.0 
  set umin2H  0.0
  set u1E     20.0
  set u1H     40.0
  set PcE     1e17
  set PcH     9.23e16
  set CrE     8e16
  set CrH     2e17
  set CsE     3.43e20
  set CsH     1e20
  set alphaE  0.55
  set alphaH  0.55
  set betaE   2.00
  set betaH   2.00
  # from ru
  set uconstE "4.9e7*$T^(-1.66)"
  set uconstH "1.05e9*$T^(-2.33)"
  set tmp {}
  # Emob
  lappend tmp "$umin1E*exp(-$PcE/(abs($Donor)+abs($Accept))) + ($uconstE-$umin2E)/(1+((abs($Donor)+abs($Accept))/$CrE)^$alphaE) - $u1E/(1+($CsE/(abs($Donor)+abs($Accept)))^$betaE)"
  # Hmob
  lappend tmp "$umin1H*exp(-$PcH/(abs($Donor)+abs($Accept))) + ($uconstH-$umin2H)/(1+((abs($Donor)+abs($Accept))/$CrH)^$alphaH) - $u1H/(1+($CsH/(abs($Donor)+abs($Accept)))^$betaH)"
  return $tmp
}
}

