namespace eval Silicon {

proc ActiveBoron args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T= Boron=Boron potential=DevPsi"
  lappend usage {Temperature (T) is in Kelvin.}
  lappend usage {The Boron concentration (Boron) is assumed to be the alagator field Boron.}
  lappend usage {The activation potential energy (potential) is optional and defaults to DevPsi.}
  set returns {alagator string}
  set description {Computes freeze out of carriers at low temperatures.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set Boron [__GetValue $args Boron ]
  if {$Boron == ""} { set Boron Boron }
  set potential [__GetValue $args potential ]
  if {$potential == ""} { set potential DevPsi }

  set delta 0.044
  set Ea (0.0438-3.037e-8*($Boron^(1./3.)))
  set k 1.38066e-23
  set q 1.602e-19
  set Vt ([expr $k/$q]*$T)
  
  return "( $Boron / (1 + (4 + 2*exp( -$delta/$Vt )) * exp(($Ea - $potential)/$Vt) ) )"
}

}
