namespace eval Silicon {

proc ActivePhosphorus args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T= Phosphorus=Phosphorus potential=DevPsi"
  lappend usage {Temperature (T) is in Kelvin.}
  lappend usage {The Phosphorus concentration (Phosphorus) is assumed to be the alagator field Phosphorus.}
  lappend usage {The activation potential energy (potential) is optional and defaults to DevPsi.}
  set returns {alagator string}
  set description {Computes freeze out of carriers at low temperatures.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set Phosphorus [__GetValue $args Phosphorus ]
  if {$Phosphorus == ""} { set Phosphorus Phosphorus }
  set potential [__GetValue $args potential ]
  if {$potential == ""} { set potential DevPsi }

  set delta 0.044
  set Ed (0.045-3.10e-8*($Phosphorus^(1./3.)))
  set k 1.38066e-23
  set q 1.602e-19
  set Vt ($k*$T/$q)
  return "( $Phosphorus / (1 + 2  * exp(($Ed - $potential)/$Vt) ) )"
}


}
