
# Goal is to use this function
proc UniformStress args {
  set name [lindex [info level 0] 0]
  lappend usage "$name stress= {0 0 0 0 0 0} miller=(001)<100>"
  lappend usage "A kp must be provided, the default is used if one is not provided."
  lappend usage "Stress vector is in Voigt notation string list format and assumed to be a vector of six zeros."
  lappend usage "Miller indicies (miller) are in (hkl)<uvw> format and defaults to 001 wafers with 100 channels."
  set description {Sets the silicon stress equations}
  set returns "nothing"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set stress [__GetValue $args stress ]
  if {$stress == ""} { set stress {0 0 0 0 0 0} }
  set miller [__GetValue $args miller]
  if {$miller == ""} { set miller (001)<100> }

  sel z= [lindex $stress 0] name= StressXX
  sel z= [lindex $stress 1] name= StressYY
  sel z= [lindex $stress 2] name= StressZZ
  sel z= [lindex $stress 3] name= StressYZ
  sel z= [lindex $stress 4] name= StressXZ
  sel z= [lindex $stress 5] name= StressXY

  set S [ __Compliance $miller Silicon ]
  set strain [__MultAx $S $stress ]
  sel z= [lindex $strain 0] name= StrainXX
  sel z= [lindex $strain 1] name= StrainYY
  sel z= [lindex $strain 2] name= StrainZZ
  sel z= [lindex $strain 3] name= StrainYZ
  sel z= [lindex $strain 4] name= StrainXZ
  sel z= [lindex $strain 5] name= StrainXY
}

set ::__Hall 0

namespace eval Silicon {

proc UniformStress args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=? kp=default stress= {0 0 0 0 0 0} miller=(001)<100>"
  lappend usage "A kp must be provided, the default is used if one is not provided."
  lappend usage "Temperature (T) is required in Kelvin."
  lappend usage "Stress vector is in Voigt notation string list format and assumed to be a vector of six zeros."
  lappend usage "Miller indicies (miller) are in (hkl)<uvw> format and defaults to 001 wafers with 100 channels."
  set description {Sets the silicon stress equations}
  set returns "nothing"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set stress [__GetValue $args stress ]
  if {$stress == ""} { set stress {0 0 0 0 0 0} }
  set miller [__GetValue $args miller]
  if {$miller == ""} { set miller (001)<100> }
  set T [__GetValue $args T "$name Error: No temperature (T) provided"]
  set kp [__GetValue $args kp]

  sel z= [lindex $stress 0] name= StressXX
  sel z= [lindex $stress 1] name= StressYY
  sel z= [lindex $stress 2] name= StressZZ
  sel z= [lindex $stress 3] name= StressYZ
  sel z= [lindex $stress 4] name= StressXZ
  sel z= [lindex $stress 5] name= StressXY


  set S [ __Compliance $miller Silicon ]
  set strain [__MultAx $S $stress ]
  sel z= [lindex $strain 0] name= StrainXX
  sel z= [lindex $strain 1] name= StrainYY
  sel z= [lindex $strain 2] name= StrainZZ
  sel z= [lindex $strain 3] name= StrainYZ
  sel z= [lindex $strain 4] name= StrainXZ
  sel z= [lindex $strain 5] name= StrainXY

  # if { $kp == "" } {
  #   set zero 1
  #   foreach s $stress {
  #     if { abs($s) > 1 } { set zero 0 } 
  #   }
  #   if { $zero } {
  #     set kp {{-0 0.128459 0.128459 0.128459 1 1 1 0.223853} {-0 0.23436 0.23436 0.23436 1 1 1 0.578158} {-0.044 0.145828 0.145828 0.145828 1 1 1 0.197989}}
  #   } else {
  #     set kp [GetKPTemp $stress $miller [expr $T*8.618352059925094e-5] ]
  #   }
  # }
  # set kp [GetKPTemp "0 0 0 0 0 0" $miller [expr $T*8.618352059925094e-5] ]
  # solution Silicon add const solve name= LHMx0 val= [lindex $kp 0 1]
  # solution Silicon add const solve name= LHMy0 val= [lindex $kp 0 2]
  # solution Silicon add const solve name= LHMz0 val= [lindex $kp 0 3]
  # solution Silicon add const solve name= HHMx0 val= [lindex $kp 1 1]
  # solution Silicon add const solve name= HHMy0 val= [lindex $kp 1 2]
  # solution Silicon add const solve name= HHMz0 val= [lindex $kp 1 3]
  # solution Silicon add const solve name= SOMx0 val= [lindex $kp 2 1]
  # solution Silicon add const solve name= SOMy0 val= [lindex $kp 2 2]
  # solution Silicon add const solve name= SOMz0 val= [lindex $kp 2 3]
  set kp [GetKPTemp $stress $miller [expr $T*8.618352059925094e-5] ]
  solution Silicon add const solve name= LHMx val= [lindex $kp 0 1]
  solution Silicon add const solve name= LHMy val= [lindex $kp 0 2]
  solution Silicon add const solve name= LHMz val= [lindex $kp 0 3]
  solution Silicon add const solve name= HHMx val= [lindex $kp 1 1]
  solution Silicon add const solve name= HHMy val= [lindex $kp 1 2]
  solution Silicon add const solve name= HHMz val= [lindex $kp 1 3]
  solution Silicon add const solve name= SOMx val= [lindex $kp 2 1]
  solution Silicon add const solve name= SOMy val= [lindex $kp 2 2]
  solution Silicon add const solve name= SOMz val= [lindex $kp 2 3]
  set EvalList [ValanceBandEdgeShiftStress $kp ]
  solution Silicon add const solve val= [lindex $EvalList 0] name=EvalShiftLH
  solution Silicon add const solve val= [lindex $EvalList 1] name=EvalShiftHH
  solution Silicon add const solve val= [lindex $EvalList 2] name=EvalShiftSO
  set DOSList [ValanceBandDOSFactorStress $kp ]
  solution Silicon add const solve val= [lindex $DOSList 0] name= RelNvLH
  solution Silicon add const solve val= [lindex $DOSList 1] name= RelNvHH
  solution Silicon add const solve val= [lindex $DOSList 2] name= RelNvSO

  # Update band gap
  sel name=Eg z= "Eg0+(SiEcondShift100+SiEcondShift010+SiEcondShift001)/3.+[Max $EvalList]"
}

proc __SiliconMobility { T } {
  if { [set ::__MOS] } {
    # get the scaling distance for the surface terms
    sel z=exp(-Dcm/1.0e-6) name=Dcr
    # Build mos mobility
    set Emob ""
    set Hmob ""
    lappend Emob [lindex [Silicon::Masetti T=$T ] 0]
    lappend Hmob [lindex [Silicon::Masetti T=$T ] 1]
    lappend Emob [lindex [Silicon::SurfacePhonon T=$T] 0]
    lappend Hmob [lindex [Silicon::SurfacePhonon T=$T] 1]
    lappend Emob [lindex [Silicon::SurfaceRoughness] 0]
    lappend Hmob [lindex [Silicon::SurfaceRoughness] 1]
    set Emob [Mathiessen Mobs=$Emob]
    set Hmob [Mathiessen Mobs=$Hmob]
  } else {
    set Emob [Silicon::KlaassenNP T=$T]
    set Hmob [Silicon::KlaassenPB T=$T]
  }
  # Material Mobility
  solution Silicon name=Emob add const solve val= [Silicon::CanaliElec Emob=$Emob T=$T F=Emfn]
  solution Silicon name=Hmob add const solve val= [Silicon::CanaliHole Hmob=$Hmob T=$T F=Emfp]
}

proc Eqns args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=? miller=(001)<100>"
  lappend usage "Temperature (T) is required in Kelvin."
  lappend usage "Miller indicies (miller) are in (hkl)<uvw> format and defaults to 001 wafers with 100 channels."
  set description {Sets the silicon stress equations}
  set returns "nothing"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set stress [__GetValue $args stress ]
  if {$stress == ""} { set stress {0 0 0 0 0 0} }
  set miller [__GetValue $args miller]
  if {$miller == ""} { set miller (001)<100> }
  set T [__GetValue $args T "$name Error: No temperature (T) provided"]
  # Silicon::SetStrainParamsList "1.476 -2.321 -3.690 4.285 0.3390 1.460 10.58 -0.2727 2.690 86.80"
  # Silicon::SetStrainParamsList "2.1 -1.0001 -4.75 6.405 0.21875 1.39 10.3376 1.34554 2.67807 86.8"
  # new -> Silicon::SetStrainParamsList "2.124 -0.1707 -4.75 6.405 0.21875 1.39 6.6638 0.5311 5.5989 86.8"

  sel z= 0 name= StrainXX
  sel z= 0 name= StrainYY
  sel z= 0 name= StrainZZ
  sel z= 0 name= StrainYZ
  sel z= 0 name= StrainXZ
  sel z= 0 name= StrainXY

  __SiliconMobility $T
  # intrensic carrier concentrations and band edge DOS
  set Nc [Silicon::Nc T=$T] ;# 1/cm^3
  set Nv [Silicon::Nv T=$T] ;# 1/cm^3
  solution Silicon add name=Eg0 const solve val= ([Silicon::BandGap T=$T]-[Silicon::OldSlotboom])
  # Compute the band gap after strain
  # This is updated in strain procs
  # This may need to be upgraded to a solution in the future
  sel name= Eg z= ([Silicon::BandGap T=$T]-[Silicon::OldSlotboom])
  solution Silicon add name= ni const solve val= [IntrinsicConc Nv=$Nv Nc=$Nc Eg=Eg T=$T]

  # Electron Equations
  set EMobilities {{RelEmob100_100 RelEmob100_010 RelEmob100_001} {RelEmob010_100 RelEmob010_010 RelEmob010_001} {RelEmob001_100 RelEmob001_010 RelEmob001_001}}
  set Elecs "(Elec100+1000) (Elec010+1000) (Elec001+1000)"
  set Xrel [__getXmobsStrain $Elecs Silicon ]
  set RawTensor ""
  set DPTensor ""
  if { [set ::__Hall] } {
    # Hall mobility should be simple... 
    # the effect only works in low doping regions anyway
    # The Hall factor r corrects for the normalization need a proof for this
    # set r [expr (3./((2*0.28*1.36+1.36*1.36)))**2]
    set r ([expr 3./((2*0.28*1.36+1.36*1.36))]*[ElecHallFactorTemp T=$T])
    sel z= $r*Emob name= HallMob
    set RawTensor [Silicon::Get3DElecDPTensorPiezoHall Elecs= $Elecs RawMobs=$Xrel Mobs=$EMobilities miller=$miller HallMob=HallMob B= "Bx By Bz" ]
    set DPTensor [3DTensorListToFlooxsTensor $RawTensor ]
  } else {
    set RawTensor [Silicon::Get3DElecDPTensor Elecs= $Elecs Mobs=$EMobilities miller=$miller]
    set DPTensor [3DTensorListToFlooxsTensor $RawTensor ]
  }
  pdbSet Silicon Qfn Equation [QFElecContinuity Elec= $DPTensor R=R G=G]
  # pdbSet Silicon Qfn Equation "ddt(Elec) + (Emob) * Elec * Velocity(GradX(Qfn), GradY(Qfn), GradZ(Qfn))  + (R) - (G)"
  # Define band edge shifts as field
  set EcondList [XBandEdgeShiftStrain miller=$miller ]
  solution Silicon add const solve val= [lindex $EcondList 0] name= SiEcondShift100
  solution Silicon add const solve val= [lindex $EcondList 1] name= SiEcondShift010
  solution Silicon add const solve val= [lindex $EcondList 2] name= SiEcondShift001
  # Define conduction band edge as constant solution
  solution Silicon add name=Econd const solve val= [Econd]
  # Define conduction density gradient inverse effective mass fields
  sel z= [DGInvMass mx=0.97 my=0.19 mz=0.19] name=SiDGInvMassElec100
  sel z= [DGInvMass mx=0.19 my=0.97 mz=0.19] name=SiDGInvMassElec010
  sel z= [DGInvMass mx=0.19 my=0.19 mz=0.97] name=SiDGInvMassElec001
  # Compute the 3 valley band populations
  solution Silicon add name=Elec100 const solve val= [CondFermiDirac Ec=Econd+SiEcondShift100-(Lambda*SiDGInvMassElec100) Nc=$Nc/3. T=$T] 
  solution Silicon add name=Elec010 const solve val= [CondFermiDirac Ec=Econd+SiEcondShift010-(Lambda*SiDGInvMassElec010) Nc=$Nc/3. T=$T] 
  solution Silicon add name=Elec001 const solve val= [CondFermiDirac Ec=Econd+SiEcondShift001-(Lambda*SiDGInvMassElec001) Nc=$Nc/3. T=$T] 
  solution Silicon add name=Elec0   const solve val= [CondFermiDirac Ec=Econd Nc=$Nc T=$T]
  solution Silicon add name=Elec const solve val= "Elec100+Elec010+Elec001"
  # Compute the relaxation time shift
  solution Silicon add name=DTau100 const solve val= [DTau T=$T Elec=(Elec100*3.)] 
  solution Silicon add name=DTau010 const solve val= [DTau T=$T Elec=(Elec010*3.)] 
  solution Silicon add name=DTau001 const solve val= [DTau T=$T Elec=(Elec001*3.)] 
  # Messy x valley mobilities
  lassign $Xrel RelEmob100 RelEmob010 RelEmob001
  lassign $RelEmob100 RelEmob100_100 RelEmob100_010 RelEmob100_001
  solution Silicon add name=RelEmob100_100 const solve val= $RelEmob100_100 
  solution Silicon add name=RelEmob100_010 const solve val= $RelEmob100_010 
  solution Silicon add name=RelEmob100_001 const solve val= $RelEmob100_001 
  lassign $RelEmob010 RelEmob010_100 RelEmob010_010 RelEmob010_001
  solution Silicon add name=RelEmob010_100 const solve val= $RelEmob010_100
  solution Silicon add name=RelEmob010_010 const solve val= $RelEmob010_010
  solution Silicon add name=RelEmob010_001 const solve val= $RelEmob010_001
  lassign $RelEmob001 RelEmob001_100 RelEmob001_010 RelEmob001_001
  solution Silicon add name=RelEmob001_100 const solve val= $RelEmob001_100
  solution Silicon add name=RelEmob001_010 const solve val= $RelEmob001_010
  solution Silicon add name=RelEmob001_001 const solve val= $RelEmob001_001

  # Set Emob and Hmob for Silicon
  solution Silicon add name=Hole const solve val= 1
  solution Silicon add name=Jn const solve val= 1
  solution Silicon add name=Jp const solve val= 1
  # Avalanche
  # solution Silicon add name= Ava const solve val= (([Silicon::vanOberstraetenHole T=$T ])+([Silicon::vanOberstraetenElec T=$T]))
  # Terrible convergence
  # solution Silicon add name= Ava const solve val= (([__CrowellSzeHole Emfp $T Jp ])+([__CrowellSzeElec Emfn $T Jn ]))
  # Produces nan
  # solution Silicon add name= Ava const solve val= (([__OkutoHole Emfp $T Jp ])+([__OkutoElec Emfn $T Jn ]))
  # Does nothing
  # solution Silicon add name= Ava const solve val= (([__SelberherrElec Emfp $T Jp ])+([__SelberherrHole Emfn $T Jn ]))
  # Ok but worse convergence
  # solution Silicon add name= Ava const solve val= (([__UniBoHole Emfp $T Jp ])+([__UniBoElec Emfn $T Jn ]))
  # Terrible convergence
  # solution Silicon add name= Ava const solve val= (([__UniBoNewHole Emfp $T Jp ])+([__UniBoNewElec Emfn $T Jn ]))

  solution Silicon add name= Avah const solve val= ([Silicon::vanOberstraetenHole T=$T])
  solution Silicon add name= Avae const solve val= ([Silicon::vanOberstraetenElec T=$T])


  # Generation and Recombination
  solution Silicon add name= SRH const solve val= [SRH ni=ni]
  solution Silicon add name= Auger const solve val= [Auger ni=ni]
  solution Silicon add name= Schenk const solve val= [Schenk Eg=Eg T=$T ni=ni Nc=$Nc Nv=$Nv]
  solution Silicon add name= R const solve val= (SRH+Auger)
  # set Ava "(Emob * sqrt(([lindex $RawTensor 0 0 0]*GradX(Qfn)+1)^2+([lindex $RawTensor 0 1 1]*GradY(Qfn)+1)^2+([lindex $RawTensor 0 2 2]*GradZ(Qfn)+1)^2) * Avae + Hmob * Hole * MagGrad(Qfp) * Avah)"
  # set Ava "(Emob * Elec * MagGrad(Qfn) * Avae + Hmob * Hole * MagGrad(Qfp) * Avah)"
  set Ava "(Emob * sqrt(([lindex $RawTensor 0 0 0]*GradX(Qfn)+1)^2+([lindex $RawTensor 0 1 1]*GradY(Qfn)+1)^2+([lindex $RawTensor 0 2 2]*GradZ(Qfn)+1)^2) * ([Silicon::vanOberstraetenElec T=$T]) + Hmob * Hole * MagGrad(Qfp) * ([Silicon::vanOberstraetenHole T=$T]))"
  solution Silicon add name= G const solve val= ($Ava+Schenk)

  # Define Magnetic fields
  sel z= 0 name= Bx
  sel z= 0 name= By
  sel z= 0 name= Bz
  
  if { 0 } {
    # Compute the hole properties
    solution Silicon add const solve name= EvalShiftLH val= (-0.0015141+2.00147*StrainXX+2.00147*StrainYY+2.00147*StrainZZ+-1.46646*StrainYZ+-1.46646*StrainXZ+-1.46646*StrainXY+-7.29362e-19*Temp*StrainXX+4.44294e-18*Temp*StrainYY+-8.20977e-19*Temp*StrainZZ+-6.82691e-18*Temp*StrainYZ+2.06216e-18*Temp*StrainXZ+-3.72973e-18*Temp*StrainXY+-237.22*StrainXX*StrainXX+247.033*StrainXX*StrainYY+247.033*StrainXX*StrainZZ+280.069*StrainXX*StrainYZ+280.069*StrainXX*StrainXZ+-546.158*StrainXX*StrainXY+-237.22*StrainYY*StrainYY+247.033*StrainYY*StrainZZ+-546.158*StrainYY*StrainYZ+280.069*StrainYY*StrainXZ+280.069*StrainYY*StrainXY+-237.22*StrainZZ*StrainZZ+280.069*StrainZZ*StrainYZ+-546.158*StrainZZ*StrainXZ+280.069*StrainZZ*StrainXY+-483.063*StrainYZ*StrainYZ+965.755*StrainYZ*StrainXZ+965.755*StrainYZ*StrainXY+-483.063*StrainXZ*StrainXZ+965.755*StrainXZ*StrainXY+-483.063*StrainXY*StrainXY)
    solution Silicon add const solve name= LHMx val= (0.192037+2.46912e-19*Temp+-1.72658*StrainXX+1.50696*StrainYY+1.50696*StrainZZ+5.09594*StrainYZ+5.09594*StrainXZ+8.48592*StrainXY+-2.68309e-16*Temp*StrainXX+-7.71469e-17*Temp*StrainYY+4.81839e-17*Temp*StrainZZ+-2.91201e-16*Temp*StrainYZ+1.1915e-16*Temp*StrainXZ+-2.87466e-18*Temp*StrainXY+2404.32*StrainXX*StrainXX+-2477.43*StrainXX*StrainYY+-2477.43*StrainXX*StrainZZ+1276.29*StrainXX*StrainYZ+1276.29*StrainXX*StrainXZ+1936.97*StrainXX*StrainXY+1533.79*StrainYY*StrainYY+-827.026*StrainYY*StrainZZ+-1032.78*StrainYY*StrainYZ+-320.62*StrainYY*StrainXZ+-1010.22*StrainYY*StrainXY+1533.79*StrainZZ*StrainZZ+-320.62*StrainZZ*StrainYZ+-1032.78*StrainZZ*StrainXZ+-1010.22*StrainZZ*StrainXY+4421.22*StrainYZ*StrainYZ+-5370.07*StrainYZ*StrainXZ+983.747*StrainYZ*StrainXY+4421.22*StrainXZ*StrainXZ+983.747*StrainXZ*StrainXY+1520.94*StrainXY*StrainXY)
    solution Silicon add const solve name= LHMy val= (0.192037+8.60624e-19*Temp+1.50696*StrainXX+-1.72658*StrainYY+1.50696*StrainZZ+8.48592*StrainYZ+5.09594*StrainXZ+5.09594*StrainXY+-1.88449e-16*Temp*StrainXX+5.32137e-17*Temp*StrainYY+-1.60761e-16*Temp*StrainZZ+-6.15355e-17*Temp*StrainYZ+-8.31301e-17*Temp*StrainXZ+3.47511e-17*Temp*StrainXY+1533.79*StrainXX*StrainXX+-2477.43*StrainXX*StrainYY+-827.026*StrainXX*StrainZZ+-1010.22*StrainXX*StrainYZ+-320.62*StrainXX*StrainXZ+-1032.78*StrainXX*StrainXY+2404.32*StrainYY*StrainYY+-2477.43*StrainYY*StrainZZ+1936.97*StrainYY*StrainYZ+1276.29*StrainYY*StrainXZ+1276.29*StrainYY*StrainXY+1533.79*StrainZZ*StrainZZ+-1010.22*StrainZZ*StrainYZ+-1032.78*StrainZZ*StrainXZ+-320.62*StrainZZ*StrainXY+1520.94*StrainYZ*StrainYZ+983.747*StrainYZ*StrainXZ+983.747*StrainYZ*StrainXY+4421.22*StrainXZ*StrainXZ+-5370.07*StrainXZ*StrainXY+4421.22*StrainXY*StrainXY)
    solution Silicon add const solve name= LHMz val= (0.192037+6.93604e-19*Temp+1.50696*StrainXX+1.50696*StrainYY+-1.72658*StrainZZ+5.09594*StrainYZ+8.48592*StrainXZ+5.09594*StrainXY+-3.45255e-16*Temp*StrainXX+1.62173e-16*Temp*StrainYY+-3.93054e-18*Temp*StrainZZ+7.90808e-18*Temp*StrainYZ+-1.34608e-17*Temp*StrainXZ+-8.44127e-17*Temp*StrainXY+1533.79*StrainXX*StrainXX+-827.026*StrainXX*StrainYY+-2477.43*StrainXX*StrainZZ+-320.62*StrainXX*StrainYZ+-1010.22*StrainXX*StrainXZ+-1032.78*StrainXX*StrainXY+1533.79*StrainYY*StrainYY+-2477.43*StrainYY*StrainZZ+-1032.78*StrainYY*StrainYZ+-1010.22*StrainYY*StrainXZ+-320.62*StrainYY*StrainXY+2404.32*StrainZZ*StrainZZ+1276.29*StrainZZ*StrainYZ+1936.97*StrainZZ*StrainXZ+1276.29*StrainZZ*StrainXY+4421.22*StrainYZ*StrainYZ+983.747*StrainYZ*StrainXZ+-5370.07*StrainYZ*StrainXY+1520.94*StrainXZ*StrainXZ+983.747*StrainXZ*StrainXY+4421.22*StrainXY*StrainXY)
    solution Silicon add const solve name= RelNvLH val= (0.101861+-5.23632e-05*Temp+13.9*StrainXX+13.6658*StrainYY+13.8634*StrainZZ+-0.205922*StrainYZ+-0.272313*StrainXZ+-0.24048*StrainXY+1.33523e-07*Temp*Temp+-0.0222859*Temp*StrainXX+-0.0220753*Temp*StrainYY+-0.0221849*Temp*StrainZZ+0.00205076*Temp*StrainYZ+0.00221018*Temp*StrainXZ+0.0021336*Temp*StrainXY+231.442*StrainXX*StrainXX+295.086*StrainXX*StrainYY+284.966*StrainXX*StrainZZ+-14.7662*StrainXX*StrainYZ+-20.1485*StrainXX*StrainXZ+-83.0992*StrainXX*StrainXY+219.769*StrainYY*StrainYY+277.839*StrainYY*StrainZZ+-73.3561*StrainYY*StrainYZ+-14.2208*StrainYY*StrainXZ+-8.53869*StrainYY*StrainXY+239.928*StrainZZ*StrainZZ+-15.7671*StrainZZ*StrainYZ+-81.6506*StrainZZ*StrainXZ+-17.2672*StrainZZ*StrainXY+-400.44*StrainYZ*StrainYZ+-284.168*StrainYZ*StrainXZ+-287.556*StrainYZ*StrainXY+-428.572*StrainXZ*StrainXZ+-273.376*StrainXZ*StrainXY+-413.822*StrainXY*StrainXY)
    solution Silicon add const solve name= EvalShiftHH val= (0.00166457+-1.71735e-19*Temp+2.11138*StrainXX+2.11138*StrainYY+2.11138*StrainZZ+1.4161*StrainYZ+1.4161*StrainXZ+1.4161*StrainXY+1.4937e-17*Temp*StrainXX+9.87367e-18*Temp*StrainYY+-5.57247e-18*Temp*StrainZZ+-8.00273e-18*Temp*StrainYZ+-1.51972e-17*Temp*StrainXZ+-1.00468e-17*Temp*StrainXY+452.401*StrainXX*StrainXX+-462.591*StrainXX*StrainYY+-462.591*StrainXX*StrainZZ+-201.655*StrainXX*StrainYZ+-201.655*StrainXX*StrainXZ+390.77*StrainXX*StrainXY+452.401*StrainYY*StrainYY+-462.591*StrainYY*StrainZZ+390.77*StrainYY*StrainYZ+-201.655*StrainYY*StrainXZ+-201.655*StrainYY*StrainXY+452.401*StrainZZ*StrainZZ+-201.655*StrainZZ*StrainYZ+390.77*StrainZZ*StrainXZ+-201.655*StrainZZ*StrainXY+1024.14*StrainYZ*StrainYZ+-809.909*StrainYZ*StrainXZ+-809.909*StrainYZ*StrainXY+1024.14*StrainXZ*StrainXZ+-809.909*StrainXZ*StrainXY+1024.14*StrainXY*StrainXY)
    solution Silicon add const solve name= HHMx val= (0.399521+-7.7696e-18*Temp+-4.07479*StrainXX+-2.39802*StrainYY+-2.39802*StrainZZ+51.4715*StrainYZ+51.4715*StrainXZ+-15.4863*StrainXY+1.45561e-20*Temp*Temp+-8.64346e-17*Temp*StrainXX+-3.84607e-16*Temp*StrainYY+-3.91566e-16*Temp*StrainZZ+2.60059e-16*Temp*StrainYZ+-9.66974e-17*Temp*StrainXZ+-2.13881e-16*Temp*StrainXY+-11003.8*StrainXX*StrainXX+12060.1*StrainXX*StrainYY+12060.1*StrainXX*StrainZZ+-3585.25*StrainXX*StrainYZ+-3585.25*StrainXX*StrainXZ+623.157*StrainXX*StrainXY+-9234.69*StrainYY*StrainYY+8279.6*StrainYY*StrainZZ+12475.7*StrainYY*StrainYZ+-8746.35*StrainYY*StrainXZ+110.352*StrainYY*StrainXY+-9234.69*StrainZZ*StrainZZ+-8746.35*StrainZZ*StrainYZ+12475.7*StrainZZ*StrainXZ+110.352*StrainZZ*StrainXY+-4685.36*StrainYZ*StrainYZ+-11815.4*StrainYZ*StrainXZ+-940.767*StrainYZ*StrainXY+-4685.36*StrainXZ*StrainXZ+-940.767*StrainXZ*StrainXY+-8022.21*StrainXY*StrainXY)
    solution Silicon add const solve name= HHMy val= (0.399521+-1.65975e-18*Temp+-2.39802*StrainXX+-4.07479*StrainYY+-2.39802*StrainZZ+-15.4863*StrainYZ+51.4715*StrainXZ+51.4715*StrainXY+-4.31224e-16*Temp*StrainXX+-2.91045e-16*Temp*StrainYY+3.00289e-16*Temp*StrainZZ+2.40227e-16*Temp*StrainYZ+2.01362e-16*Temp*StrainXZ+1.91756e-16*Temp*StrainXY+-9234.69*StrainXX*StrainXX+12060.1*StrainXX*StrainYY+8279.6*StrainXX*StrainZZ+110.352*StrainXX*StrainYZ+-8746.35*StrainXX*StrainXZ+12475.7*StrainXX*StrainXY+-11003.8*StrainYY*StrainYY+12060.1*StrainYY*StrainZZ+623.157*StrainYY*StrainYZ+-3585.25*StrainYY*StrainXZ+-3585.25*StrainYY*StrainXY+-9234.69*StrainZZ*StrainZZ+110.352*StrainZZ*StrainYZ+12475.7*StrainZZ*StrainXZ+-8746.35*StrainZZ*StrainXY+-8022.21*StrainYZ*StrainYZ+-940.767*StrainYZ*StrainXZ+-940.767*StrainYZ*StrainXY+-4685.36*StrainXZ*StrainXZ+-11815.4*StrainXZ*StrainXY+-4685.36*StrainXY*StrainXY)
    solution Silicon add const solve name= HHMz val= (0.399521+-6.73803e-18*Temp+-2.39802*StrainXX+-2.39802*StrainYY+-4.07479*StrainZZ+51.4715*StrainYZ+-15.4863*StrainXZ+51.4715*StrainXY+1.09391e-20*Temp*Temp+-5.83826e-16*Temp*StrainXX+-5.48141e-16*Temp*StrainYY+-2.53098e-16*Temp*StrainZZ+1.29503e-16*Temp*StrainYZ+-1.61255e-16*Temp*StrainXZ+-1.96329e-16*Temp*StrainXY+-9234.69*StrainXX*StrainXX+8279.6*StrainXX*StrainYY+12060.1*StrainXX*StrainZZ+-8746.35*StrainXX*StrainYZ+110.352*StrainXX*StrainXZ+12475.7*StrainXX*StrainXY+-9234.69*StrainYY*StrainYY+12060.1*StrainYY*StrainZZ+12475.7*StrainYY*StrainYZ+110.352*StrainYY*StrainXZ+-8746.35*StrainYY*StrainXY+-11003.8*StrainZZ*StrainZZ+-3585.25*StrainZZ*StrainYZ+623.157*StrainZZ*StrainXZ+-3585.25*StrainZZ*StrainXY+-4685.36*StrainYZ*StrainYZ+-940.767*StrainYZ*StrainXZ+-11815.4*StrainYZ*StrainXY+-8022.21*StrainXZ*StrainXZ+-940.767*StrainXZ*StrainXY+-4685.36*StrainXY*StrainXY)
    solution Silicon add const solve name= RelNvHH val= (1.02953+-0.00109259*Temp+107.076*StrainXX+112.437*StrainYY+107.133*StrainZZ+3.39407*StrainYZ+0.393355*StrainXZ+0.344772*StrainXY+1.82632e-06*Temp*Temp+-0.167114*Temp*StrainXX+-0.176501*Temp*StrainYY+-0.167226*Temp*StrainZZ+-0.0503582*Temp*StrainYZ+-0.0422518*Temp*StrainXZ+-0.0421287*Temp*StrainXY+-2249.56*StrainXX*StrainXX+5563.15*StrainXX*StrainYY+5497.44*StrainXX*StrainZZ+2066.58*StrainXX*StrainYZ+1998.84*StrainXX*StrainXZ+-6381.29*StrainXX*StrainXY+-2074.21*StrainYY*StrainYY+5581.48*StrainYY*StrainZZ+-6303.15*StrainYY*StrainYZ+2007.24*StrainYY*StrainXZ+2001.46*StrainYY*StrainXY+-2257.2*StrainZZ*StrainZZ+2067.36*StrainZZ*StrainYZ+-6383.58*StrainZZ*StrainXZ+1995.07*StrainZZ*StrainXY+-2459.36*StrainYZ*StrainYZ+10679.8*StrainYZ*StrainXZ+10683.8*StrainYZ*StrainXY+-2789.2*StrainXZ*StrainXZ+10586.1*StrainXZ*StrainXY+-2806.84*StrainXY*StrainXY)
    solution Silicon add const solve name= EvalShiftSO val= (-0.0441505+4.36983e-19*Temp+2.06715*StrainXX+2.06715*StrainYY+2.06715*StrainZZ+0.0503543*StrainYZ+0.0503543*StrainXZ+0.0503543*StrainXY+1.19986e-17*Temp*StrainXX+2.9304e-17*Temp*StrainYY+3.2047e-17*Temp*StrainZZ+2.03375e-17*Temp*StrainYZ+-6.15734e-17*Temp*StrainXZ+-4.45607e-17*Temp*StrainXY+-215.181*StrainXX*StrainXX+215.557*StrainXX*StrainYY+215.557*StrainXX*StrainZZ+-78.4137*StrainXX*StrainYZ+-78.4137*StrainXX*StrainXZ+155.387*StrainXX*StrainXY+-215.181*StrainYY*StrainYY+215.557*StrainYY*StrainZZ+155.387*StrainYY*StrainYZ+-78.4137*StrainYY*StrainXZ+-78.4137*StrainYY*StrainXY+-215.181*StrainZZ*StrainZZ+-78.4137*StrainZZ*StrainYZ+155.387*StrainZZ*StrainXZ+-78.4137*StrainZZ*StrainXY+-541.079*StrainYZ*StrainYZ+-155.845*StrainYZ*StrainXZ+-155.845*StrainYZ*StrainXY+-541.079*StrainXZ*StrainXZ+-155.845*StrainXZ*StrainXY+-541.079*StrainXY*StrainXY)
    solution Silicon add const solve name= SOMx val= (0.209273+-3.32609e-18*Temp+-4.28684*StrainXX+2.11817*StrainYY+2.11817*StrainZZ+-12.6279*StrainYZ+-12.6279*StrainXZ+-5.1134*StrainXY+-5.70157e-17*Temp*StrainXX+4.04599e-17*Temp*StrainYY+1.532e-17*Temp*StrainZZ+-4.08275e-17*Temp*StrainYZ+7.38235e-17*Temp*StrainXZ+-4.72564e-17*Temp*StrainXY+98.3928*StrainXX*StrainXX+-102.938*StrainXX*StrainYY+-102.938*StrainXX*StrainZZ+143.61*StrainXX*StrainYZ+143.61*StrainXX*StrainXZ+134.055*StrainXX*StrainXY+373.241*StrainYY*StrainYY+-642.867*StrainYY*StrainZZ+78.4702*StrainYY*StrainYZ+-221.353*StrainYY*StrainXZ+-63.2251*StrainYY*StrainXY+373.241*StrainZZ*StrainZZ+-221.353*StrainZZ*StrainYZ+78.4702*StrainZZ*StrainXZ+-63.2251*StrainZZ*StrainXY+1491.04*StrainYZ*StrainYZ+1408.01*StrainYZ*StrainXZ+119.433*StrainYZ*StrainXY+1491.04*StrainXZ*StrainXZ+119.433*StrainXZ*StrainXY+1301.93*StrainXY*StrainXY)
    solution Silicon add const solve name= SOMy val= (0.209273+-1.6807e-18*Temp+2.11817*StrainXX+-4.28684*StrainYY+2.11817*StrainZZ+-5.1134*StrainYZ+-12.6279*StrainXZ+-12.6279*StrainXY+-2.60757e-16*Temp*StrainXX+-1.89298e-17*Temp*StrainYY+5.62596e-17*Temp*StrainZZ+1.94808e-17*Temp*StrainYZ+-9.61366e-17*Temp*StrainXZ+-1.34549e-16*Temp*StrainXY+373.241*StrainXX*StrainXX+-102.938*StrainXX*StrainYY+-642.867*StrainXX*StrainZZ+-63.2251*StrainXX*StrainYZ+-221.353*StrainXX*StrainXZ+78.4702*StrainXX*StrainXY+98.3928*StrainYY*StrainYY+-102.938*StrainYY*StrainZZ+134.055*StrainYY*StrainYZ+143.61*StrainYY*StrainXZ+143.61*StrainYY*StrainXY+373.241*StrainZZ*StrainZZ+-63.2251*StrainZZ*StrainYZ+78.4702*StrainZZ*StrainXZ+-221.353*StrainZZ*StrainXY+1301.93*StrainYZ*StrainYZ+119.433*StrainYZ*StrainXZ+119.433*StrainYZ*StrainXY+1491.04*StrainXZ*StrainXZ+1408.01*StrainXZ*StrainXY+1491.04*StrainXY*StrainXY)
    solution Silicon add const solve name= SOMz val= (0.209273+9.83751e-20*Temp+2.11817*StrainXX+2.11817*StrainYY+-4.28684*StrainZZ+-12.6279*StrainYZ+-5.1134*StrainXZ+-12.6279*StrainXY+-1.86512e-17*Temp*StrainXX+-7.07453e-17*Temp*StrainYY+5.35673e-18*Temp*StrainZZ+-9.52555e-17*Temp*StrainYZ+-4.22824e-17*Temp*StrainXZ+1.78138e-16*Temp*StrainXY+373.241*StrainXX*StrainXX+-642.867*StrainXX*StrainYY+-102.938*StrainXX*StrainZZ+-221.353*StrainXX*StrainYZ+-63.2251*StrainXX*StrainXZ+78.4702*StrainXX*StrainXY+373.241*StrainYY*StrainYY+-102.938*StrainYY*StrainZZ+78.4702*StrainYY*StrainYZ+-63.2251*StrainYY*StrainXZ+-221.353*StrainYY*StrainXY+98.3928*StrainZZ*StrainZZ+143.61*StrainZZ*StrainYZ+134.055*StrainZZ*StrainXZ+143.61*StrainZZ*StrainXY+1491.04*StrainYZ*StrainYZ+119.433*StrainYZ*StrainXZ+1408.01*StrainYZ*StrainXY+1301.93*StrainXZ*StrainXZ+119.433*StrainXZ*StrainXY+1491.04*StrainXY*StrainXY)
    solution Silicon add const solve name= RelNvSO val= (-0.00854304+9.82465e-05*Temp+1.25667*StrainXX+1.26484*StrainYY+1.25462*StrainZZ+-0.222676*StrainYZ+-0.222057*StrainXZ+-0.220665*StrainXY+-9.27207e-08*Temp*Temp+-0.00103365*Temp*StrainXX+-0.00101494*Temp*StrainYY+-0.00106247*Temp*StrainZZ+0.0008253*Temp*StrainYZ+0.000831694*Temp*StrainXZ+0.000826674*Temp*StrainXY+-40.7434*StrainXX*StrainXX+142.595*StrainXX*StrainYY+142.179*StrainXX*StrainZZ+-46.438*StrainXX*StrainYZ+-45.7955*StrainXX*StrainXZ+13.6215*StrainXX*StrainXY+-39.2734*StrainYY*StrainYY+142.317*StrainYY*StrainZZ+12.7335*StrainYY*StrainYZ+-46.7205*StrainYY*StrainXZ+-47.1317*StrainYY*StrainXY+-41.7203*StrainZZ*StrainZZ+-45.041*StrainZZ*StrainYZ+14.4562*StrainZZ*StrainXZ+-45.143*StrainZZ*StrainXY+-162.627*StrainYZ*StrainYZ+-29.312*StrainYZ*StrainXZ+-28.859*StrainYZ*StrainXY+-162.39*StrainXZ*StrainXZ+-29.4927*StrainXZ*StrainXY+-163.625*StrainXY*StrainXY)
  } else {
    solution Silicon add const solve name= LHMx val= 0.173694
    solution Silicon add const solve name= LHMy val= 0.173694
    solution Silicon add const solve name= LHMz val= 0.173694
    solution Silicon add const solve name= HHMx val= 0.541312
    solution Silicon add const solve name= HHMy val= 0.541312
    solution Silicon add const solve name= HHMz val= 0.541312
    solution Silicon add const solve name= SOMx val= 0.20868
    solution Silicon add const solve name= SOMy val= 0.20868
    solution Silicon add const solve name= SOMz val= 0.208
    solution Silicon add const solve val= 0 name= EvalShiftLH
    solution Silicon add const solve val= 0 name= EvalShiftHH
    solution Silicon add const solve val= 0 name= EvalShiftSO
    solution Silicon add const solve val= 0.0931482 name= RelNvLH
    solution Silicon add const solve val= 0.851902  name= RelNvHH
    solution Silicon add const solve val= 0.0549498 name= RelNvSO
  }

  solution Silicon add const solve val= [DGInvMass mx=LHMx my=LHMy mz=LHMz] name= DGLHInvMass
  solution Silicon add const solve val= [DGInvMass mx=HHMx my=HHMy mz=HHMz] name= DGHHInvMass
  solution Silicon add const solve val= [DGInvMass mx=SOMx my=SOMy mz=SOMz] name= DGSOInvMass

  solution Silicon add name=Eval  const solve val= [Eval Eg=Eg0]
  solution Silicon add name=LightHole const solve val= [ValFermiDirac Ev=Eval+EvalShiftLH-(Lambda*DGLHInvMass) Nv=$Nv*RelNvLH T=$T] 
  solution Silicon add name=HeavyHole const solve val= [ValFermiDirac Ev=Eval+EvalShiftHH-(Lambda*DGHHInvMass) Nv=$Nv*RelNvHH T=$T] 
  solution Silicon add name=SOHole    const solve val= [ValFermiDirac Ev=Eval+EvalShiftSO-(Lambda*DGSOInvMass) Nv=$Nv*RelNvSO T=$T] 
  solution Silicon add name=Hole const solve val= LightHole+HeavyHole+SOHole
  solution Silicon add name=Hole0 const solve val= [ValFermiDirac Ev=Eval Nv=$Nv T=$T]

  set kp {{NA LHMx LHMy LHMz NA NA NA NA} {NA HHMx HHMy HHMz NA NA NA NA} {NA SOMx SOMy SOMz NA NA NA NA}}
  # set HoleTensor [3DTensorListToFlooxsTensor [Get3DHoleDPTensor "(LightHole+1000) (HeavyHole+1000)" $kp $miller ]]
  set HoleTensor [3DTensorListToFlooxsTensor [Get3DHoleDPTensor "(LightHole+1000) (HeavyHole+1000) (SOHole+1000)" $kp $miller ]]

  # pdbSet Silicon Qfp Equation [QFHoleContinuity Hole= $HoleTensor R=R G=G]
  # pdbSet Silicon Qfp Equation [QFHoleContinuity Hole= Hole R=R G=G]
  pdbSet Silicon Qfp Equation "ddt(Hole) - (Hmob) * Hole * Velocity(GradX(Qfp), GradY(Qfp), GradZ(Qfp))  + (R) - (G)"
  
  pdbSet Silicon DevPsi Equation [Poisson er= 11.8]
  pdbSet Silicon DevPsi Abs.Error 1.0e-9
  pdbSet Silicon DevPsi DampValue 0.025
  pdbSet Silicon Lambda Equation [DGEquation T=$T]
  pdbSet Silicon Lambda DampValue 0.025
  # Update band gap
  sel name=Eg z= "Eg0+(SiEcondShift100+SiEcondShift010+SiEcondShift001)/3."

  solution Silicon add name= Jn const solve val= [CurrentMag Conc=((((((RelEmob100_100)))*(1.0*((Elec100+1000)))))+(((((RelEmob010_100)))*(1.0*((Elec010+1000)))))+(((((RelEmob001_100)))*(1.0*((Elec001+1000)))))) Mob=Emob Qf=Qfn]
  solution Silicon add name= Jp const solve val= [CurrentMag Conc=Hole Mob=Hmob Qf=Qfp]


}

proc SGEqns args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=?"
  lappend usage "Temperature (T) is required in Kelvin."
  set description {Sets the silicon SG equations}
  set returns "nothing"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set T [__GetValue $args T "$name Error: No temperature (T) provided"]

  # Set Emob and Hmob for Silicon
  # Force mosfet model until Klaassen is sorted out
  set ::__MOS 1
  __SiliconMobility $T
  # solution Silicon name=Emob add const solve val= 300
  # solution Silicon name=Hmob add const solve val= 150
  # intrensic carrier concentrations and band edge DOS
  set Nc [Silicon::Nc T=$T] ;# 1/cm^3
  set Nv [Silicon::Nv T=$T] ;# 1/cm^3
  solution Silicon add name=Eg0 const solve val= ([Silicon::BandGap T=$T]-[Silicon::OldSlotboom])
  # Compute the band gap after strain
  # This is updated in strain procs
  # This may need to be upgraded to a solution in the future
  sel name= Eg z= ([Silicon::BandGap T=$T]-[Silicon::OldSlotboom])
  solution Silicon add name= ni const solve val= [IntrinsicConc Nv=$Nv Nc=$Nc Eg=Eg T=$T]
  # Recombination
  solution Silicon add name= Ava const solve val= (([Silicon::vanOberstraetenHole T=$T ])+([Silicon::vanOberstraetenElec T=$T]))
  solution Silicon add name= SRH const solve val= [SRH ni=ni]
  solution Silicon add name= Auger const solve val= [Auger ni=ni]
  # Schenk B2B is recombination
  # solution Silicon add name= B2B const solve val= [Schenk Eg=Eg T=$T ni=ni Nc=$Nc Nv=$Nv]
  # B2BE? is is generation
  solution Silicon add name= B2B const solve val= [B2BE1]/1e8

  solution Silicon add name= R const solve val= (SRH+Auger)
  solution Silicon add name= G const solve val= (Ava+B2B)

  set q 1.602176634e-19
  set k 1.38066e-23
  set Vt "([expr $k/$q]*$T)"
  solution Silicon add name= Qfp const solve val= "DevPsi + $Vt*log((Hole+1)/ni)"
  solution Silicon add name= Qfn const solve val= "DevPsi - $Vt*log((Elec+1)/ni)"

  # Electron Equations
  pdbSet Silicon Elec Equation [SGElecContinuity T=$T R=R G=G]
  pdbSet Silicon Elec Abs.Error 1.0e-5

  # Define band edges as constant solution
  solution Silicon add name=Econd const solve val= [Econd]
  solution Silicon add name=Eval  const solve val= [Eval Eg=Eg0]
  
  # Hole Equations
  pdbSet Silicon Hole Equation [SGHoleContinuity T=$T R=R G=G]
  pdbSet Silicon Hole Abs.Error 1.0e-5

  pdbSet Silicon DevPsi Equation [Poisson er= 11.8]
  pdbSet Silicon DevPsi Abs.Error 1.0e-9
  pdbSet Silicon DevPsi DampValue 0.025
}

proc KandaEqns args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=? stress= {0 0 0 0 0 0} miller=(001)<100>"
  lappend usage "Temperature (T) is required in Kelvin."
  lappend usage "Stress vector is in Voigt notation string list format and assumed to be a vector of six zeros."
  lappend usage "Miller indicies (miller) are in (hkl)<uvw> format and defaults to 001 wafers with 100 channels."
  set description {Sets the silicon stress equations}
  set returns "nothing"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set stress [__GetValue $args stress ]
  if {$stress == ""} { set stress {0 0 0 0 0 0} }
  set miller [__GetValue $args miller]
  if {$miller == ""} { set miller (001)<100> }
  set T [__GetValue $args T "$name Error: No temperature (T) provided"]


  __SiliconMobility $T
  # intrensic carrier concentrations and band edge DOS
  set Nc [Silicon::Nc T=$T] ;# 1/cm^3
  set Nv [Silicon::Nv T=$T] ;# 1/cm^3
  solution Silicon add name=Eg0 const solve val= ([Silicon::BandGap T=$T]-[Silicon::OldSlotboom])
  # Compute the band gap after strain
  # This is updated in strain procs
  # This may need to be upgraded to a solution in the future
  sel name= Eg z= ([Silicon::BandGap T=$T]-[Silicon::OldSlotboom])
  solution Silicon add name= ni const solve val= [IntrinsicConc Nv=$Nv Nc=$Nc Eg=Eg T=$T]
  # Recombination
  solution Silicon add name= SRH const solve val= [SRH ni=ni]
  solution Silicon add name= Auger const solve val= [Auger ni=ni]
  solution Silicon add name= Schenk const solve val= [Schenk Eg=Eg T=$T ni=ni Nc=$Nc Nv=$Nv]
  solution Silicon add name= R const solve val= (SRH+Auger)
  # Above here is the same as Eqns
  # get the kanda tensors
  set ETensor [KandaFirstElecConductivtyFactorTensor stress=$stress miller=$miller mat=Silicon ]
  set HTensor [KandaFirstHoleConductivtyFactorTensor stress=$stress miller=$miller mat=Silicon ]
  set Ava "(Emob * Elec * sqrt(([lindex $ETensor 0 0 0]*GradX(Qfn)+1)^2+([lindex $ETensor 0 1 1]*GradY(Qfn)+1)^2+([lindex $ETensor 0 2 2]*GradZ(Qfn)+1)^2) * Avae + Hmob * Hole * sqrt(([lindex $HTensor 0 0 0]*GradX(Qfp)+1)^2+([lindex $HTensor 0 1 1]*GradY(Qfp)+1)^2+([lindex $HTensor 0 2 2]*GradZ(Qfp)+1)^2) * Avah)"
  solution Silicon add name= G const solve val= ($Ava+Schenk)

  # Electron Equations
  set KandaTensor [3DTensorListToFlooxsTensor $ETensor ]
  pdbSet Silicon Qfn Equation [QFElecContinuity Elec= "(Elec+1000) * $KandaTensor" R=R G=G]
  solution Silicon add name=Econd const solve val= [Econd]
  solution Silicon add name=Elec const solve val= [CondFermiDirac Ec=Econd Nc=$Nc T=$T]
  
  # Hole
  set KandaTensor [3DTensorListToFlooxsTensor $HTensor ]
  pdbSet Silicon Qfp Equation [QFHoleContinuity Hole= "(Hole+1000) * $KandaTensor" R=R G=G]
  solution Silicon add name=Eval  const solve val= [Eval Eg=Eg0]
  solution Silicon add name=Hole const solve val= [ValFermiDirac Ev=Eval Nv=$Nv T=$T]
  
  pdbSet Silicon DevPsi Equation [Poisson er= 11.8]
  pdbSet Silicon DevPsi Abs.Error 1.0e-9
  pdbSet Silicon DevPsi DampValue 0.025
}

proc EqnsSimple args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=?"
  lappend usage "Temperature (T) is required in Kelvin."
  set description {Sets the silicon stress equations}
  set returns "nothing"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set T [__GetValue $args T "$name Error: No temperature (T) provided"]

  # Set Emob and Hmob for Silicon
  __SiliconMobility $T
  # solution Silicon name=Emob add const solve val= 300
  # solution Silicon name=Hmob add const solve val= 150
  # intrensic carrier concentrations and band edge DOS
  set Nc [Silicon::Nc T=$T] ;# 1/cm^3
  set Nv [Silicon::Nv T=$T] ;# 1/cm^3
  solution Silicon add name=Eg0 const solve val= ([Silicon::BandGap T=$T])
  solution Silicon add name= ni const solve val= [IntrinsicConc Nv=$Nv Nc=$Nc Eg=Eg0 T=$T]
  # Avalanche
  solution Silicon add name= Ava const solve val= (([Silicon::vanOberstraetenHole T=$T ])+([Silicon::vanOberstraetenElec T=$T]))

  # Recombination
  solution Silicon add name= SRH const solve val= [SRH ni=ni]
  solution Silicon add name= Auger const solve val= [Auger ni=ni]
  # Schenk B2B is recombination
  solution Silicon add name= B2B const solve val= [B2BE1_5]/1e4

  solution Silicon add name= R const solve val= (SRH+Auger)
  solution Silicon add name= G const solve val= (Ava+B2B)

  # Electron Equations
  solution Silicon add name=Elec   const solve val= [CondFermiDirac Ec=Econd Nc=$Nc T=$T]
  sel z= 0 name= Bx
  sel z= 0 name= By
  sel z= 0 name= Bz
  # put in the tesla to Gauss scaling factor
  sel z=Emob/100./100. name=HallEmob
  set Ten "Elec*Tensor(1,HallEmob*Bz,-HallEmob*By,-HallEmob*Bz,1,HallEmob*Bx,HallEmob*By,-HallEmob*Bx,1)"
  pdbSet Silicon Qfn Equation [QFElecContinuity Elec=$Ten R=R G=G]
  
  # Hole
  pdbSet Silicon Qfp Equation [QFHoleContinuity R=R G=G]
  solution Silicon add name=Eval const solve val= [Eval Eg=Eg0]
  solution Silicon add name=Hole const solve val= [ValFermiDirac Ev=Eval Nv=$Nv T=$T]
  
  pdbSet Silicon DevPsi Equation [Poisson er= 11.8]
  pdbSet Silicon DevPsi Abs.Error 1.0e-9
  pdbSet Silicon DevPsi DampValue 0.025
}

proc EqnsSimpleVel args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=? stress= {0 0 0 0 0 0} miller=(001)<100>"
  lappend usage "Temperature (T) is required in Kelvin."
  lappend usage "Stress vector is in Voigt notation string list format and assumed to be a vector of six zeros."
  lappend usage "Miller indicies (miller) are in (hkl)<uvw> format and defaults to 001 wafers with 100 channels."
  set description {Sets the silicon stress equations}
  set returns "nothing"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set stress [__GetValue $args stress ]
  if {$stress == ""} { set stress {0 0 0 0 0 0} }
  set miller [__GetValue $args miller]
  if {$miller == ""} { set miller (001)<100> }
  set T [__GetValue $args T "$name Error: No temperature (T) provided"]

  # Set Emob and Hmob for Silicon
  # solution Silicon name=Emob add const solve val= 8
  # solution Silicon name=Hmob add const solve val= 8
  __SiliconMobility $T
  # intrensic carrier concentrations and band edge DOS
  set Nc [Silicon::Nc T=$T] ;# 1/cm^3
  set Nv [Silicon::Nv T=$T] ;# 1/cm^3
  # solution Silicon add name= Eg const solve val= ([Silicon::BandGap T=$T]-[Silicon::OldSlotboom])
  sel name= Eg z= ([Silicon::BandGap T=$T]-[Silicon::OldSlotboom])
  solution Silicon add name= ni const solve val= [IntrinsicConc Nv=$Nv Nc=$Nc Eg=Eg T=$T]
  # Avalanche
  set scal 1.0
  solution Silicon add name= Ava const solve val= (([Silicon::vanOberstraetenHole T=$T F= ($scal*Emfp)])+([Silicon::vanOberstraetenElec T=$T F= ($scal*Emfn)]))

  # Generation and Recombination
  solution Silicon add name= SRH const solve val= [SRH ni=ni]
  solution Silicon add name= Auger const solve val= [Auger ni=ni]
  # Schenk B2B is recombination
  solution Silicon add name= Schenk const solve val= [Schenk Eg=Eg T=$T ni=ni Nc=$Nc Nv=$Nv]
  solution Silicon add name= R const solve val= (SRH+Auger)
  solution Silicon add name= G const solve val= (Ava+Schenk)

  # Elec
  solution Silicon add name=Econd const solve val= [Econd]
  solution Silicon add name=Elec  const solve val= Quad([CondFermiDirac Ec=Econd Nc=$Nc T=$T])
  solution Silicon add name=ElecNoQ  const solve val= [CondFermiDirac Ec=Econd Nc=$Nc T=$T]
  set floor 1e3
  set vel 1
  if { $vel } {
    pdbSet Silicon Qfn Equation "ddt(Elec) + Emob * (ElecNoQ+$floor) * Velocity(GradX(Qfn), GradY(Qfn), GradZ(Qfn)) + (R) - (G)"
  } else {
    pdbSet Silicon Qfn Equation "ddt(Elec) + Emob * (Elec+$floor) * grad(Qfn) + (R) - (G)"
  }
  
  # Compute the hole properties
  solution Silicon add name=Eval const solve val= [Eval Eg=Eg]
  solution Silicon add name=Hole const solve val= Quad([ValFermiDirac Ev=Eval Nv=$Nv T=$T])
  solution Silicon add name=HoleNoQ const solve val= [ValFermiDirac Ev=Eval Nv=$Nv T=$T]
  if { $vel } {
    pdbSet Silicon Qfp Equation "ddt(Hole) - Hmob * (HoleNoQ+$floor) * Velocity(GradX(Qfp), GradY(Qfp), GradZ(Qfp)) + (R) - (G)"
  } else {
    pdbSet Silicon Qfp Equation "ddt(Hole) - Hmob * (Hole+$floor) * grad(Qfp) + (R) - (G)"
  }
  
  pdbSet Silicon DevPsi Equation [Poisson er= 11.8]
  pdbSet Silicon DevPsi Equation [Poisson er= 11.8 Elec= ElecNoQ Hole=HoleNoQ]
  pdbSet Silicon DevPsi Abs.Error 1.0e-9
  pdbSet Silicon DevPsi DampValue 0.025

}

proc EqnsSimpleVel args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=? stress= {0 0 0 0 0 0} miller=(001)<100>"
  lappend usage "Temperature (T) is required in Kelvin."
  lappend usage "Stress vector is in Voigt notation string list format and assumed to be a vector of six zeros."
  lappend usage "Miller indicies (miller) are in (hkl)<uvw> format and defaults to 001 wafers with 100 channels."
  set description {Sets the silicon stress equations}
  set returns "nothing"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set stress [__GetValue $args stress ]
  if {$stress == ""} { set stress {0 0 0 0 0 0} }
  set miller [__GetValue $args miller]
  if {$miller == ""} { set miller (001)<100> }
  set T [__GetValue $args T "$name Error: No temperature (T) provided"]

  # Set Emob and Hmob for Silicon
  # solution Silicon name=Emob add const solve val= 8
  # solution Silicon name=Hmob add const solve val= 8
  __SiliconMobility $T
  # intrensic carrier concentrations and band edge DOS
  set Nc [Silicon::Nc T=$T] ;# 1/cm^3
  set Nv [Silicon::Nv T=$T] ;# 1/cm^3
  # solution Silicon add name= Eg const solve val= ([Silicon::BandGap T=$T]-[Silicon::OldSlotboom])
  sel name= Eg z= ([Silicon::BandGap T=$T]-[Silicon::OldSlotboom])
  solution Silicon add name= ni const solve val= [IntrinsicConc Nv=$Nv Nc=$Nc Eg=Eg T=$T]
  # Avalanche
  set scal 1.0
  solution Silicon add name= Ava const solve val= (([Silicon::vanOberstraetenHole T=$T F= ($scal*Emfp)])+([Silicon::vanOberstraetenElec T=$T F= ($scal*Emfn)]))

  # Generation and Recombination
  solution Silicon add name= SRH const solve val= [SRH ni=ni]
  solution Silicon add name= Auger const solve val= [Auger ni=ni]
  # Schenk B2B is recombination
  solution Silicon add name= Schenk const solve val= [Schenk Eg=Eg T=$T ni=ni Nc=$Nc Nv=$Nv]
  solution Silicon add name= R const solve val= (SRH+Auger)
  solution Silicon add name= G const solve val= (Ava+Schenk)

  # Elec
  solution Silicon add name=Econd const solve val= [Econd]
  solution Silicon add name=ElecQ  const solve val= Quad([CondFermiDirac Ec=Econd Nc=$Nc T=$T])
  solution Silicon add name=Elec  const solve val= [CondFermiDirac Ec=Econd Nc=$Nc T=$T]
  set floor 1e3
  set vel 1
  if { $vel } {
    pdbSet Silicon Qfn Equation "ddt(Elec) + Emob * (Elec+$floor) * Velocity(GradX(Qfn), GradY(Qfn), GradZ(Qfn)) + (R) - (G)"
  } else {
    pdbSet Silicon Qfn Equation "ddt(Elec) + Emob * (Elec+$floor) * grad(Qfn) + (R) - (G)"
  }
  
  # Compute the hole properties
  solution Silicon add name=Eval const solve val= [Eval Eg=Eg]
  solution Silicon add name=HoleQ const solve val= Quad([ValFermiDirac Ev=Eval Nv=$Nv T=$T])
  solution Silicon add name=Hole const solve val= [ValFermiDirac Ev=Eval Nv=$Nv T=$T]
  if { $vel } {
    pdbSet Silicon Qfp Equation "ddt(Hole) - Hmob * (Hole+$floor) * Velocity(GradX(Qfp), GradY(Qfp), GradZ(Qfp)) + (R) - (G)"
  } else {
    pdbSet Silicon Qfp Equation "ddt(Hole) - Hmob * (Hole+$floor) * grad(Qfp) + (R) - (G)"
  }
  
  pdbSet Silicon DevPsi Equation [Poisson er= 11.8]
  pdbSet Silicon DevPsi Abs.Error 1.0e-9
  pdbSet Silicon DevPsi DampValue 0.025

  solution Silicon add name= Jn const solve val= [CurrentMag Conc=ElecQ Mob=Emob Qf=Qfn]
  solution Silicon add name= Jp const solve val= [CurrentMag Conc=HoleQ Mob=Hmob Qf=Qfp]

}

# proc EqnsSimpleVel args {
#   set name [lindex [info level 0] 0]
#   lappend usage "$name T=? stress= {0 0 0 0 0 0} miller=(001)<100>"
#   lappend usage "Temperature (T) is required in Kelvin."
#   lappend usage "Stress vector is in Voigt notation string list format and assumed to be a vector of six zeros."
#   lappend usage "Miller indicies (miller) are in (hkl)<uvw> format and defaults to 001 wafers with 100 channels."
#   set description {Sets the silicon stress equations}
#   set returns "nothing"
#   if { [__Help $args $name $usage $returns $description] } {
#     return ""
#   }
#   set stress [__GetValue $args stress ]
#   if {$stress == ""} { set stress {0 0 0 0 0 0} }
#   set miller [__GetValue $args miller]
#   if {$miller == ""} { set miller (001)<100> }
#   set T [__GetValue $args T "$name Error: No temperature (T) provided"]
# 
#   # Set Emob and Hmob for Silicon
#   solution Silicon name=Emob add const solve val= 8
#   solution Silicon name=Hmob add const solve val= 8
#   # intrensic carrier concentrations and band edge DOS
#   set Nc [Silicon::Nc T=$T] ;# 1/cm^3
#   set Nv [Silicon::Nv T=$T] ;# 1/cm^3
#   # solution Silicon add name= Eg const solve val= ([Silicon::BandGap T=$T]-[Silicon::OldSlotboom])
#   sel name= Eg z= ([Silicon::BandGap T=$T]-[Silicon::OldSlotboom])
#   solution Silicon add name= ni const solve val= [IntrinsicConc Nv=$Nv Nc=$Nc Eg=Eg T=$T]
#   # Avalanche
#   set scal 1.2
#   solution Silicon add name= Ava const solve val= (([Silicon::vanOberstraetenHole T=$T F= ($scal*Emfp)])+([Silicon::vanOberstraetenElec T=$T F= ($scal*Emfn)]))
# 
#   # Generation and Recombination
#   solution Silicon add name= SRH const solve val= [SRH ni=ni]
#   solution Silicon add name= Auger const solve val= [Auger ni=ni]
#   # Schenk B2B is recombination
#   solution Silicon add name= Schenk const solve val= [Schenk Eg=Eg T=$T ni=ni Nc=$Nc Nv=$Nv]
#   solution Silicon add name= R const solve val= (SRH+Auger)
#   solution Silicon add name= G const solve val= (Ava+Schenk)
# 
#   # Elec
#   solution Silicon add name=Econd const solve val= [Econd]
#   solution Silicon add name=Elec  const solve val= [CondFermiDirac Ec=Econd Nc=$Nc T=$T]
#   set floor 1e3
#   set vel 1
#   if { $vel } {
#     pdbSet Silicon Qfn Equation "ddt(Elec) + Emob * (Elec+$floor) * Velocity(GradX(Qfn), GradY(Qfn), GradZ(Qfn)) + (R) - (G)"
#   } else {
#     pdbSet Silicon Qfn Equation "ddt(Elec) + Emob * (Elec+$floor) * grad(Qfn) + (R) - (G)"
#   }
#   
#   # Compute the hole properties
#   solution Silicon add name=Eval const solve val= [Eval Eg=Eg]
#   solution Silicon add name=Hole const solve val= [ValFermiDirac Ev=Eval Nv=$Nv T=$T]
#   if { $vel } {
#     pdbSet Silicon Qfp Equation "ddt(Hole) - Hmob * (Hole+$floor) * Velocity(GradX(Qfp), GradY(Qfp), GradZ(Qfp)) + (R) - (G)"
#   } else {
#     pdbSet Silicon Qfp Equation "ddt(Hole) - Hmob * (Hole+$floor) * grad(Qfp) + (R) - (G)"
#   }
#   
#   pdbSet Silicon DevPsi Equation [Poisson er= 11.8]
#   pdbSet Silicon DevPsi Abs.Error 1.0e-9
#   pdbSet Silicon DevPsi DampValue 0.025
# 
#   solution Silicon add name= Jn const solve val= [CurrentMag Conc=Elec Mob=Emob Qf=Qfn]
#   solution Silicon add name= Jp const solve val= [CurrentMag Conc=Hole Mob=Hmob Qf=Qfp]
# 
# }

}
