namespace eval Silicon {

proc CanaliHole args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=? Hmob=? F=E"
  lappend usage {Hole mobility (Hmob).}
  lappend usage {Driving force (F).}
  lappend usage {Temperature (T) is in Kelvin.}
  set returns {alagator expression}
  set description {Canali Hole velocity saturation mobility model.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set Hmob [__GetValue $args Hmob "$name Error: No hole mobility (Hmob) specified"]
  set F [__GetValue $args F ]
  if {$F == ""} { set F E }

  set alpha 0
  set beta0 1.213
  set betaexp 0.17
  set beta "($beta0*($T/300.)^$betaexp)"
  set Vsat0 8.73e6
  set Vsatexp 0.52
  set Vsat "($Vsat0*(300./$T)^($Vsatexp))"
  return "([expr $alpha+1]*($Hmob)/($alpha+(1.0+([expr $alpha+1]*($Hmob)*($F)/$Vsat)^($beta))^(1.0/$beta)))"
}

proc CanaliElec args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=? Emob=? F=E"
  lappend usage {Temperature (T) is in Kelvin.}
  lappend usage {Electron mobility (Emob).}
  lappend usage {Driving force (F).}
  set returns {alagator expression}
  set description {Canali electron velocity saturation mobility model.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set Emob [__GetValue $args Emob "$name Error: No electron mobility (Emob) specified"]
  set F [__GetValue $args F ]
  if {$F == ""} { set F E }
  set alpha 0
  set beta0 1.109
  set betaexp 0.66
  set beta "($beta0*($T/300.)^($betaexp))"
  set Vsat0 1.07e7 ;# cm/s
  set Vsatexp 0.87
  set Vsat "($Vsat0*(300./$T)^($Vsatexp))"
  return "([expr $alpha+1]*($Emob)/($alpha+(1.0+([expr $alpha+1]*($Emob)*($F)/$Vsat)^($beta))^(1.0/$beta)))"
}

proc CanaliHoleFactor args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=? v=Hmob*Emfp"
  lappend usage {Velocity (v) defaults to the hole mobility times the hole electromotive force.}
  lappend usage {Temperature (T) is in Kelvin.}
  set returns {alagator expression}
  set description {Canali Hole velocity saturation mobility model.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set v [__GetValue $args v ]
  if {$v == ""} { set v Hmob*Emfp }

  set alpha 0
  set beta0 1.213
  set betaexp 0.17
  set beta "($beta0*($T/300.)^$betaexp)"
  set Vsat0 8.73e6
  set Vsatexp 0.52
  set Vsat "($Vsat0*(300./$T)^($Vsatexp))"
  return "([expr $alpha+1]/($alpha+(1.0+([expr $alpha+1]*($v)/$Vsat)^($beta))^(1.0/$beta)))"
}

proc CanaliElecFactor args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=? v=Emob*Emfn"
  lappend usage {Velocity (v) defaults to the elec mobility times the elec electromotive force.}
  lappend usage {Temperature (T) is in Kelvin.}
  lappend usage {Driving force (F).}
  set returns {alagator expression}
  set description {Canali electron velocity saturation mobility model.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set Emob [__GetValue $args Emob "$name Error: No elec mobility (Emob) specified"]
  set v [__GetValue $args v ]
  if {$v == ""} { set v Emob*Emfn }
  set alpha 0
  set beta0 1.109
  set betaexp 0.66
  set beta "($beta0*($T/300.)^($betaexp))"
  set Vsat0 1.07e7 ;# cm/s
  set Vsatexp 0.87
  set Vsat "($Vsat0*(300./$T)^($Vsatexp))"
  return "([expr $alpha+1]/($alpha+(1.0+([expr $alpha+1]*($v)/$Vsat)^($beta))^(1.0/$beta)))"
}

# puts "h 0. [expr [string map {^ ** DevPsiMag (0.)} [CanaliHole 400 300]]]"
# puts "e 0. [expr [string map {^ ** DevPsiMag (0.)} [CanaliElec 400 300]]]"
# for { set x 1.0 } { $x < 1e10 } { set x [expr $x*2] } {
#   puts "h $x [expr [string map {^ ** DevPsiMag ($x)} [CanaliHole 400 300]]]"
#   puts "e $x [expr [string map {^ ** DevPsiMag ($x)} [CanaliElec 400 300]]]"
# }
# gets stdin
}
