namespace eval Silicon {
proc KlaassenNP args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T= Accept=Accept Donor=Donor Elec=Elec Hole=Hole expand"
  lappend usage {Temperature (T) is in Kelvin.}
  lappend usage {Donor concentration (Donor) is assumed to be the alagator field Donor and dopant type phosphorus.}
  lappend usage {Accept concentration (Accept) is assumed to be the alagator field Accept.}
  lappend usage {Hole concentration (Hole) is assumed to be the alagator field Hole.}
  lappend usage {Elec concentration (Elec) is assumed to be the alagator field Elec.}
  lappend usage {The expand flag is used to expand the terms into a full string for testing purposes.}
  set returns {alagator string}
  set description {Klaassen, aka Philips, Phosphorus Donor Mobility Model.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set expand [__Contains $args expand]
  set Accept [__GetValue $args Accept ]
  if {$Accept == ""} { set Accept Accept }
  set Donor [__GetValue $args Donor ]
  if {$Donor == ""} { set Donor Donor }
  set Elec [__GetValue $args Elec ]
  if {$Elec == ""} { set Elec Elec }
  set Hole [__GetValue $args Hole ]
  if {$Hole == ""} { set Hole Hole }
  
  #Parameter values for Phospherous
  set KPEmumax 1414.0
  set KPEmumin 68.5
  set KPENref1 9.2e16
  set KPEalph1 0.711
  set KPEcD 0.21
  set KPENrefD 4.0e20
  set KPEcA 0.50
  set KPENrefA 7.2e20
  set KPEtheta_e 2.285
  set KPEtheta_h 2.247
  
  #Klaussen's fit parameters
  set KPEme 1.0
  set KPEmh 1.258
  set KPEfcw 2.459
  set KPEfbh 3.828
  
  # Lattice scattering
  set KPEmuL "($KPEmumax*(300.0/$T)^$KPEtheta_e)"
  if { !$expand } {
    term name=KPEmuL add eqn=$KPEmuL
  }
  
  # Impurity and Hole Scattering.
  set KPEmuiN [expr "($KPEmumax*$KPEmumax/($KPEmumax-$KPEmumin))"]
  set KPEmuiN "($KPEmuiN*($T/300.0)^(3.0*$KPEalph1-1.5))"
  set KPEmuiC [expr "($KPEmumax*$KPEmumin/($KPEmumax-$KPEmumin))"]
  set KPEmuiC "($KPEmuiC*(300.0/$T)^0.5)"
  set KPENsc "($Donor+$Accept+$Hole)"
  set KPEPBH "(1.36e20/$Elec*$KPEme*($T/300.0)^2.0)"
  set KPEZ   "(1.0+1.0/($KPEcA+($KPENrefA/$KPENsc)^2.0))"
  if { !$expand } {
    term name=KPEZ add eqn=$KPEZ
  }
  set KPEPCW "(3.97e13*((1.0/((KPEZ)^3.0*$KPENsc))*($T/300.0)^3.0)^(2.0/3.0))"
  set KPEP   "(1.0/($KPEfcw/($KPEPCW)+$KPEfbh/($KPEPBH)))"
  if { !$expand } {
  term name=KPEP add eqn=$KPEP
  }
  
  set KPEs1 0.89233
  set KPEs2 0.41372
  set KPEs3 0.19778
  set KPEs4 0.28227
  set KPEs5 0.005978
  set KPEs6 1.80618
  set KPEs7 0.72169
  set KPE_G  "(1.0-$KPEs1/($KPEs2+(($T/300.0/$KPEme)^$KPEs4)*(KPEP))^$KPEs3+$KPEs5/((($KPEme*300.0/$T)^$KPEs7)*KPEP)^$KPEs6)"
  if { !$expand } {
  term name=KPE_G add eqn=$KPE_G
  }
  
  set KPEr1 0.7643
  set KPEr2 2.2999
  set KPEr3 6.5502
  set KPEr4 2.3670
  set KPEr5 -0.01552
  set KPEr6 0.678
  set KPE_F  "(($KPEr1*KPEP^$KPEr6+$KPEr2+$KPEr3*$KPEme/$KPEmh)/(KPEP^$KPEr6+$KPEr4+$KPEr5*$KPEme/$KPEmh))"
  if { !$expand } {
  term name=KPE_F add eqn=$KPE_F
  }
  
  set KPENsceff "($Donor+KPE_G*$Accept+$Hole/KPE_F)"
  
  set KPEmuDAh "($KPEmuiN*$KPENsc/$KPENsceff*($KPENref1/$KPENsc)^$KPEalph1+$KPEmuiC*($Elec+$Hole)/$KPENsceff)"
  if { !$expand } {
  term name=KPEmuDAh add eqn=$KPEmuDAh
  }
  
  # Combine using Mathiesson's rule
  if { $expand } {
    # expand the string by term 10 iterations is just to make sure
    # the whole expression gets expanded. Eval is key to the expand
    set ret "(KPEmuL*KPEmuDAh/(KPEmuL+KPEmuDAh))"
    for { set i 0 } { $i < 10 } { incr i } {
      eval set ret [string map {K "$\K"} $ret]
    }
    return $ret
  } else {
    return "(KPEmuL*KPEmuDAh/(KPEmuL+KPEmuDAh))"
  }
}


}
