namespace eval Silicon {
# Masetti Mobility Model from Sentaurus, P at end of parameter is Phospherous, B at end of parameter is Boron
proc Masetti args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T= Accept=Accept Donor=Donor"
  lappend usage {Temperature (T) is in Kelvin.}
  lappend usage {Donor concentration (Donor) is assumed to be the alagator field Donor and dopant type As.}
  lappend usage {Accept concentration (Accept) is assumed to be the alagator field Accept.}
  set returns {A list of two alagator expressions, the first item is the electron mobility and the second is the hole mobility }
  set description {Masetti majority carrier mobility model. Great to use for many applications where the majority carrier is the only concern.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set expand [__Contains $args expand]
  set Accept [__GetValue $args Accept ]
  if {$Accept == ""} { set Accept Accept }
  set Donor [__GetValue $args Donor ]
  if {$Donor == ""} { set Donor Donor }
  
  set umin1E  52.2
  set umin1H  44.9
  set umin2E  52.2
  set umin2H  0.0
  set uconstE "1417 *($T/300.0)^(-2.5)"
  set uconstH "470.5*($T/300.0)^(-2.2)"
  set u1E     43.4
  set u1H     29.0
  set CrE     9.68e16
  set CrH     2.23e17
  set CsE     3.43e20
  set CsH     6.10e20
  set alphaE  0.680
  set alphaH  0.719
  set betaE   2.00
  set betaH   2.00
  set PcH     9.23e16
  set tmp {}
  # Emob
  lappend tmp "$umin1E + ($uconstE-$umin2E)/(1+((abs($Donor)+abs($Accept))/$CrE)^$alphaE) - $u1E/(1+($CsE/(abs($Donor)+abs($Accept)))^$betaE)"
  # Hmob
  lappend tmp "$umin1H*exp(-$PcH/(abs($Donor)+abs($Accept))) + ($uconstH-$umin2H)/(1+((abs($Donor)+abs($Accept))/$CrH)^$alphaH) - $u1H/(1+($CsH/(abs($Donor)+abs($Accept)))^$betaH)"
  return $tmp
}
}

