namespace eval Silicon {
proc SurfacePhonon args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T= Accept=Accept Donor=Donor Fperp=Eperp Dcr=Dcr"
  lappend usage {Temperature (T) is in Kelvin.}
  lappend usage {Donor concentration (Donor) is assumed to be the alagator field Donor and dopant type As.}
  lappend usage {Accept concentration (Accept) is assumed to be the alagator field Accept.}
  lappend usage {The perpendicular driving force (Fperp) defaults to the perpendicular electric field (Eperp).}
  lappend usage {The interface decay factor (Dcr) defaults to Dcr.}
  set returns {A list of two alagator expressions, the first item is the electron mobility and the second is the hole mobility }
  set description {Surface acoustic phonon model.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set expand [__Contains $args expand]
  set Accept [__GetValue $args Accept ]
  if {$Accept == ""} { set Accept Accept }
  set Donor [__GetValue $args Donor ]
  if {$Donor == ""} { set Donor Donor }
  set Fperp [__GetValue $args Fperp ]
  if {$Fperp == ""} { set Fperp Eperp }
  set Dcr [__GetValue $args Dcr ]
  if {$Dcr == ""} { set Dcr Dcr }
   
   set Mobs ""
   # acoustic surface phonon scattering electrons
   set B 4.75e7
   set C 5.80e2
   set lamb 0.125
   lappend Mobs "($B / $Fperp) + ($C * (($Accept+$Donor)^$lamb) / ($Fperp^0.3333)*($T/300.0))"
   
   # acoustic surface phonon scattering hole
   set B 9.925e6
   set C 2.947e3
   set lamb 0.0317
   lappend Mobs "($B / $Fperp) + ($C * (($Accept+$Donor)^$lamb) / ($Fperp^0.3333)*($T/300.0))"
   return $Mobs
}
proc SurfaceRoughness args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Fperp=Eperp Dcr=Dcr"
  lappend usage {The perpendicular driving force (Fperp) defaults to the perpendicular electric field (Eperp).}
  lappend usage {The interface decay factor (Dcr) defaults to Dcr.}
  set returns {A list of two alagator expressions, the first item is the electron mobility and the second is the hole mobility }
  set description {Surface roughness mobility model.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set Fperp [__GetValue $args Fperp ]
  if {$Fperp == ""} { set Fperp Eperp }
  set Dcr [__GetValue $args Dcr ]
  if {$Dcr == ""} { set Dcr Dcr }

  set Mobs ""
  # elec surface roughness scattering
  set del 5.82e14 
  set eta 5.82e30
  lappend Mobs  "(1. / (($Fperp*$Fperp)/$del + ($Fperp*$Fperp*$Fperp)/$eta) / ($Dcr))"
  # hole surface roughness scattering
  set del 2.0546e14 
  set eta 2.0546e30
  lappend Mobs "(1. / (($Fperp*$Fperp)/$del + ($Fperp*$Fperp*$Fperp)/$eta) / Dcr)"
  return $Mobs
}
}

