Grid2D

set Temp 300.
window col= 1 width=1000 height=1000

source ../../../procs.tcl

proc nisquared_Nmaj { Nmaj Nmin StressVector6 StressRotation CrystalRotation } {
  
  source ../../constants.tcl
  set Econdlist [XBandEdgeShiftStress $StressVector6 $StressRotation $CrystalRotation] Silicon
  set ml  [ expr 0.97 ]
  set mt1 [ expr 0.19 ]
  set mt2 [ expr 0.19 ]
  # use two fold degeneracy
  set mn  [ expr 2.**(2./3.) * ($ml*$mt1*$mt2)**(1./3.) * $m0 ]
  set Nc  [ expr 2*( ( $mn*$k*$T )/( 2*$pi*$hbar**2 ) )**1.5 /100**3 ] ;# 1/cm^3
  set Elec  [ expr "$Nc * ( exp(-[lindex $Econdlist 0]/$Vt) + exp (-[lindex $Econdlist 1]/$Vt) + exp(-[lindex $Econdlist 2]/$Vt) ) " ]
  
  set kp    [GetKP $StressVector6 $StressRotation $CrystalRotation ]
  set kp_LH_N [lindex [ValanceBandDOSFactorStress $kp] 0]
  set kp_HH_N [lindex [ValanceBandDOSFactorStress $kp] 1]
  chart graph= "2" curve= LH xval= [expr [lindex $StressVector6 0] /1.0e6] yval= [expr ($kp_LH_N-1.0)*100]
  chart graph= "2" curve= HH xval= [expr [lindex $StressVector6 0] /1.0e6] yval= [expr ($kp_HH_N-1.0)*100]
  set kp_LH_E [lindex [ValanceBandEdgeShiftStress $kp] 0]
  set kp_HH_E [lindex [ValanceBandEdgeShiftStress $kp] 1]
  set Ev_hh $kp_HH_E
  set Ev_lh $kp_LH_E
  
  # calculate Nv
  set mp_lh [ expr 0.16 ]
  set mp_hh [ expr 0.7618 ]
  set Nv_lh [ expr 2*( ( $m0*$mp_lh*$k*$T )/( 2*$pi*$hbar**2) )**1.5 /100**3 ] ;# 1/cm^3
  set Nv_hh [ expr 2*( ( $m0*$mp_hh*$k*$T )/( 2*$pi*$hbar**2) )**1.5 /100**3 ] ;# 1/cm^3
  
  set Hole  [ expr "$kp_LH_N*$Nv_lh*exp($Ev_lh/$Vt) + $kp_HH_N*$Nv_hh*exp($Ev_hh/$Vt)" ]
  #puts $Elec; puts $Hole; gets stdin
  #return [ expr $Elec*$Hole/$Nmaj*[ F12 $Eg ] ]
  return [ expr $Elec*$Hole/$Nmaj*exp(1.1/$Vt) ]
  
}


proc KandaElec { StressVector6 StressRotation CrystalRotation } {
  global ModelsDirectory
  source $ModelsDirectory/constants.tcl
  #set some very simple equations
  
  set alpha [fullalpha [expr [lindex $StressRotation 0]*acos(-1.0)/180. ] [expr [lindex $StressRotation 1]*acos(-1.0)/180. ] [expr [lindex $StressRotation 2]*acos(-1.0)/180. ]]
  vexpr { Stress = alpha*StressVector6 }
  set dc [ DSigma_from_Stress&Pi [ fullnpi_prime [expr [lindex $CrystalRotation 0]*acos(-1.0)/180. ] [expr [lindex $CrystalRotation 1]*acos(-1.0)/180. ] [expr [lindex $CrystalRotation 2]*acos(-1.0)/180. ] ] $Stress ]
  return $dc
}
proc KandaHole { StressVector6 StressRotation CrystalRotation } {
  global ModelsDirectory
  source $ModelsDirectory/constants.tcl
  #set some very simple equations
  
  set alpha [fullalpha [expr [lindex $StressRotation 0]*acos(-1.0)/180. ] [expr [lindex $StressRotation 1]*acos(-1.0)/180. ] [expr [lindex $StressRotation 2]*acos(-1.0)/180. ]]
  vexpr { Stress = alpha*StressVector6 }
  set dc [ DSigma_from_Stress&Pi [ fullnpi_prime [expr [lindex $CrystalRotation 0]*acos(-1.0)/180. ] [expr [lindex $CrystalRotation 1]*acos(-1.0)/180. ] [expr [lindex $CrystalRotation 2]*acos(-1.0)/180. ] ] $Stress ]
  set dc [ DSigma_from_Stress&Pi [ fullppi_prime [expr [lindex $CrystalRotation 0]*acos(-1.0)/180. ] [expr [lindex $CrystalRotation 1]*acos(-1.0)/180. ] [expr [lindex $CrystalRotation 2]*acos(-1.0)/180. ] ] $Stress ]
  # The tesnor format is
  # [lindex $dc 0 0], [lindex $dc 0 1], [lindex $dc 0 2]
  # [lindex $dc 1 0], [lindex $dc 1 1], [lindex $dc 1 2]
  # [lindex $dc 2 0], [lindex $dc 2 1], [lindex $dc 2 2]
  return $dc
}

set Nmaj  1e16
set Nmin  1e15
set StressVector6   "0. 0. 0. 0. 0. 0."
set StressRotation  "0. 0. 0."
set CrystalRotation "0. 0. 0."

set ni0squared_Nmaj [nisquared_Nmaj $Nmaj $Nmin $StressVector6 $StressRotation $CrystalRotation ]


set fil [ open 3D.csv w ]

#puts $fil "Vertical,Biaxial,IcChange_By_ni"
for { set V -200e6 } { $V < 201e6 } {set V [expr $V+10e6] } {
  for { set Bi -300e6 } { $Bi < 301e6 } {set Bi [expr $Bi+10e6] } {
    set StressVector6   "$Bi $V $Bi 0. 0. 0."
    set StressVector6   "$V $Bi $Bi 0. 0. 0."
    set delta [ expr ([nisquared_Nmaj $Nmaj $Nmin $StressVector6 $StressRotation $CrystalRotation ]/$ni0squared_Nmaj - 1.)*100. ]
    
    puts $fil "$V $Bi $delta"
    
  }
}

close $fil

gets stdin


