
proc __GetStateHash {} {
  options !echo
  set state "[pdbDump]\n"
  foreach c [contact list] {
    if { [string first "Reflect" $c] == -1 } {
      set state "$state $c=[contact get_supply name= $c]"
    }
  }
  foreach s [solution list] { 
    set state "$state $s=[solution name=$s print]"
  }
  foreach m [mater list] {
    foreach s [solution list] { 
      set state "$state $s\_$m=[solution $m name=$s print]"
    }
  }
  options echo
  # This produces a new has everytime... probably has to do with machine noise
  # set state "$state [fs md5]"
  return [md5 $state]
}

proc ClearCache args {
  set name [lindex [info level 0] 0]
  lappend usage "$name"
  set returns "nothing"
  set description "Clears the cache, just runs rm -f structures/cache/*"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  exec sh -c "rm -f structures/cache/*"
}

proc GetContactSupplies args {
  set name [lindex [info level 0] 0]
  lappend usage "$name"
  set returns "list of contact spplies"
  set description "gets contact supplies in list format"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set supplies ""
  foreach c [contact list] {
    if { [string first "Reflect" $c] == -1 } {
      # 5 orders of precision for bias voltages
      lappend supplies [expr round([contact get_supply name= $c]*1e5)/1e5]
    }
  }
  return $supplies;
}

proc SetContactSupplies args {
  set name [lindex [info level 0] 0]
  lappend usage "$name list_of_contact_supplies"
  set returns "nothing"
  set description "sets contact supplies by list"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  lassign $args supplies
  set i 0
  foreach c [contact list] {
    if { [string first "Reflect" $c] == -1 } {
      set s [lindex $supplies $i]
      incr i
      contact supply= $s name= $c
    }
  }
}

namespace eval Cache {
  set Bool 0
}

proc device_cache args {
  set name [lindex [info level 0] 0]
  lappend usage "$name ?init? n=? ?norunonexists?"
  set returns "nothing"
  set description "runs a cached device solve with init optional and n= specifies the number of device commands to run"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  
  set norunonexists [__Contains $args norunonexists]
  set init [__Contains $args init]
  set n [__GetValue $args n]
  if { n == "" } { set n 0 }
  file mkdir structures/cache
  set fil structures/cache/[__GetStateHash].str
  if { [file exists $fil] } {
    set s [GetContactSupplies]
    struct nodedatain= $fil
    SetContactSupplies $s
    
    if { !$norunonexists } {
      if {$init} {
        set status [catch {device init}]
      } else {
        set status [catch device]
      }
    }
    
    return 0
  } else {
    set status 0
    if {$init} {
      set status [catch {device init}]
    } else {
      set status [catch device]
    }
    if { !$status } {
      for { set i 1 } { $i < $n } { incr i } { set status [catch device]; }
      if { $Cache::Bool && !$status } {
        struct nodedataout= $fil
      }
    }
    return $status
  }
}


