
proc __mult_sweep [info args __linear_sweep] [string map {$i-$step $i/$step $i+$step $i*$step $step/2. ($step-1)/2.+1} [info body __linear_sweep]]

proc __bias_end { biases end } {
  return [expr [lindex $biasses 0 [llength $biases]-1]-$end < 1e-14 ]
}

proc __merge_fxn_lists { l1 l2 } {
  if { [llength $l1] != [llength $l2] } { 
    puts "failed to merge lists"
    exit
  } 
  set dat ""
  for { set i 0 } { $i < [llength $l1] } { incr i } {
    set tmp [concat [lindex $l1 $i] [lindex $l2 $i]]
    lappend dat $tmp
  }
  return $dat
}

proc FEQF_continuation args {
  set name [lindex [info level 0] 0]
  lappend usage "$name contact=s start=v end=v step=end-start fxn=optional_post_run_function n=1"
  set returns "a list of xy values where the x values are the values that were pamped and the y values are what was returned from the optional fxn function."
  set description "Does a linear ramp of a contact switching to current source when needed and calls an optional function on each iteration"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set contact [__GetValue $args contact "$name Error: No contact name provided"]
  set start [__GetValue $args start "$name Error: No start value provided"]
  set end [__GetValue $args end "$name Error: No end value provided"]
  set step [__GetValue $args step]
  if { $step == "" } { set step [expr $end-$start] }
  set fxn [__GetValue $args fxn]
  set n [__GetValue $args n]
  if { $n == "" } { set n 1 }
  

  global __FEQF_continuation_prev_cur __FEQF_continuation_prev_bias
  set __FEQF_continuation_prev_cur  0
  set __FEQF_continuation_prev_bias 0

  ##### RAMP PROCS ######
  set tmp "set contact $contact;"
  proc fxn_loc { start bias } [concat $tmp {
    global __FEQF_continuation_prev_cur __FEQF_continuation_prev_bias
    set cur [ expr abs([contact name=$contact sol=Qfn flux] - [contact name=$contact sol=Qfp flux])]
    if { abs([contact name=$contact sol=Qfn flux]) > abs([contact name=$contact sol=Qfp flux]) } {
      set cont_type Qfn
    } else {
      set cont_type Qfp
    }
    set bias [contact name=$contact avgval= $cont_type]
    set deltabias [expr ($bias-[set __FEQF_continuation_prev_bias])]
    set ret 0
    if {abs($bias-$start) > 1e-10 && abs($deltabias) > 1e-15} {
      # set cond [expr ($cur-[set __FEQF_continuation_prev_cur])/$deltabias]
      set cond_logcur [expr abs((log10($cur)-log10([set __FEQF_continuation_prev_cur]))/$deltabias)]
      # chart graph= 1 curve= 1 xval= $bias yval= $cond_logcur
      # chart graph= 2 curve= 1 xval= $bias yval= log10($cur)
      if { $cur > 1e-12 && $cond_logcur > 2.5 } { set ret 1 }
    }
    set __FEQF_continuation_prev_bias $bias
    set __FEQF_continuation_prev_cur  $cur
    return $ret
  }]
  proc current_bias {contact supply} {
    contact name= $contact supply= -$supply
  }
  proc current_check {start bias} {
    return [expr ![fxn_loc $start $bias]]
  }
  ##### END RAMP PROCS ######

  set cont_type Qfn
  set dat {{} {}}
  set timeout 10
  while { $timeout && abs([contact name=$contact avgval= $cont_type]-$end) > 1e-10 } {
    # voltage ramp
    if { $fxn != "" } {
      proc __fxn { trash } "set bias \[contact name=$contact avgval= $cont_type\]; $fxn \$bias"
    } else { proc __fxn { trash } {} }
    set dat [__merge_fxn_lists $dat [linear_ramp contact=$contact start=$start end=$end step=$step continue_check_fxn= "fxn_loc $start" fxn=__fxn]]
    if { abs([contact name=$contact avgval= $cont_type]-$end) < 1e-10 } { break }

    # current ramp
    if { abs([contact name=$contact sol=Qfn flux]) > abs([contact name=$contact sol=Qfp flux]) } {
      set cont_type Qfn
    } else {
      set cont_type Qfp
    }
    contact name= $contact supply= [expr [contact name=$contact sol=$cont_type flux]*1.001]
    FEQFOhmic T= 300 name= $contact current
    if { [device_cache init n=2] } {
      puts "Initial current bias failed"
      exit
    }

    set dat [__merge_fxn_lists $dat [__mult_sweep [expr abs([contact name=$contact sol=$cont_type flux])*1.001] 1e100 1.2 "current_bias $contact" __fxn 1 "current_check $start"]]

    # reset to voltage ramp
    FEQFOhmic T= 300 name= $contact
    set start [contact name=$contact avgval= $cont_type]
    contact name= $contact supply= $start
    device init

    incr timeout -1
  }
  if { !$timeout } {
    puts "Error timeout"
    exit 1
  }
  
  set __FEQF_continuation_prev_bias 0
  set __FEQF_continuation_prev_cur  0
  return $dat
}


