
proc __linear_sweep { start end step preDevice plotAndStore {n 1} {continueCheckFxn ""} } {
  set x ""
  set y ""
  set recover 0
  set timeout 0
  # get the correct sign for the step
  set sign [expr ($end-$start)/abs($end-$start)]
  set step [expr $sign * abs($step)]
  # set max step end-start
  set step [expr abs($step) > abs($end-$start) ? $end-$start : $step]
  set step0 $step
  # don't run the device command if no plotting is being requested
  set norunonexists ""
  if { $plotAndStore == "" } {
    set norunonexists norunonexists
  }

  # start the sweep loop
  device store
  set last $start
  set ind 0
  for { set i $start } { $step < 0 ? ($i > $end+$step/100.) : ($i < $end+$step/100.) } { set i [ expr $i+$step ] } {
    # if { $ind && (($sign > 0) ? (abs($i) < abs($start)-1e-10) : (abs($i) > abs($start)+1e-10)) } {
    #   puts "Failed to increment to final voltage. Couldn't converge on start voltage. Voltage is $i step was $step and start voltage is $start."
    #   return "{$x} {$y}"
    # }
    set o [expr round(14-[Odr $i])] ;# order of the current bias
    set i [format "%.[expr $o]f" $i]
    {*}$preDevice $i
    if { ![device_cache n=$n $norunonexists] } {
      if { $continueCheckFxn != "" && [{*}$continueCheckFxn $i] } {
        return "{$x} {$y}"
      }
      set last $i
      device store
      lappend x $i
      if { $plotAndStore != "" } {
        lappend y [{*}$plotAndStore $i]
      }
      # allow the simulation to recover a step size after 3 successful
      # and if the value is divisible by step
      # if { abs($step-$step0) > 1e-10 && !(round($i*1e15)%round($step0*1e15)) && $recover > 3 }
      if { abs($step-$step0) > 1e-10 && $recover > 3 } {
        set step [expr $step*2.]
        set recover 0
        set timeout [expr $timeout-1]
      }
      incr recover
    } else {
      # I am in here because I need to cut the step back
      device restore
      set i [expr $i-$step]
      set step [expr $step/2.]
      incr timeout
      set recover 0
      if { [Odr $step] < -$o } {
        puts "Failed to increment to final voltage. Failed on $i with step size $step"
        return "{$x} {$y}"
      }
    }
    incr ind
  }
  # undo last for loop iteration
  set i [ expr $i-$step ]
  # If we are not at the end then solve the end
  if { abs($i-$end) > abs($step/10.) } {
    eval "$preDevice $end"
    device_cache
    lappend x $end
    if { $plotAndStore != "" } {
      lappend y [{*}$plotAndStore $end]
    }
  }
  return "{$x} {$y}"
}


proc linear_ramp args {
  set name [lindex [info level 0] 0]
  lappend usage "$name contact=s start=v end=v step=end-start fxn=optional_post_run_function prefxn=optional_pre_run_function n=1"
  set returns "a list of xy values where the x values are the values that were pamped and the y values are what was returned from the optional fxn function."
  set description "Does a linear ramp of a contact and calls an optional function on each iteration"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set contact [__GetValue $args contact "$name Error: No contact name provided"]
  set start [__GetValue $args start "$name Error: No start value provided"]
  set end [__GetValue $args end "$name Error: No end value provided"]
  set step [__GetValue $args step]
  if { $step == "" } { set step [expr $end-$start] }
  set fxn [__GetValue $args fxn]
  set prefxn [__GetValue $args prefxn]
  set n [__GetValue $args n]
  if { $n == "" } { set n 1 }
  set continue_check_fxn [__GetValue $args continue_check_fxn]

  proc __bias { contact prefxn supply } {
    set ret 0
    if { $prefxn != "" } {
      set ret [{*}$prefxn]
      if { $ret == "" } { set ret 0 }
    }
    foreach c $contact {
      contact name= $c supply = $supply
    }
    return $ret
  }
  return [__linear_sweep $start $end $step "__bias {$contact} {$prefxn}" $fxn $n $continue_check_fxn]
}


