# ################################
#  Use Proper Orthogonal Decomposition 
#  (POD) for initial guesses
# ##################################

# lets seperate all the POD variables
namespace eval POD {
  set SolQfn ""
  set SolQfp ""
  set SolDevPsi ""
  set modesQfn ""
  set modesQfp ""
  set modesDevPsi ""
  set EqnsQfn ""
  set EqnsQfp ""
  set EqnsDevPsi ""
  set biases ""
  set prev ""
  set i 0
  # to do pod we need a set of solutions
  # this procedure adds a solution as a function of bias
  proc AddSolution { bias } {
   incr POD::i
   lappend POD::biases $bias
   sel !store z= Qfn name= Qfn$POD::i
   lappend POD::SolQfn Qfn$POD::i
   sel !store z= Qfp name= Qfp$POD::i
   lappend POD::SolQfp Qfp$POD::i
   sel !store z= DevPsi name= DevPsi$POD::i
   lappend POD::SolDevPsi DevPsi$POD::i
  }
  # Next we need to compute the POD modes, fit the data
  # and build the alagator equations for each solution pde
  proc ComputeModesFull {} {
    foreach pde "Qfn Qfp DevPsi" {
      set POD::modes$pde [pod mode_name= Mode$pde data= [set POD::Sol$pde]]
      set params [pod data= [set POD::Sol$pde] bias= $POD::biases modes= [set POD::modes$pde] ]
      set POD::Eqns$pde ""
      for { set i 0 } { $i < [llength $params]-1} { incr i 2 } {
        set e "[lindex $params $i+1] * \$bias + [lindex $params $i]"
        lappend POD::Eqns$pde $e
      }
      set i 0
      foreach bias $POD::biases {
        set lin "0"
        for {set j 0} { $j < [llength [set POD::modes$pde]] } { incr j } {
          set m [lindex [set POD::modes$pde] $j]
          set p [expr [lindex [set POD::Eqns$pde] $j]]
          set lin "$lin+$p*$m"
        }
        # sel z= (($lin)-[lindex [set POD::Sol$pde] $i])
        # plot1d graph= 3 name= $bias
        incr i
      }
    }
  }
  # only take a subset of modes with this proc for ramping purposes
  proc ComputeModes {} {
    foreach pde "Qfn Qfp DevPsi" {
      set subset_sol ""
      set subset_bias ""
      set numSol [llength [set POD::Sol$pde]]
      set maxNumModes 8
      # compute on most recent set of solutions
      for { set i [expr ($numSol>$maxNumModes?($numSol-$maxNumModes):0)] } { $i < $numSol } { incr i } {
        lappend subset_sol [lindex [set POD::Sol$pde] $i]
        lappend subset_bias [lindex $POD::biases $i]
      }
      set POD::modes$pde [pod mode_name= Mode$pde data= $subset_sol]
      set params [pod data= $subset_sol bias= $subset_bias modes= [set POD::modes$pde] ]
      set POD::Eqns$pde ""
      for { set i 0 } { $i < [llength $params]-1} { incr i 2 } {
        set e "[lindex $params $i+1] * \$bias + [lindex $params $i]"
        lappend POD::Eqns$pde $e
      }
      set i 0
      foreach bias $subset_bias {
        set lin "0"
        for {set j 0} { $j < [llength [set POD::modes$pde]] } { incr j } {
          set m [lindex [set POD::modes$pde] $j]
          set p [expr [lindex [set POD::Eqns$pde] $j]]
          set lin "$lin+$p*$m"
        }
        incr i
      }
    }
  }
  # lastly we can make guesses using the POD equations for each solution
  proc Init { bias } {
    foreach pde "Qfn Qfp DevPsi" {
      set lin "0"
      for {set j 0} { $j < [llength [set POD::modes$pde]] } { incr j } {
        set m [lindex [set POD::modes$pde] $j]
        set p [expr [lindex [set POD::Eqns$pde] $j]]
        set lin "$lin+$p*$m"
      }
      sel z= $lin name= $pde
    }
  }
  proc Callback { fxn bias } {
    # add the solution and compute the modes each bias step
    POD::AddSolution $bias
    if { [llength $POD::biases] > 2 } {
      POD::ComputeModes
    }
    $fxn $bias
  }
  proc nop args {}
}


proc FEQF_POD_linear_ramp args {
  set name [lindex [info level 0] 0]
  lappend usage "$name contact=s start=v end=v step=end-start fxn=optional_post_run_function prefxn=optional_pre_run_function n=1"
  set returns "a list of xy values where the x values are the values that were pamped and the y values are what was returned from the optional fxn function."
  set description "Does a linear ramp of a contact and calls an optional function on each iteration"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set contact [__GetValue $args contact "$name Error: No contact name provided"]
  set start [__GetValue $args start "$name Error: No start value provided"]
  set end [__GetValue $args end "$name Error: No end value provided"]
  set step [__GetValue $args step]
  if { $step == "" } { set step [expr $end-$start] }
  set fxn [__GetValue $args fxn]
  if { $fxn == "" } {
    set fxn POD::nop
  }
  set prefxn [__GetValue $args prefxn]
  set n [__GetValue $args n]
  if { $n == "" } { set n 1 }
  set continue_check_fxn [__GetValue $args continue_check_fxn]

  proc __bias { contact prefxn supply } {
    set ret 0
    if { [llength $POD::biases] > 2 && ( $POD::prev == "" || $supply > $POD::prev ) } {
      POD::Init $supply
      set POD::prev $supply
    }
    if { $prefxn != "" } {
      set ret [{*}$prefxn]
      if { $ret == "" } { set ret 0 }
    }
    foreach c $contact {
      contact name= $c supply = $supply
    }
    return $ret
  }
  return [__linear_sweep $start $end $step "__bias {$contact} {$prefxn}" "POD::Callback $fxn" $n $continue_check_fxn]
}


