
proc __interpolate_linear { x y xv } {
  for { set i 0 } { $i < ([llength $x]-1) } { incr i } {
    if { [lindex $x $i] > $xv } { break }
  }
  set x0 [lindex $x $i-1]
  set x1 [lindex $x $i] 
  set dx [expr $x1-$x0]
  set y0 [lindex $y $i-1]
  set y1 [lindex $y $i] 
  return [expr ($x1-$xv)/$dx*$y0+($xv-$x0)/$dx*$y1]
}

# Taken from the cubic spline code in this directory

proc __SolveTridiag {N_sub N_diag N_sup N_b n} {
    upvar 1 $N_sub sub
    upvar 1 $N_diag diag
    upvar 1 $N_sup sup
    upvar 1 $N_b b

    # Factorization and forward substitution
    for {set i 2} {$i <= $n} {incr i} {
        set i0 [expr {$i - 1}]
        set sub($i) [expr {$sub($i) / $diag($i0)}]
        set diag($i) [expr {$diag($i) - $sub($i) * $sup($i0)}]
        set b($i) [expr {$b($i) - $sub($i) * $b($i0)}]
    }
    set b($n) [expr {$b($n) / $diag($n)}]
    for {set i [expr {$n - 1}]} {$i >= 1} {incr i -1} {
        set i2 [expr {$i + 1}]
        set b($i) [expr {($b($i) - $sup($i) * $b($i2)) / $diag($i)}]
    }
}

proc __interpolate_cubic { x y xv } {
  set np [expr [llength $x]]
  if {$np <= 1} return

  for {set i 0} { $i < [llength $x] } { incr i } {
      set X($i) [lindex $x $i]
      set Y($i) [lindex $y $i]
  }
  
  for {set i 1; set last $X(0)} {$i < $np} {set last $X($i); incr i} {
      set h($i) [expr {double($X($i) - $last)}]
      if {$h($i) == 0} return
  }
  
  if {$np > 2} {
      for {set i 1} {$i < $np-1} {incr i} {
          set i2 [expr {$i + 1}]
          set i0 [expr {$i - 1}]
          set diag($i) [expr {($h($i) + $h($i2))/3.0}]
          set sup($i) [expr {$h($i2) / 6.0}]
          set sub($i) [expr {$h($i) / 6.0}]
          set a($i) [expr {($Y($i2) - $Y($i))/$h($i2) - \
                            ($Y($i) - $Y($i0)) / $h($i)}]
      }
      __SolveTridiag sub diag sup a [expr {$np - 2}]
  }
  set a(0) [set a([expr {$np - 1}]) 0]
        
  # Now interpolate
  for {set i 1} {$i < $np} {incr i} {
      set i0 [expr {$i - 1}]
      set dx  [expr  $xv - [lindex $x $i0]]
      if { [lindex $x $i] < $xv } { continue }
      set t1 [expr $dx]
      set t2 [expr {$h($i) - $t1}]
      set y [expr {((-$a($i0)/6 * ($t2 + $h($i)) * $t1 + \
                      $Y($i0))* $t2 + (-$a($i)/6 * \
                     ($t1+$h($i)) * $t2 + $Y($i)) * $t1)/$h($i)}]
      return $y
  }
}
proc interpolate_xy args {
  set name [lindex [info level 0] 0]
  lappend usage "$name x=? y=? xv=? type=cubic"
  lappend usage "A list of x coordinates (x) is required"
  lappend usage "A list of y coordinates (y) is required"
  lappend usage "An a x value to interpolate (xv) is required"
  lappend usage "The type of interpolation is either cubic or linear, cubic is the default"
  set returns "Interpolated value"
  set description "Cubic or linear interpolation of xy data of a single value"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set x [__GetValue $args x "$name error: no list of x coordinates provided"]
  set y [__GetValue $args y "$name error: no list of y coordinates provided"]
  set xv [__GetValue $args xv "$name error: x value provided"]
  set type [__GetValue $args type]
  if {$type == ""} { set type cubic }
  if { [llength $x] != [llength $y] } { puts "$name error: x and y lengths are not the same"; exit; }
  if { [lindex $x 0]-1e-10 > $xv || [lindex $x [llength $x]-1]+1e-10 < $xv } { puts "$name error: x $x xv $xv outside interpolation space"; exit; }

  if { $type == "cubic" } { return [__interpolate_cubic $x $y $xv] }
  else { return [__interpolate_linear $x $y $xv] }

}

# cubic test
if { 0 } {
window
set x ""
set y ""
for { set i 0 } { $i < 5 } { incr i } {
  lappend x $i
  lappend y [expr $i*$i]
  chart graph= 0 curve= 0 xval= $i yval= $i*$i 
}
for { set i 0 } { $i < 41 } { incr i } {
  chart graph= 0 curve= 1 xval= $i/10. yval= [interpolate_xy x=$x y=$y xv=[expr $i/10.] ]
}
}


