
proc __grad { errorFxn params delta {nop ""} {mask ""} } {
  if { ! [ catch [ set y0 [ $errorFxn $params ] ] ] } {
    puts "Error in y0"
    exit
  }
  
  set derivs $params ;# do this to initialize array length
  for { set i 0 } { $i < [llength $params] } { incr i } {
    if { ( [llength $params] == [llength $mask] ) && ![lindex $mask $i] } {
      lset derivs $i 0
      continue;
    }
    # puts "\tComputing partial derivative for parameter $i"
    set params_delta $params
    lset params_delta $i [ expr [lindex $params $i]+$delta ]
    if { ! [ catch [ set y1 [ $errorFxn $params_delta ] ] ] } {
      puts "Error in y1"
      exit
    }
    lset derivs $i [ expr ($y1 - $y0)/($delta) ]
  }
  return "$y0 {$derivs}"
}

# for each parameter, if the partial derivative of the error function with respect to that parameter is positive, decrease the value of the parameter, if it is negative, increase the value of the parameter
# the magnitude of the parameter change is dependent on the learning rate and if it is too grate, it is reduced to the damp value
proc __updateParams { params learningRates grads damp {names ""} {mask ""} } {
  set damp_list [expr [llength $params] == [llength $damp]]
  
  for { set i 0 } { $i < [llength $params] } { incr i } {
    if { ([llength $mask] == [llength $params]) && ![lindex $mask $i] } { continue; }
    set p0 [lindex $params $i]
    set name $i
    if { [llength $params] == [llength $names] } {
      set name [lindex $names $i]
    }
    puts -nonewline "\tUpdating parameter $name: $p0 - [Rnd [lindex $learningRates $i]] * [Rnd [lindex $grads $i]]"
    set up [ expr [lindex $learningRates $i] * [lindex $grads $i] ]
    if { $damp_list } {
      if { $up > [lindex $damp $i] } { set up [lindex $damp $i] }
      if { $up < -[lindex $damp $i] } { set up -[lindex $damp $i] }
    } else {
      if { $up > $damp } { set up $damp }
      if { $up < -$damp } { set up -$damp }
    }
    lset params $i [ expr $p0 - $up ]
    puts " = [lindex $params $i]"
  }
  return $params
}

# updates the learning rate based on how much the previous parameters changed divided by the gradient difference between the two points
proc __updateLearningRates { params prevParams grads prevGrads } {
  for { set i 0 } { $i < [llength $params] } { incr i } {
    set gradDiff [expr [lindex $grads $i] - [lindex $prevGrads $i]]
    # set a baseline learning rate in case a parameter comes back
    if { abs($gradDiff) < 1e-100 } {
      lappend learningRates 1e-5
    } else {
      # lappend learningRates [ expr ([lindex $params $i] - [lindex $prevParams $i])*($gradDiff)/($gradDiff**2) ]
      lappend learningRates [ expr abs(([lindex $params $i] - [lindex $prevParams $i])*($gradDiff))/($gradDiff**2) ]
    }
  }
  return $learningRates
}

proc __optimize_legacy { errorFxn params iterations {damp 1e9} } {
  
  puts "Start Parameter Optimization"
  set initialStep 1e-2
  if { [pdbIsAvailable optimize initialStep] } {
    set initialStep [pdbGet optimize initialStep]
  }
  lassign [__grad $errorFxn $params $delta nop] y0 grads
  set learningRates ""
  for { set i 0 } { $i < [llength $params] } { incr i } {
    if { abs([lindex $grads $i]) < 1e-100 } {
      lappend learningRates 1e-5
      #lset grads $i $initialStep
    } else {
      lappend learningRates [expr $initialStep/[lindex $grads $i]]
    }
  }
  set grads0 $grads
  set params0 $params
  puts "Initial Parameters $params0"
  for { set i 0 } { $i < $iterations } { incr i } {
    set params [__updateParams $params $learningRates $grads $damp]
    puts "Iteration $i Updated Parameters $params"
    
    lassign [__grad $errorFxn $params $delta nop] y0 grads
    puts "Iteration $i Error: $y0"
    set learningRates [__updateLearningRates $params $params0 $grads $grads0]
    set params0 $params
    set grads0 $grads
  }
  set params [__updateParams $params $learningRates $grads $damp]
  puts "Final Error: [ $errorFxn $params ]"
  puts "Final Parameters $params"
  return $params
  
}










