#### LARS test procs ####
#expr ([lindex y_del $i] - $y0)/($delta)
# These are to optimization procs
proc __WT {num file_name} {
  set fo [open $file_name a]
  puts $fo "$num starts waiting"
  flush $fo
  after ${num}000
  puts $fo "waited for $num sec"
  close $fo
}

# job_ID: the ID number of the current job
proc __wait_srun {srun_name output} {
  set job_ID $::env(SLURM_JOB_ID)
  set of [open $output a]
  puts $of "# looking at job $job_ID"
  after 100    ;# wait to make sure that the srun steps finish submitting and will show up when we look for them
  
  set no_run 0
  
  while { $no_run < 5 } {
    if { ![catch {exec sh -c "sacct --jobs=$job_ID --format=jobname,jobid,uid,state | grep $srun_name | grep -cE 'RUNNING|PENDING' "} r_val ] } {
      puts $of "# there are still $r_val processes running"
      flush $of
      set no_run 0
    } else {
      puts $of "# nothing running $no_run"
      flush $of
      incr no_run
    }
    after 200
  }
  puts $of "# all sruns are done!"
  flush $of
}

# errorfxn: name of the error function that will be run
# errorPath: the path to the file containing the error function. It has be be sources
# params: the input parameters for the error function
# o_file: the path to the output file where the results from the error function will be stored
# print_str: the string that will be combined be combined with the error fucntion results and printed to the output file
proc __run_errorfxn { errorfxn params o_file print_str} {
  set of [open $o_file a]
  puts $of "# we are inside run_err_yi, with parameters: $params and print string: $print_str"
  flush $of
  # evaluate the passed in error function proc
  eval $errorfxn
  
  if { ! [ catch [ set y [ errorfxn $params ] ] ] } {
    puts $of "### error with running errorfxn ###"
    puts $of "exit"
  } else {
    puts $of "$print_str $y"
  }
  flush $of
}

# errorfxn: name of the error function that will be run
# errorPath: the path to the file containing the error function. It has be be sources
# params: the input parameters for the error function
# grad_type: the type of gradient that will be calculated (forwards, backwards, or central)
proc __grad_slurm { errorFxn params delta grad_type {mask ""} } {
  #set o_file ./srun_output.tcl
  file mkdir srun_outs
  set o_file [exec mktemp -p ./srun_outs srun_out_XXXX.tcl ]  ;#consider using this to make the output files unique for earh call ofthe fucntion
  #another option is: file tempfile ?nameVar? ?template?
  set of [open $o_file a]
  set step_name "srun_step" ;# used to label and identify the job steps that will be launched during the proc
  
  puts $of "## starting $grad_type gradient calculations\n# params: $params,\tstep_name: $step_name\n# errorFxn: $errorFxn"
  
  puts $of "# delta is $delta"
  
  # y0 with current parameters
  set y0_print_str "set y0"
  set errorfxn "proc errorfxn {[info args $errorFxn]} { [info body $errorFxn] }"
  set flooxs_string "__run_errorfxn {$errorfxn} {$params} $o_file {$y0_print_str}"
  set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS &>/dev/null"
  #  set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS"
  exec sh -c $sh_str &
  
  # find the y values after having changed each of the parameters
  for { set i 0 } { $i < [llength $params] } { incr i } {
    lappend y_dels 0
    lappend y_f_dels 0 ;# only used for central diff
    lappend y_b_dels 0 ;# only used for central diff
    if { ( [llength $params] == [llength $mask] ) && ![lindex $mask $i] } {
      continue;
    }
    puts $of "# \tComputing partial derivative for parameter $i"
    flush $of
    
    switch $grad_type {
      forward {
        set params_delta $params
        lset params_delta $i [ expr [lindex $params $i] + $delta ]
        set yi_print_str "lset y_dels $i"
        set flooxs_string "__run_errorfxn {$errorfxn} {$params_delta} $o_file {$yi_print_str}"
        set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS &>/dev/null"
        #    set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS"
        exec sh -c $sh_str &
      }
      backward {
        set params_delta $params
        lset params_delta $i [ expr [lindex $params $i] - $delta ]
        set yi_print_str "lset y_dels $i"
        set flooxs_string "__run_errorfxn {$errorfxn} {$params_delta} $o_file {$yi_print_str}"
        set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS &>/dev/null"
        #    set sh_str "echo '$flooxs_string' | srun -N 1 --job-name=$step_name $::FLOOXS"
        exec sh -c $sh_str &
        
      }
      central {
        # calculations must be done for both forward (f) and backward (b), gradient calculations
        set params_f_delta $params
        set params_b_delta $params
        lset params_f_delta $i [ expr [lindex $params $i] + $delta ]
        lset params_b_delta $i [ expr [lindex $params $i] - $delta ]
        set yi_f_print_str "lset y_f_dels $i"
        set yi_b_print_str "lset y_b_dels $i"
        set flooxs_f_string "__run_errorfxn {$errorfxn} {$params_f_delta} $o_file {$yi_f_print_str}"
        set flooxs_b_string "__run_errorfxn {$errorfxn} {$params_b_delta} $o_file {$yi_b_print_str}"
        set sh_f_str "echo '$flooxs_f_string' | srun -N 1 --job-name=$step_name $::FLOOXS &>/dev/null"
        set sh_b_str "echo '$flooxs_b_string' | srun -N 1 --job-name=$step_name $::FLOOXS &>/dev/null"
        exec sh -c $sh_f_str &
        exec sh -c $sh_b_str &
        # this could be done with only one exec if the tow sh_x_str string are combined and separated by ;, but this format is better for readability and debugging
      }
    }
  }
  
  __wait_srun $step_name $o_file
  # this switch statement can be replaced by only "source $o_file", but it is currently here for debugging
  switch $grad_type {
    forward {
      puts $of "# pre source y_dels:  $y_dels"
      source $o_file
      puts $o_file
      puts $of "# post source y_dels: $y_dels \n# y0: $y0"
    }
    backward {
      puts $of "# pre source y_dels:  $y_dels"
      source $o_file
      puts $of "# post source y_dels: $y_dels \n# y0: $y0"
    }
    central {
      puts $of "# pre source y_f_dels:  $y_f_dels"
      puts $of "# pre source y_b_dels:  $y_b_dels"
      source $o_file
      puts $of "# post source y_f_dels: $y_f_dels"
      puts $of "# post source y_b_dels: $y_b_dels \n# y0: $y0"
    }
  }
  file delete $o_file
  
  for { set i 0 } { $i < [llength $params] } { incr i } {
    if { ( [llength $params] == [llength $mask] ) && ![lindex $mask $i] } {
      lappend derivs 0
      continue;
    }
    switch $grad_type {
      forward {
        lappend derivs [ expr ([lindex $y_dels $i] - $y0)/($delta) ]
      }
      backward {
        lappend derivs [ expr ($y0 - [lindex $y_dels $i])/($delta) ]
      }
      central {
        lappend derivs [ expr ([lindex $y_f_dels $i] - [lindex $y_b_dels $i])/($delta) ]
      }
    }
  }
  
  puts $of "# the derivatives are: $derivs\n# the iteration error is $y0"
  flush $of
  set error $y0
  set grads $derivs
  set r_list [list $error $grads]
  return $r_list
}


# declares the error function, the initial input parameters, the number of iteration to be used, and the damping level
# { errorFxn params {options {}} }
proc optimize args {
  set name [lindex [info level 0] 0]
  lappend usage "$name errorFxn=? params=? initial_step=0.01 max_iterations=1000 rel_error=1e-2 damp=none mask=all names=none grad_type=forward h=1e-4"
  lappend usage "An error function (errorFxn) to optimize must be specified, this is a procedure accesable by the flooxs script that takes a parameter list as its only argument."
  lappend usage "The initial parameter list (params) are required as the initial condition of the optimization."
  lappend usage "The initial step size (initial_step) is the initial amount to change every parameter. This is a number that defaults to 0.01."
  lappend usage "After a max number of iterations (max_iterations) the optimization process stops. This is a number that defaults to 1000."
  lappend usage "After a certain relative error reduction (rel_error) the optimization process stops. This is a number that defaults to 0.01 or 1e-2."
  lappend usage "Every optimization step, the parameters can be damped by a certain value.  If a single value is passed then that damping value is applied to all parameters. If a damping value list equal in length to the parameter list length is passed in, each parameter is damped accordingly. No damping is enabled by default."
  lappend usage "Not all parameters of a parameter list need to be optimized with every optimize function call. A parameter mask (mask) is can be given as a list of ones and zeros for parameters that want to be optimized and not optimized. By default all parameters are optimized."
  lappend usage "For log file convenience, each parameter can be named. If no parameter names are provided then the index number of the parameter is used."
  lappend usage "The numeric differentiation type (grad_type) can be set to forward, backward, and central difference."
  lappend usage "The numeric differentiation approximation (h) defaults to 1e-4."
  set returns "An optimized parameter list."
  set description {Optimizes and error function (errorFxn) that takes parameters (params) with some options}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set errorFxn [__GetValue $args errorFxn "$name Error: No error function (errorFxn) was provided to optimize"]
  set params [__GetValue $args params "$name Error: No initial parameters (params) were provided to optimize"]
  set damp [__GetValue $args damp]
  if {$damp == ""} { set damp 1e9 }
  set initial_step [__GetValue $args initial_step]
  if {$initial_step == ""} { set initial_step 1e-2 }
  set max_iterations [__GetValue $args max_iterations]
  if {$max_iterations == ""} { set max_iterations 1000 }
  set rel_error [__GetValue $args rel_error]
  if {$rel_error == ""} { set rel_error 1e-2 }
  set names [__GetValue $args names]
  set mask [__GetValue $args mask]
  set grad_type [__GetValue $args grad_type]
  if {$grad_type == ""} { set grad_type forward }
  set delta [__GetValue $args h]
  if {$delta == ""} { set delta 1e-4 }
  
  puts "Start Parameter Optimization On Error Function $errorFxn"
  puts Options:
  puts "\trel_error reduction target is $rel_error"
  puts "\tmax_iterations is $max_iterations"
  puts "\tinitial_step is $initial_step"
  puts "\tgrad_type is $grad_type"
  puts "\tThe derivative denominator (h) is $delta"
  ## Detect names
  if { ([llength $params] == [llength $names]) } {
    puts "\tList of parameter names detected: $names"
  } else {
    if { [llength $names] > 1 } {
      puts "Invalid length for names got [llength $names] names and expected [llength $params]"
      exit
    }
    puts "\tParameters are not named with names"
  }
  ## Detect parameter mask
  proc ApplyMask { mask l } {
    set tmp ""
    for { set i 0 } { $i < [llength $mask] } { incr i } {
      if { [lindex $mask $i] } {
        lappend tmp [lindex $l $i]
      }
    }
    return $tmp
  }
  if { [expr [llength $params] == [llength $mask]] } {
    if { $names != "" } {
      puts "\tParameter mask detected, will optimize only [ApplyMask $mask $names]"
    } else {
      puts "\tParameter mask detected, will optimize according to $mask"
    }
  } else {
    puts "\tParameter mask not detected, optimizing all parameters"
  }
  ## Detect damping parameters
  set damp_list [expr ([llength $params] == [llength $damp])]
  if { $damp_list } {
    puts "\tList of damp parameters detected: $damp"
  } else {
    if { [llength $damp] > 1 } {
      puts "Invalid length for damp got [llength $damp] paramters and expected [llength $params]"
      exit
    }
    puts "\tAll parameters damped by damp $damp"
  }
  puts "Initial Parameters $params"
  
  flush stdout
  set grad __grad
  if { [info exists ::env(SLURM_JOB_ID)] } {
    set grad __grad_slurm
    puts "Batch job detected $::env(SLURM_JOB_ID)"
  }
  
  # get the gradient of each parameter at the current value
  lassign [$grad $errorFxn $params $delta $grad_type $mask] error grads
  
  # set the learning rate of each parameter, a larger gradient gives a smaller learning rate, since small changes would have a large effect
  set learningRates ""
  for { set i 0 } { $i < [llength $params] } { incr i } {
    if { abs([lindex $grads $i]) < 1e-100 } {
      lappend learningRates 1e-5
    } else {
      lappend learningRates [expr $initial_step/[lindex $grads $i]]
    }
  }
  
  set initial_error $error
  if { $initial_error < 1e-100 } {
    puts "Initial Error Is 0, initial parameters are perfect!"
    return $params
  }
  set grads0 $grads
  set params0 $params
  for { set i 0 } { $i < $max_iterations } { incr i } {
    puts "\nIteration #$i:"
    set params [__updateParams $params $learningRates $grads $damp $names $mask]
    puts -nonewline "Iteration $i Updated Parameters "
    for {set j 0} {$j < [llength $params]} {incr j} {
      set name ""
      if { [llength $params] == [llength $names] } {
        set name "[lindex $names $j]="
      }
      set p [lindex $params $j]
      puts -nonewline "$name[Rnd $p] "
    }
    puts ""
    puts "Iteration $i Abs Error $error Rel Error [format "%0.4e" [expr $error/$initial_error]]"
    if { $error/$initial_error < $rel_error } {
      puts ""
      puts "Rel Error Reduction Target of $rel_error Met"
      break
    }
    lassign [$grad $errorFxn $params $delta $grad_type $mask] error grads  ;# central gradient
    set learningRates [__updateLearningRates $params $params0 $grads $grads0]
    set params0 $params
    set grads0 $grads
  }
  if { $i == $max_iterations } {
    set params [__updateParams $params $learningRates $grads $damp]
    puts "Max number of iterations reached"
  }
  puts "\nFinal results"
  puts "Final Error: $error"
  puts "Final Parameters: $params"
  return $params
}









