
proc XBandEdgeShiftStrain args {
  set name [lindex [info level 0] 0]
  lappend usage "$name strain={StrainXX StrainYY StrainZZ StrainYZ StrainXZ StrainXY} miller=(001)<100> mat=Silicon"
  lappend usage "Strain vector is required to be in Voigt notation string list format."
  lappend usage "Miller indicies (miller) are in (hkl)<uvw> format and defaults to 001 wafers with 100 channels."
  lappend usage "Material (mat) is optional and defaults to germanium."
  set returns "list of 4 conduction band energy band level shifts in eV"
  set description { Uses deformation potentials to compute X valley shifts on the three principle axies }
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set strain [__GetValue $args strain]
  if { $strain == "" } {
    set strain {StrainXX StrainYY StrainZZ StrainYZ StrainXZ StrainXY}
  }
  set miller [__GetValue $args miller]
  if {$miller == ""} { set miller (001)<100> }
  set Material [__GetValue $args mat ]
  if {$Material == ""} { set Material Silicon }
  set Material Silicon

  if {[llength $strain] != 6} {
    puts "Bad strain vector length [llength $strain] in $name"
    exit
  }
  
  # We have L-Valleys along 111, -111, 1-11, 11-1, -1-11, 1-1-1, -11-1, -1-1-1

  if { [ pdbIsAvailable $Material LValley Xiu  ] } { set Xiu  [ pdbGet $Material LValley Xiu  ] } else {
    puts "Xiu constant not specified for material $Material"; exit;
  }
  set L111  $Xiu*(StrainXX+StrainYY+StrainZZ+StrainYZ+StrainXZ+StrainXY)
  set Lm111 $Xiu*(StrainXX+StrainYY+StrainZZ-StrainYZ+StrainXZ+StrainXY)
  set L1m11 $Xiu*(StrainXX+StrainYY+StrainZZ+StrainYZ-StrainXZ+StrainXY)
  set L11m1 $Xiu*(StrainXX+StrainYY+StrainZZ+StrainYZ+StrainXZ-StrainXY)
  set Econdlist ""
  lappend Econdlist $L111
  lappend Econdlist $Lm111
  lappend Econdlist $L1m11
  lappend Econdlist $L11m1

  return $Econdlist
  
}

proc __getLmobsStrain { Elecs Material {raw 0} } {
  
  set Strain {StrainXX StrainYY StrainZZ StrainYZ StrainXZ StrainXY}
  if { [ pdbIsAvailable $Material LTotalMass ] } { set LTotalMass [ pdbGet $Material LTotalMass ] } else {
    puts "L Band total electron mass (LTotalMass) not specifited for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material LTransverseMass  ] } { set LTransverseMass [ pdbGet $Material LTransverseMass ] } else {
    puts "L Band Transverse electron mass (LTransverseMass) not specifited for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material LLongitudinalMass  ] } { set LLongitudinalMass [ pdbGet $Material LLongitudinalMass ] } else {
    puts "L Band Longitudinal electron mass (LLongitudinalMass) not specifited for material $Material"; exit;
  }
  
  set Emob_long [ expr $LTotalMass/$LLongitudinalMass ]
  set Emob_tran [ expr $LTotalMass/$LTransverseMass ]
  
  return "\"$mobVecX\"\
          \"$mobVecY\"\
          \"$mobVecZ\""
}

