# alpha is in radians
proc __alphaInv {Rotation} {
  set R1 [expr -[lindex $Rotation 0] ]
  set R2 [expr -[lindex $Rotation 1] ]
  set R3 [expr -[lindex $Rotation 2] ]
  return [__alpha "$R1 $R2 $R3"]
}

proc __alpha {Rotation} {
  return [__xyzalpha $Rotation]
}

# returns the rotation matrix or alpha matrix for a symetric tensor
# Rotates about z axis then Y then x
proc __xyzalpha {XYZRotation} {
  set Xrot [expr [lindex $XYZRotation 0]*acos(-1.0)/180. ]
  set Yrot [expr [lindex $XYZRotation 1]*acos(-1.0)/180. ]
  set Zrot [expr [lindex $XYZRotation 2]*acos(-1.0)/180. ]
  # set the vector rotation matricies using the three axis rotation angles
  set m_psi   {{cos($Zrot) sin($Zrot) 0.0}
    {-sin($Zrot) cos($Zrot) 0.0}
  {0.0        0.0       1.0}}
  set m_theta {{cos($Yrot) 0.0 -sin($Yrot)}
    {0.0         1.0  0.0       }
  {sin($Yrot)  0.0  cos($Yrot)}}
  set m_phi   {{1.0       0.0       0.0}
    { 0.0       cos($Xrot) sin($Xrot)}
  { 0.0      -sin($Xrot) cos($Xrot)}}
  for {set i 0} {$i<3} {incr i} {
    for {set j 0} {$j<3} {incr j} {
      lset m_psi   $i $j [expr [lindex $m_psi $i $j]]
      lset m_theta $i $j [expr [lindex $m_theta $i $j]]
      lset m_phi   $i $j [expr [lindex $m_phi $i $j]]
    }
  }
  # multiple the three vector rotations to get the a matrix
  set a [__MultNxN $m_psi [__MultNxN $m_theta $m_phi 3] 3]
  return [__approxFirstToSecondOrderRotation $a]
}

proc __euleralpha {psi theta phi} {
  # set the vector rotation matricies using the three axis rotation angles
  set m_psi   {{cos($psi) sin($psi) 0.0}
    {-sin($psi) cos($psi) 0.0}
  {0.0        0.0       1.0}}
  set m_theta {{cos($theta) 0.0 -sin($theta)}
    {0.0          1.0  0.0        }
  {sin($theta)  0.0  cos($theta)}}
  set m_phi   {{cos($phi) sin($phi) 0.0}
    {-sin($phi) cos($phi) 0.0}
  {0.0       0.0      1.0}}
  for {set i 0} {$i<3} {incr i} {
    for {set j 0} {$j<3} {incr j} {
      lset m_psi $i $j [expr [lindex $m_psi $i $j]]
      lset m_theta $i $j [expr [lindex $m_theta $i $j]]
      lset m_phi $i $j [expr [lindex $m_phi $i $j]]
    }
  }
  # multiple the three vector rotations to get the a matrix
  set a [__MultNxN $m_psi [__MultNxN $m_theta $m_phi 6] 6]
  
  return [__approxFirstToSecondOrderRotation $a]
}

proc __approxFirstToSecondOrderRotation {a} {
  set debug 0
  # unpack the a matrix and apply the formula to get the alpha rotation tensor
  set alpha {{{[lindex $a 0 0]**2}     {[lindex $a 0 1]**2}     {[lindex $a 0 2]**2}     {2*[lindex $a 0 1]*[lindex $a 0 2]}             {2*[lindex $a 0 2]*[lindex $a 0 0]}             {2*[lindex $a 0 0]*[lindex $a 0 1]}             }
    {{[lindex $a 1 0]**2}     {[lindex $a 1 1]**2}     {[lindex $a 1 2]**2}     {2*[lindex $a 1 1]*[lindex $a 1 2]}             {2*[lindex $a 1 2]*[lindex $a 1 0]}             {2*[lindex $a 1 0]*[lindex $a 1 1]}             }
    {{[lindex $a 2 0]**2}     {[lindex $a 2 1]**2}     {[lindex $a 2 2]**2}     {2*[lindex $a 2 1]*[lindex $a 2 2]}             {2*[lindex $a 2 2]*[lindex $a 2 0]}             {2*[lindex $a 2 0]*[lindex $a 2 1]}             }
    {{[lindex $a 1 0]*[lindex $a 2 0]} {[lindex $a 1 1]*[lindex $a 2 1]} {[lindex $a 1 2]*[lindex $a 2 2]} {[lindex $a 1 1]*[lindex $a 2 2]+[lindex $a 2 1]*[lindex $a 1 2]} {[lindex $a 1 2]*[lindex $a 2 0]+[lindex $a 2 2]*[lindex $a 1 0]} {[lindex $a 1 1]*[lindex $a 2 0]+[lindex $a 2 1]*[lindex $a 1 0]} }
    {{[lindex $a 2 0]*[lindex $a 0 0]} {[lindex $a 2 1]*[lindex $a 0 1]} {[lindex $a 2 2]*[lindex $a 0 2]} {[lindex $a 2 1]*[lindex $a 0 2]+[lindex $a 0 1]*[lindex $a 2 2]} {[lindex $a 2 2]*[lindex $a 0 0]+[lindex $a 0 2]*[lindex $a 2 0]} {[lindex $a 2 1]*[lindex $a 0 0]+[lindex $a 0 1]*[lindex $a 2 0]} }
  {{[lindex $a 0 0]*[lindex $a 1 0]} {[lindex $a 0 1]*[lindex $a 1 1]} {[lindex $a 0 2]*[lindex $a 1 2]} {[lindex $a 0 1]*[lindex $a 1 2]+[lindex $a 1 1]*[lindex $a 0 2]} {[lindex $a 0 2]*[lindex $a 1 0]+[lindex $a 1 2]*[lindex $a 0 0]} {[lindex $a 0 1]*[lindex $a 1 0]+[lindex $a 1 1]*[lindex $a 0 0]} }}
  # set alpha as values not expressions
  for {set i 0} {$i<6} {incr i} {
    for {set j 0} {$j<6} {incr j} {
      lset alpha $i $j [expr [lindex $alpha $i $j]]
    }
  }
  if {$debug} {
    puts "Printing alpha"
    for {set i 0} {$i<6} {incr i} {
      puts "[lindex $alpha $i 0] [lindex $alpha $i 1] [lindex $alpha $i 2] [lindex $alpha $i 3] [lindex $alpha $i 4] [lindex $alpha $i 5]"
    }
  }
  # return the tensor rotation matrix
  return $alpha
}

# Rotate a 4th order tensor being represented by a matrix as a 4nd Order Tensor and return it
# This is no approximation
proc __rotTen_F_Mat {psi Mat} {
  set debug 0
  # set the vector rotation matrix
  set m_psi   {{cos($psi) sin($psi) 0.0}
    {-sin($psi) cos($psi) 0.0}
  {0.0        0.0       1.0}}
  
  for {set i 0} {$i<3} {incr i} {
    for {set j 0} {$j<3} {incr j} {
      lset m_psi $i $j [expr [lindex $m_psi $i $j]]
    }
  }
  set alpha $m_psi
  set Ten {}
  set Ten_new {}
  for {set i 0} {$i<3} {incr i} {
    for {set j 0} {$j<3} {incr j} {
      for {set k 0} {$k<3} {incr k} {
        for {set l 0} {$l<3} {incr l} {
          if { $i == 0 && $j == 0 } { set ind1 0 }
          if { $i == 1 && $j == 1 } { set ind1 1 }
          if { $i == 2 && $j == 2 } { set ind1 2 }
          if { ($i == 1 && $j == 2) || ($i == 2 && $j == 1) } { set ind1 3 }
          if { ($i == 0 && $j == 2) || ($i == 2 && $j == 0) } { set ind1 4 }
          if { ($i == 0 && $j == 1) || ($i == 0 && $j == 1) } { set ind1 5 }
          if { $k == 0 && $l == 0 } { set ind2 0 }
          if { $k == 1 && $l == 1 } { set ind2 1 }
          if { $k == 2 && $l == 2 } { set ind2 2 }
          if { ($k == 1 && $l == 2) || ($k == 2 && $l == 1) } { set ind2 3 }
          if { ($k == 0 && $l == 2) || ($k == 2 && $l == 0) } { set ind2 4 }
          if { ($k == 0 && $l == 1) || ($k == 0 && $l == 1) } { set ind2 5 }
          lset Ten $i $j $k $l [expr [lindex $Mat $ind1 $ind2]]
        }
      }
    }
  }
  for {set i 0} {$i<3} {incr i} {
    for {set j 0} {$j<3} {incr j} {
      for {set k 0} {$k<3} {incr k} {
        for {set l 0} {$l<3} {incr l} {
          for {set m 0} {$m<3} {incr m} {
            for {set n 0} {$n<3} {incr n} {
              for {set o 0} {$o<3} {incr o} {
                for {set p 0} {$p<3} {incr p} {
                  lset Ten_new $i $j $k $l [expr [lindex $Ten_new $i $j $k $l] + [lindex $alpha $i $m]*[lindex $alpha $j $n]*[lindex $alpha $k $o]*[lindex $alpha $l $p]*[lindex $Ten $m $n $o $p]]
                }
              }
            }
          }
        }
      }
    }
  }
  return $Ten_new
}

# This is an implimented strain projection from pg 306 of
# Bir and Pikus
# I think it is equivalent to only doing the inverse rotation matrix
proc __VoigtStrainToDPStrain {XYZRotation VoigtStrain} {
  set Xrot [expr [lindex $XYZRotation 0]*acos(-1.0)/180. ]
  set Yrot [expr [lindex $XYZRotation 1]*acos(-1.0)/180. ]
  set Zrot [expr [lindex $XYZRotation 2]*acos(-1.0)/180. ]
  # set the vector rotation matricies using the three axis rotation angles
  set m_psi   {{cos($Zrot) sin($Zrot) 0.0}
    {-sin($Zrot) cos($Zrot) 0.0}
  {0.0        0.0       1.0}}
  set m_theta {{cos($Yrot) 0.0 -sin($Yrot)}
    {0.0         1.0  0.0       }
  {sin($Yrot)  0.0  cos($Yrot)}}
  set m_phi   {{1.0       0.0       0.0}
    { 0.0       cos($Xrot) sin($Xrot)}
  { 0.0      -sin($Xrot) cos($Xrot)}}
  for {set i 0} {$i<3} {incr i} {
    for {set j 0} {$j<3} {incr j} {
      lset m_psi   $i $j [expr [lindex $m_psi $i $j]]
      lset m_theta $i $j [expr [lindex $m_theta $i $j]]
      lset m_phi   $i $j [expr [lindex $m_phi $i $j]]
    }
  }
  # multiple the three vector rotations to get the a matrix
  set a [__MultNxN $m_psi [__MultNxN $m_theta $m_phi 3] 3]
  proc dot { a i v } {
    set tmp 0
    for { set j 0 } { $j < 3 } { incr j } {
      set tmp [expr [lindex $a $i $j]*[lindex $v $j]+$tmp]
    }
    return $tmp
  }
  set initial {{0 0 0} {0 0 0} {0 0 0}}
  for { set i 0 } { $i < 3 } { incr i } {
    lset initial $i $i [lindex $VoigtStrain $i]
  }
  lset initial 1 2 [expr [lindex $VoigtStrain 3]/2.]
  lset initial 2 1 [expr [lindex $VoigtStrain 3]/2.]
  lset initial 0 2 [expr [lindex $VoigtStrain 4]/2.]
  lset initial 2 0 [expr [lindex $VoigtStrain 4]/2.]
  lset initial 0 1 [expr [lindex $VoigtStrain 5]/2.]
  lset initial 1 0 [expr [lindex $VoigtStrain 5]/2.]
  
  set out {{0 0 0} {0 0 0} {0 0 0}}
  for { set i 0 } { $i < 3 } { incr i } {
    set v_i "0 0 0"
    lset v_i $i 1
    for { set j 0 } { $j < 3 } { incr j } {
      set v_j "0 0 0"
      lset v_j $j 1
      for { set k 0 } { $k < 3 } { incr k } {
        set theta_i [expr acos([dot $a $k $v_i])]
        for { set l 0 } { $l < 3 } { incr l } {
          set theta_j [expr acos([dot $a $l $v_j])]
          set val [expr [lindex $out $i $j]+(cos($theta_i)*cos($theta_j))*[lindex $initial $k $l] ]
          lset out $i $j $val
        }
      }
    }
  }
  set VoigtOut {0 0 0 0 0 0}
  lset VoigtOut 0 [lindex $out 0 0]
  lset VoigtOut 1 [lindex $out 1 1]
  lset VoigtOut 2 [lindex $out 2 2]
  lset VoigtOut 3 [expr [lindex $out 1 2 ]*2.]
  lset VoigtOut 4 [expr [lindex $out 0 2 ]*2.]
  lset VoigtOut 5 [expr [lindex $out 0 1 ]*2.]
  return $VoigtOut
}
# puts [VoigtStrainToDPStrain "0 0 0" "1 1 1 1 1 1"]
# puts [VoigtStrainToDPStrain "0 0 45." "1 1 1 1 1 1"]








