
proc MaxExpr args {
  set ret "(([lindex $args 0])^20"
  for { set i 1 } { $i < [ llength $args ] } { incr i } {
    set ret "$ret+([lindex $args $i])^20"
  }
  set ret "$ret)^0.05"
}
proc MinExpr args {
  # if { [ llength $args ] == 1 } { return $args }
  # set ret "(([lindex $args 1]<[lindex $args 0])?[lindex $args 0]:[lindex $args 1])"
  # for { set i 1 } { $i < [ llength $args ] } { incr i } {
  #   set prev [lindex $args ($i-1)]
  #   set cur [lindex $args $i]
  #   set "$ret
  # }
  set ret "1./(((1./([lindex $args 0]))^20"
  for { set i 1 } { $i < [ llength $args ] } { incr i } {
    set ret "$ret+(1./([lindex $args $i]))^20"
  }
  set ret "$ret)^0.05)"
}

proc tcl::mathfunc::F12 args {
  set name [lindex [info level 0] 0]
  lappend usage "$name value"
  set returns "Fermi-Dirac integral of this value"
  set description {Uses Gaussian quadrature to integrate $1/(1+\exp(x))$ where 0 is the band edge, negative values are below the gap, and positive values are in the band. This function has the same behavior as the alagator expression, just written in tcl.}
  if { [llength $args] != 1 || [__Help $args $name $usage $returns $description] } {
    return ""
  }
  #all negative values only
  set ivalue [expr [lindex $args 0]]
  set fermi 1.
  if { $ivalue < 0 } {
    set termone   [ expr           exp(1.*$ivalue) ]
    set termtwo   [ expr -0.353568*exp(2.*$ivalue) ]
    set termthree [ expr  0.192439*exp(3.*$ivalue) ]
    set termfour  [ expr -0.122973*exp(4.*$ivalue) ]
    set termfive  [ expr  0.077134*exp(5.*$ivalue) ]
    set termsix   [ expr -0.036228*exp(6.*$ivalue) ]
    set termseven [ expr  0.008346*exp(7.*$ivalue) ]
    set fermi [ expr  $termone + $termtwo + $termthree + $termfour + $termfive + $termsix + $termseven ]
  } elseif { $ivalue > 0 } {
    #positive values
    #positive values greater than 4
    if { $ivalue > 4. } {
      set multiplier [ expr      $ivalue**(3.0/2.0) ]
      set tone       [ expr  0.752253               ]
      set ttwo       [ expr  0.928195/($ivalue**2.)  ]
      set tthree     [ expr  0.680839/($ivalue**4.)  ]
      set tfour      [ expr   25.7829/($ivalue**6.)  ]
      set tfive      [ expr  -553.636/($ivalue**8.)  ]
      set tsix       [ expr   3531.43/($ivalue**10.) ]
      set tseven     [ expr  -3254.65/($ivalue**12.) ]
      set fermi [ expr $multiplier*($tone+$ttwo+$tthree+$tfour+$tfive+$tsix+$tseven)]
    } elseif { $ivalue < 4. } {
      #positive values less than 4
      #positive values from 0 to 2
      if { $ivalue < 2. } {
        set term1 [ expr  0.765147*($ivalue**0.) ]
        set term2 [ expr  0.604911*($ivalue**1.) ]
        set term3 [ expr  0.189885*($ivalue**2.) ]
        set term4 [ expr  0.020307*($ivalue**3.) ]
        set term5 [ expr -0.004380*($ivalue**4.) ]
        set term6 [ expr -0.000366*($ivalue**5.) ]
        set term7 [ expr  0.000133*($ivalue**6.) ]
        set fermi [ expr  $term1 + $term2 + $term3 + $term4 + $term5 + $term6 + $term7 ]
      } elseif { $ivalue > 2. } {
        set t1 [ expr 0.777114*($ivalue**0.) ]
        set t2 [ expr 0.581307*($ivalue**1.) ]
        set t3 [ expr 0.206132*($ivalue**2.) ]
        set t4 [ expr 0.017680*($ivalue**3.) ]
        set t5 [ expr-0.006549*($ivalue**4.) ]
        set t6 [ expr 0.000784*($ivalue**5.) ]
        set t7 [ expr-0.000036*($ivalue**6.) ]
        set fermi [ expr $t1 + $t2 + $t3 + $t4 + $t5 + $t6 + $t7 ]
      }
    }
  }
  return $fermi
}

proc F12 {v} {
  return [tcl::mathfunc::F12 $v]
}


