#build an equation for Di-Interstitials
proc 2Defect { Mat Sol } {
    set pM [pdbName $Mat]

    #get the forming defect
    set Base [pdbGetString $pM $Sol BaseDefect]

    #compute the parameters for the forward reaction
    set K2f [pdbDelayDouble $pM $Sol Kforward]

    #get the reverse rate dependence
    set Bind [pdbDelayDouble $pM $Sol Bind]

    #add a term for the formation dissolution
    term add name = ${Sol}Form eqn = "$K2f * ( ($Base ^ 2) - $Bind * $Sol )"

    #create the di whatever equation
    set Di2 [pdbDelayDouble $pM $Sol D0]
    set de [pdbGetString $pM $Sol Equation]
    pdbSetString $pM $Sol Equation "$de + ddt(${Sol}) - ${Sol}Form - $Di2 * grad(${Sol})"

    #subtract the formation of these from the base defect type
    set de [pdbGetString $pM $Base Equation]
    pdbSetString $pM $Base Equation "$de + 2 * ${Sol}Form"

    #now work on a term for capture of the anti-defect
    set Recomb [pdbGetString $pM $Sol RecombDefect]

    if { $Recomb != 0 } {
	#compute the parameters for the recombination reaction
	set K2f [pdbDelayDouble $pM $Sol KRecomb]

	#work on getting equilibriums
	set Cstar [pdbDelayDouble $pM $Sol Cstar]
	set Crs [pdbDelayDouble $pM $Recomb Cstar]
	set Cbs [pdbDelayDouble $pM $Base Cstar]

	#create the term
	term add name = ${Sol}Recomb eqn = "$K2f * ( $Sol * $Recomb - $Crs * $Cstar * $Base / $Cbs )"

	#add it to the three involved equations
	set de [pdbGetString $pM $Sol Equation]
	pdbSetString $pM $Sol Equation "$de + ${Sol}Recomb"

	set de [pdbGetString $pM $Base Equation]
	pdbSetString $pM $Base Equation "$de - ${Sol}Recomb"

	set de [pdbGetString $pM $Recomb Equation]
	pdbSetString $pM $Recomb Equation "$de + ${Sol}Recomb"
    }

    #get the name of any molecular
    set Recomb [pdbGetString $pM $Sol BiMolecular]
    if { $Recomb != 0 } {
	#build a name to use and see if it is already defined...
	set tname ${Sol}${Recomb}Recomb

	#get the equilibrium concentration of the other things
	set REq [pdbDelayDouble $pM $Recomb Cstar]

	#get the recombination rate constant
	set K2f [pdbDelayDouble Si $Sol KBiMole]

	term name = $tname add eqn = "$K2f * ($Sol * $Recomb - $Cstar * $REq)"

	#subtract from the two equations
	set de [pdbGetString $pM $Sol Equation]
	pdbSetString $pM $Sol Equation "$de + $tname"

	set de [pdbGetString $pM $Recomb Equation]
	pdbSetString $pM $Recomb Equation "$de + $tname"
    }
}

proc 2BoundSide { Mat Side Sol } {
    #get the reverse rate dependence
    set Ci2 [pdbDelayDouble $Side $Sol Cstar]

    #get the forward term
    set Ksurf [pdbDelayDouble $Mat $Sol Ksurf]

    pdbSetString $Mat $Sol Equation_${Side} "- $Ksurf * (${Sol}_${Side} - $Ci2)"
}

proc 2Bound { Mat Sol } {
    set pdbMat [pdbName $Mat]

    set s1 [FirstMat $pdbMat]
    set s2 [SecondMat $pdbMat]

    if {[pdbIsAvailable $s1 $Sol EquationProc]} {
	2BoundSide $pdbMat $s1 $Sol
    }

    if {[pdbIsAvailable $s2 $Sol EquationProc]} {
	2BoundSide $pdbMat $s2 $Sol
    }
}

proc SmicEqn { Mat Sol } {
    set pM [pdbName $Mat]

    #get the forming defect
    set Base [pdbGetString $pM $Sol BaseDefect]

    #compute the parameters for the forward reaction
    set K2f [pdbDelayDouble $pM $Sol Kforward]

    #get the reverse rate dependence
    set Bind [pdbDelayDouble $pM $Sol Bind]

    #add a term for the formation / dissolution
    term add name = ${Sol}Form eqn = "$K2f * ( $Base * I2 - $Bind * $Sol )"

    #compute the rate of di-vacancy recombination
    set K2r [pdbDelayDouble $pM $Sol KRecomb]
    set Cstar [pdbDelayDouble $pM $Sol Cstar]
    set Cv2star [pdbDelayDouble $pM V2 Cstar]
    set Cbase [pdbDelayDouble $pM $Base Cstar]
    term add name = ${Sol}Recomb eqn = "$K2r * ($Sol * V2 - $Base * $Cstar * $Cv2star / $Cbase)"

    #create the equation
    set de [pdbGetString $pM $Sol Equation]
    pdbSetString $pM $Sol Equation "$de + ddt(${Sol}) - ${Sol}Form + ${Sol}Recomb"

    #subtract the formation of these from the base defect type
    set de [pdbGetString $pM I2 Equation]
    pdbSetString $pM I2 Equation "$de + ${Sol}Form"

    set de [pdbGetString $pM $Base Equation]
    pdbSetString $pM $Base Equation "$de + ${Sol}Form - ${Sol}Recomb"

    set de [pdbGetString $pM V2 Equation]
    pdbSetString $pM V2 Equation "$de + ${Sol}Recomb"
}

