proc DefectBulk { Mat Sol } {
    set pdbMat [pdbName $Mat]
 
    #get the base level and energy level contributions to Cstar
    set cis [pdbDelayDouble $pdbMat $Sol Cstar]
    set neu [pdbDelayDouble $pdbMat $Sol neutral]
    set neg [pdbDelayDouble $pdbMat $Sol negative]
    set dng [pdbDelayDouble $pdbMat $Sol dnegative]
    set pos [pdbDelayDouble $pdbMat $Sol positive]
    set dps [pdbDelayDouble $pdbMat $Sol dpositive]
    set den "($neu + $neg + $dng + $pos + $dps)"
    set num "($neu + Noni*($neg + Noni*$dng) + Poni*($pos + Poni*$dps))"
    term name = Eq${Sol} add eqn = "$cis * $num * Pressure${Sol} / $den" $Mat
    term name = Scale${Sol} add eqn = "$Sol/Eq${Sol}" $Mat

    #create bulk recombination - note:  Requires Int and Vac as names!
    #basic iv recombination
    set pM $pdbMat
    set Recomb [pdbGetString $pM $Sol RecombDefect]
    if { $Recomb != 0 } {
	set Kb [format {[DiffLimit %s {%s %s} 0.0]} $pM $Sol $Recomb]
	term name = IVRecomb add eqn = "$Kb * ($Sol * $Recomb - Eq$Sol * Eq$Recomb)" $Mat
    }
    #works for 1D stress for now..
    set terms [term list]
    if { [ lsearch $terms Pressure${Sol} ] == -1} {
	set sgn -1.0;
	if { $Sol == "Int" } {
	    set sgn 1.0;
	}
	#get the dimension of the structure
	set dim [ flps_grid qual dimension]
	if { $dim == 1.0 } {
	    set HPressure {-Stress_x}
	}
	if { $dim == 2.0 } {
	    set HPressure { -0.5*(Stress_x+Stress_y) }
	}
	if { $dim == 3.0 } {
	    set HPressure { -1.0/3.0*(Stress_x+Stress_y+Stress_z) }
	}
	
	set Vol [pdbDelayDouble $pdbMat $Sol Volume]
	set Kt {((Temp + 273.0) * 1.38062e-16)}
	set eq "exp(-1.0 *$sgn*( $HPressure * $Vol  ) / $Kt )"
	term name = Pressure${Sol} add eqn = $eq
    }
    #get the diffusivity - should be Fermi dependent some day!
    set D0 "[pdbDelayDouble $pdbMat $Sol D0] * Eq${Sol}"

    set eqn "ddt($Sol) - $D0 * grad(Scale$Sol) + IVRecomb"
    set de [pdbGetString $pdbMat $Sol Equation ]
    pdbSetString $pdbMat $Sol Equation "$de + $eqn"
}

proc DefectInit { Mat Sol } {
    set pdbMat [pdbName $Mat]
    pdbSetString $pdbMat $Sol Equation ""
}


proc DefectBoundSide { Mat Side Sol } {
    #Since we are on a boundary, we can't use the bulk defined terms

    #get the base level and energy level contributions to Cstar
    set Cs "Eq${Sol}_${Side}"
    set SS ${Sol}_${Side}

    #get the velocity scaling
    set vscale [pdbDelayDouble $Mat $Sol Scale]

    #get the recombination velocity scale factor
    set Kr "([pdbDelayDouble $Mat $Sol Krat] * Velocity_mag / $vscale + 1)"

    #recombination expressions
    set Ks "[pdbDelayDouble $Mat $Sol Ksurf] * ($SS - $Cs)"
    set KsI "$Kr * $Ks"

    #velocity dependent generation terms
    set vm [pdbDelayDouble $Side LatticeDensity]
    set theta [pdbDelayDouble $Mat $Sol theta]
    set Gen "$vm * $theta"

    #set trapping equation
    set Ktrap 0.0
    set Kt [pdbDelayDouble $Mat $Sol Ktrap]
    if { ![string compare $Sol Int] } {
	set Ktrap "Vac * $SS * $Kt"
	set de [pdbGetString $Mat Vac Equation]
	pdbSetString $Mat Vac Equation "$de - $Ktrap"
    } elseif {![string compare $Sol Vac]} {
	set Ktrap "(1.4e15 - Vac) * $SS * $Kt"
	set de [pdbGetString $Mat Vac Equation]
	pdbSetString $Mat Vac Equation "ddt(Vac) + $de + $Ktrap"
    }

    pdbSetString $Mat $Sol Equation_${Side} "$Gen * Velocity_mag - $KsI - $Ktrap"
    puts "$Mat $Sol Equation_${Side} $Gen * Velocity_mag - $KsI - $Ktrap"
}


#boundary conditions - Hu style...
proc DefectBound { Mat Sol } {
    set pdbMat [pdbName $Mat]

    #get the names of the sides
    set s1 [FirstMat $pdbMat]
    set s2 [SecondMat $pdbMat]

    if {[pdbIsAvailable $s1 $Sol DiffModel]} {
	set s [pdbGetSwitch $s1 $Sol DiffModel]
	if {$s == 1} {DefectBoundSide $pdbMat $s1 $Sol}
    }

    if {[pdbIsAvailable $s2 $Sol DiffModel]} {
	set s [pdbGetSwitch $s2 $Sol DiffModel]
	if {$s == 1} {DefectBoundSide $pdbMat $s2 $Sol}
    }
}
