
proc logN { AvrRad } {
    return "exp(-0.5*((log($AvrRad)-Mean)^2)/(ssqr))"
}
proc LoopEng { pdbMat Sol AvgRad sign} {
    set mu [pdbGetDouble $pdbMat $Sol mu]    ;#shear modulus dyne/cm^2
    set b [pdbGetDouble $pdbMat $Sol b]      ;#mag. of Burgers vec. cm
    set ohm [pdbGetDouble $pdbMat $Sol ohm]  ;#atomic volume of Si 2.0e-23
    set kerg [pdbGetDouble $pdbMat $Sol kerg];# erg/K
    set nu [pdbGetDouble $pdbMat $Sol nu]    ;#Poisson's ratio 
    set er 1.0                               ;#0.0 < er < 1.0 
    set pi 3.141592 
    set TempK { ([simGetDouble Diffuse temp]+273.0)}
    set pre "([expr -1.0 *$mu * $b * $ohm / (4*$pi*(1.0-$nu)*$kerg)])"
    set coef "[expr 8/$b]"
    set loopengeqn "exp(($sign*$pre/$AvgRad)/$TempK*log($coef*$AvgRad))"
    return $loopengeqn
}
proc IntNumIntegral { pdbMat Sol sgn} {
    set sgn [expr -1.0*$sgn]
    set R "(2.0*([pdbGetString $pdbMat $Sol CpEqnRp]+1.0e-8))";#upper limit of integral
    set x "1.0e-8" ;#lower limit of integral
    set spacing [pdbGetDouble $pdbMat $Sol NumericalIntegralSpacing] 
    set a1 "([expr 1.0/$spacing]*($R-$x))"
    set a2 [logN $x]
    set a3 [LoopEng $pdbMat $Sol $x $sgn]
    set eqn1 "($a2*$a3)/($x)"
    set sumeqn "0.0"
    set i 1; #i=1..$spacing-1
    set uplmt [expr int([expr $spacing -1])];
    while { $i <= $uplmt } {
	set step "($x+$i*$a1)"
	set a4 [logN $step]
	set a5 [LoopEng $pdbMat $Sol $step $sgn]
	term name = Intsum${i} add eqn = "($a4*$a5)/$step"
	set sumeqn "$sumeqn+ Intsum$i"
	incr i
    }
    set a6 [logN $R]
    set a7 [LoopEng $pdbMat $Sol $R $sgn]
    set eqn2 "($a6*$a7)/($R)"
    set integraleqn "(0.5*$a1*($eqn1+2.0*($sumeqn)+$eqn2))"
    return $integraleqn
}
proc VacNumIntegral { pdbMat Sol sgn} {
    set sgn [expr -1.0*$sgn]
    set R "(2.0*([pdbGetString $pdbMat $Sol CpEqnRp]+1.0e-8))";#upper limit of integral
    set x "1.0e-8" ;#lower limit of integral
    set spacing [pdbGetDouble $pdbMat $Sol NumericalIntegralSpacing] 
    set a1 "([expr 1.0/$spacing]*($R-$x))"
    set a2 [logN $x]
    set a3 [LoopEng $pdbMat $Sol $x $sgn]
    set eqn1 "($a2*$a3)/($x)"
    set sumeqn "0.0"
    set i 1; #i=1..$spacing-1
    set uplmt [expr int([expr $spacing -1])];
    while { $i <= $uplmt } {
	set step "($x+$i*$a1)"
	set a4 [logN $step]
	set a5 [LoopEng $pdbMat $Sol $step $sgn]
	term name = Vacsum${i} add eqn = "($a4*$a5)/$step"
	set sumeqn "$sumeqn+ Vacsum$i"
	incr i
    }
    set a6 [logN $R]
    set a7 [LoopEng $pdbMat $Sol $R $sgn]
    set eqn2 "($a6*$a7)/($R)"
    set integraleqn "(0.5*$a1*($eqn1+2.0*($sumeqn)+$eqn2))"
    return $integraleqn
}
proc SoRBulk {Mat Sol} {
    set pdbMat [pdbName $Mat]
    set Ks0 0.33
    set Ks1 5e4
    pdbSetString $pdbMat $Sol Equation "$Sol-($Ks0+$Ks1*Rp)"
}

proc LoopBulkNall { Mat Sol } {
    set pdbMat [pdbName $Mat]
    set eqnNall "ddt($Sol)"
    set CpEqRp [pdbGetString $pdbMat $Sol CpEqnRp]
    set deflist [pdbGetString $pdbMat $Sol DefectsGrow] 
    foreach def $deflist {
	set sgn 1.0

	#set SoR Equation
# 	set Ks0 0.33
# 	set Ks1 5e4
# 	pdbSetString $pdbMat SoR Equation "SoR-($Ks0+$Ks1*$CpEqRp)"

	#Now set the Rp term Params
 	set sigmasqr "(log(1+(SoR)^2))"
	term name = ssqr add eqn = $sigmasqr
	set na 1.5e15
	set M "(log(sqrt(Nall/(Dall*$na*3.1415)))-ssqr)"
	term name = Mean add eqn = $M
	#Rp term
	term name = Rp add eqn = "(exp( Mean + ssqr/2))"

	#set up Nall Equation Params
	set Do [pdbDelayDouble $pdbMat $def D0]
	set latticesp [expr 2.0*[pdbGetDouble $pdbMat LatticeSpacing]]
	set gb [pdbGetDouble $pdbMat $def gb]
	set IntnumIntg "([IntNumIntegral $pdbMat $Sol $sgn])"
	set K [pdbDelayDouble $pdbMat $def K]
	set Kl "$CpEqRp*$K*4.0*3.141592*$Do*$latticesp"
	set Dall [pdbGetString $pdbMat $Sol CpEqnDall] 

	#interstitial emisson and absorption term
	term name = Loop${def} add eqn = "$Kl*$Dall*($def-Eq${def}*$gb*($IntnumIntg)/(sqrt(2.0*3.141592*ssqr)))"

	set Cib "(Eq${def}*$gb*[ LoopEng $pdbMat $Sol ($CpEqRp+1.0e-8) -1])"
	term name = Cib add eqn = $Cib
	#add the term to Int equation
	set equ [pdbGetString $pdbMat $def Equation]
 	pdbSetString $pdbMat $def Equation "$equ+ Loop$def"
	set Ktemp 1.0
 	if { [pdbGetBoolean $pdbMat $Sol NoNucleation] } {
	    set Ktemp 0.0 ;
	    #this will eliminate 311 unfaulting terms in Nall & Dall
 	}

	# {311} unfaulting term
	set K311 [pdbDelayDouble $pdbMat $Sol K311]
	set C311 [pdbGetString $pdbMat $Sol 311]
	term add name = LpNall311 eqn = "($Ktemp*$K311*$C311)"

	#add the term to C311 eqn
	set equ [pdbGetString $pdbMat $C311 Equation]
	pdbSetString $pdbMat $C311 Equation "$equ + LpNall311"

	#Now Nall Equation
	set eqnNall "$eqnNall - Loop${def} - LpNall311"
    }
    
    set deflist [pdbGetString $pdbMat $Sol DefectsShrink]
    foreach def $deflist {
	set sgn -1.0
	set Do [pdbDelayDouble $pdbMat $def D0]
	set latticesp [expr 2.0*[pdbGetDouble $pdbMat LatticeSpacing]]
	set Kv "$CpEqRp*[ expr [pdbDelayDouble $pdbMat $def K]*4.0* 3.141592*$Do*$latticesp]"
	set gb [pdbGetDouble $pdbMat $def gb]
	set VacnumIntg "([VacNumIntegral $pdbMat $Sol $sgn])"
	set Dall [pdbGetString $pdbMat $Sol CpEqnDall] 

	#vacancy emisson and absorption term
	term name = Loop${def} add eqn = "$Kv*$Dall*($def-Eq${def}*$gb*($VacnumIntg)/(sqrt(2.0*3.141592*ssqr)))"

	#add the term to Vac equation
	set equ [pdbGetString $pdbMat $def Equation]
 	pdbSetString $pdbMat $def Equation "$equ + Loop$def"
	set eqnNall "$eqnNall +  Loop$def"
    }
    pdbSetString $pdbMat $Sol Equation $eqnNall
    #puts "Solution Equation == $eqnNall "
}
 
proc LoopBulkDall { Mat Sol } {
    set pdbMat [pdbName $Mat]
    set KRp [pdbDelayDouble $pdbMat $Sol KRp]
    set CpEqnRp [pdbGetString $pdbMat $Sol CpEqnRp]
    set Ktemp 1.0
    
    set Nall [pdbGetString $pdbMat $Sol CpEqnNall]
    if { [pdbGetBoolean $pdbMat $Nall NoNucleation] } {
	#This will  make LpDall311 zero
	set Ktemp 0.0
    }
    # {311} unfaulting term
    set K311 [pdbDelayDouble $pdbMat $Nall K311]
    set D311 [pdbGetString $pdbMat $Sol 311]
    term add name = LpDall311 eqn = "($Ktemp*$K311*$D311)"
  
    #add the term to D311 eqn
    set equ [pdbGetString $pdbMat $D311 Equation]
    pdbSetString $pdbMat $D311 Equation "$equ + LpDall311"
    
    #set up Dall equation
    
    pdbSetString $pdbMat $Sol Equation "ddt($Sol)+1.0/(10.0+Int/Cib)*$KRp*$Sol*(2.0/($CpEqnRp)^2) - LpDall311"
    
}
