#*************************************************************************
#*									 *
#*     Copyright c 1998 The board of regents of the University of 	 *
#*                      Florida.  All rights reserved.			 *
#*     This subroutine may not be used outside of the FLOO* computer	 *
#*     programs without the prior written consent of the University of 	 *
#*     Florida.								 *
#*									 *
#*************************************************************************/
#*   FileMenu.tcl                Version 1.4     */
#*   Last Modification : 06/11/98 10:20:04  */

set Orient 1
set Size 0
set Color color

proc PrintOptions {} {
    global Orient Size Color
    toplevel .po
    table .po
    wm title .po "Print Options"
    label .po.l1 -text "Orientation"
    radiobutton .po.r1 -text landscape -variable Orient -value 1
    radiobutton .po.r2 -text portrait -variable Orient -value 0
    label .po.l2 -text "Size"
    set Size 0
    radiobutton .po.r3 -text "Fill Page" -variable Size -value 1 -command {
	.po.wl configure -enable
    }
    radiobutton .po.r4 -text "Same as Window" -variable Size -value 0 -command {
    }
    radiobutton .po.r8 -text "Set Size" -variable Size -value 2 -command {
    }

    label .po.wl -text "Width"
    entry .po.wv -width 5
    label .po.hl -text "Height"
    entry .po.hv -width 5

    label .po.l3 -text "Color"
    radiobutton .po.r5 -text "Full Color" -variable Color -value color
    radiobutton .po.r6 -text "Grey Scale" -variable Color -value grey
    radiobutton .po.r7 -text "Mono" -variable Color -value mono
    button .po.c1 -text Cancel -command {destroy .po}
    button .po.c2 -text Update -command {
	global Orient Size graph
	$graph postscript configure -landscape $Orient
	if { $Size != 2 } {
	    $graph postscript configure -maxpect $Size
	} else {
	    $graph postscript configure -maxpect 0
	    $graph postscript configure -height [.po.hv get]i 
	    $graph postscript configure -width [.po.hv get]i
	    puts "Size [.po.wv get] [.po.hv get]"
	}
	$graph postscript configure -colormode $Color
	destroy .po
    }

    table .po \
	.po.l2 0,0	\
	.po.r3 0,1	\
	.po.r4 0,2	\
	.po.r8 0,3	\
	.po.wl 1,0	\
	.po.wv 1,1	\
	.po.hl 1,2	\
	.po.hv 1,3	\
	.po.l1 2,0	\
	.po.r1 2,1	\
	.po.r2 2,2	\
	.po.l3 3,0	\
	.po.r5 3,1	\
	.po.r6 3,2	\
	.po.r7 3,3	\
	.po.c1 0,4	\
	.po.c2 1,4	

    grab .po
    tkwait window .po
}

set tmpLoc "tmp.[pid]"
set Dest 1

proc Print {} {
    global graph PrintCmd
    toplevel .print
    table .print

    button .print.c1 -text Cancel -command {destroy .print}
    button .print.c2 -text Print -command {
	global graph PrintCmd tmpLoc Dest
	set PrintCmd [.print.cmd get]
	if {! $Dest } {
	    set file [tk_getSaveFile]
	    $graph postscript output $file
	} else {
	    $graph postscript output $tmpLoc
	    eval exec $PrintCmd $tmpLoc
	    eval exec rm $tmpLoc
	}
	destroy .print
    }

    label .print.l1 -text "Destination"
    frame .print.f1 -relief sunken -width 2 -bd 2
    frame .print.f2 -bd 2 -height 2 -relief sunken
    radiobutton .print.r1 -text "File" -variable Dest -value 0
    radiobutton .print.r2 -text "Printer" -variable Dest -value 1
    label .print.c3 -text "Print Command"
    entry .print.cmd -width 20
    .print.cmd delete 0 end
    .print.cmd insert 0 $PrintCmd

    button .print.c4 -text "Options" -command PrintOptions

    table .print \
	.print.l1 0,0	\
	.print.r1 1,0	-fill x \
	.print.r2 2,0	-fill x 
    table .print \
	.print.f2 3,0	-fill x -pady 10 -cspan 4 \
	.print.c3 4,0	-fill x -cspan 2 \
	.print.cmd 5,0  -fill x -cspan 2 \
	.print.c4 5,3	-fill x 
    table .print \
	.print.f1 0,2   -rspan 3 -fill y -padx 10 \
	.print.c1 1,3 -fill x 	\
	.print.c2 2,3 -fill x
}


proc New {} {
    foreach line [.graph element name] {
	.graph element delete $line
    }
    foreach line [.graph marker names] {
	.graph marker delete $line
    }
    foreach line [.graph pen names] {
	if {! [string match "active*" $line] } {
	    .graph pen delete $line
	}
    }
}


proc SavePlot {File} {
    set f [open $File w]

    #save all pens
    foreach line [.graph pen name] {
	set lead ".graph pen configure $line"
	if {! [string match "active*" $line] } {
	    puts $f ".graph pen create $line"
	    foreach opt [.graph pen configure $line] {
		set str [format "$lead %s {%s}" [lindex $opt 0] [lindex $opt 4]]
		puts $f $str
	    }
	}
    }

    #save all elements
    foreach line [.graph element name] {
	set lead ".graph element configure $line"
	puts $f ".graph element create $line"
	foreach opt [.graph element configure $line] {
	    set str [format "$lead %s {%s}" [lindex $opt 0] [lindex $opt 4]]
	    puts $f $str
	}
    }

    #save all markers
    foreach line [.graph marker name] {
	set lead ".graph marker configure $line"
	if {! [string match "zoom*" $line] } {
	    puts $f ".graph marker create [.graph marker type $line] -name $line"
	    foreach opt [.graph marker configure $line] {
		set str [format "$lead %s {%s}" [lindex $opt 0] [lindex $opt 4]]
		puts $f $str
	    }
	}
    }

    #save axis options
    foreach line [.graph axis name] {
	set lead ".graph axis configure $line"
	set optlist "-title -titlefont -hide -logscale -min -max -descending"
	foreach opt $optlist {
	    set val [.graph axis cget $line $opt]
	    set str [format "$lead %s {%s}" $opt $val]
	    puts $f $str
	}
    }

    set title [.graph cget -title]
    puts $f ".graph configure -title {$title}"

    close $f
}


proc SaveDefault {} {
    global PreferenceFile
    set f [open $PreferenceFile w]

    #save active pen information
    set lead ".graph pen configure activeLine"
    foreach opt [.graph pen configure activeLine] {
	set str [format "$lead %s {%s}" [lindex $opt 0] [lindex $opt 4]]
	puts $f $str
    }

    #save axis options
    foreach line [.graph axis name] {
	set lead ".graph axis configure $line"
	set optlist "-title -titlefont -loose -descending -tickfont"
	foreach opt $optlist {
	    set val [.graph axis cget $line $opt]
	    set str [format "$lead %s {%s}" $opt $val]
	    puts $f $str
	}
    }

    global PrintCmd
    puts $f "set PrintCmd {$PrintCmd}"

    #save legend options
    set lead ".graph legend configure"
    set opt [.graph legend configure -font] 
    set str [format "$lead %s {%s}" [lindex $opt 0] [lindex $opt 4]]
    puts $f $str
    set opt [.graph legend configure -position] 
    set str [format "$lead %s {%s}" [lindex $opt 0] [lindex $opt 4]]
    puts $f $str
    set opt [.graph legend configure -anchor] 
    set str [format "$lead %s {%s}" [lindex $opt 0] [lindex $opt 4]]
    puts $f $str

    #save main graph options
    set lead ".graph configure"
    set opt [.graph configure -title] 
    set str [format "$lead %s {%s}" [lindex $opt 0] [lindex $opt 4]]
    puts $f $str
    set opt [.graph configure -font] 
    set str [format "$lead %s {%s}" [lindex $opt 0] [lindex $opt 4]]
    puts $f $str

    close $f
}


proc Export {File} {
    set f [open $File w]

    #find the longest data list
    set maxlen 0
    foreach line [.graph element name] {
	set len [llength [.graph element cget $line -xdata]]
	if { $len > $maxlen } {set maxlen $len}
    }

    #write out the column titles
    foreach line [.graph element name] {
	puts -nonewline $f "${line}X\t${line}Y\t"
    }
    puts $f ""

    for {set i 0} {$i < $maxlen} {incr i} {
	foreach line [.graph element name] {
	    set xl [.graph element cget $line -xdata]
	    set xd [lindex $xl $i]
	    puts -nonewline $f "$xd\t"
	    set yl [.graph element cget $line -ydata]
	    set yd [lindex $yl $i]
	    puts -nonewline $f "$yd\t"
	}
	puts $f ""
    }

    close $f
}


proc SaveImage { file } {
    .graph snap .image
    .image write $file -format gif
}
