#*************************************************************************
#*									 *
#*     Copyright c 1998 The board of regents of the University of 	 *
#*                      Florida.  All rights reserved.			 *
#*     This subroutine may not be used outside of the FLOO* computer	 *
#*     programs without the prior written consent of the University of 	 *
#*     Florida.								 *
#*									 *
#*************************************************************************/
#*   LineMenu.tcl                Version 1.1     */
#*   Last Modification : 01/23/98 08:52:52  */

proc MakeFont {font wght slant pxl} {
    set sl [string index $slant 0]
    return *-${font}-${wght}-${sl}-Normal-*-${pxl}-[expr $pxl*10]-*
}

proc SelectFont {title deffnt} {
    toplevel .sf
    table .sf
    wm title .sf $title
    global font wght slant pxl

    tk_optionMenu .sf.fam font Helvetica Times Palatino Courier Bookman	
    set font [lindex [split $deffnt -] 1]
    .sf.fam configure -width 9

    tk_optionMenu .sf.wght wght Bold Medium 
    .sf.wght configure -width 6
    set wght [lindex [split $deffnt -] 2]

    tk_optionMenu .sf.slant slant roman italic 
    .sf.slant configure -width 6
    set slant [lindex [split $deffnt -] 3]
    if {[string compare $slant r] == 0} {set slant roman} else {set slant italic}

    tk_optionMenu .sf.pxl pxl 8 10 12 14 18 24 36
    .sf.pxl configure -width 2
    set pxl [lindex [split $deffnt -] 6]

    table .sf \
	.sf.fam 0,0	\
	.sf.wght 0,1	\
	.sf.slant 0,2	\
	.sf.pxl 0,3	\
    
    label .sf.msg -text "ABCDEFGHIJKLMNOPQRSTUVWXY\nabcdefghijklmnopqrstuvwxy\n0123456790" -font $deffnt
    
    table .sf .sf.msg 1,0 -cspan 3 -fill x 

    frame .sf.sep -height 2 -relief sunken -bd 2
    table .sf .sf.sep 2,0 -cspan 3 -fill x -padx 4

    button .sf.b1 -text "Display" -command {
	global font wght slant pxl
	set foo [catch {.sf.msg configure -font [MakeFont $font $wght $slant $pxl]} err ]
	if {$foo} { tk_messageBox -icon error -message $err -type ok }
    }
    button .sf.b2 -text "Cancel" -command "
	global font wght slant pxl
	set font [lindex [split $deffnt -] 1]
	set wght [lindex [split $deffnt -] 2]
	set slant [lindex [split $deffnt -] 3]
	set pxl [lindex [split $deffnt -] 6]
	destroy .sf
    "
    button .sf.b3 -text Update -command {
	global font wght slant pxl
	set foo [catch {
	    .sf.msg configure -font [MakeFont $font $wght $slant $pxl]} err ]
	if {$foo} { 
	    tk_messageBox -icon error -message $err -type ok 
	} else {
	    destroy .sf
	}
    }
    table .sf \
	.sf.b1 3,0	\
	.sf.b2 3,2	\
	.sf.b3 3,3

    grab .sf
    tkwait window .sf
    return [MakeFont $font $wght $slant $pxl]
}

proc Toggle {w varName on off} {
    upvar #0 $varName var
    if { [string compare $var $off] == 0 } {
	$w configure -text $on
	set var $on
    } else {
	$w configure -text $off
	set var $off
    }
}

proc ToggleButton {w varName on off} {
    upvar #0 $varName var
    set won [string length $on]
    set woff [string length $off]
    set max [expr ($won > $woff)?$won:$woff]

    button $w -text $off -width $max -command "Toggle $w $varName $on $off"

    if ![info exists var] {set var $on}
    if { [string compare $var $on] == 0 } {
	$w configure -text $on
    } else {
	$w configure -text $off
    }
    return $w
}


proc InvertColor {color} {
    set inv [string index $color 0]
    if {[string compare $inv "#"] != 0} {return white}
    for {set var 1} {$var < [string length $color]} {incr var} {
	switch [string index $color $var] {
	    0 {set inv ${inv}f}
	    1 {set inv ${inv}e}
	    2 {set inv ${inv}d}
	    3 {set inv ${inv}c}
	    4 {set inv ${inv}b}
	    5 {set inv ${inv}a}
	    6 {set inv ${inv}9}
	    7 {set inv ${inv}8}
	    8 {set inv ${inv}7}
	    9 {set inv ${inv}6}
	    a {set inv ${inv}5}
	    b {set inv ${inv}4}
	    c {set inv ${inv}3}
	    d {set inv ${inv}2}
	    e {set inv ${inv}1}
	    f {set inv ${inv}0}
	}
    }
    return $inv
}

proc LineMenu {graph lm} {
    $lm delete 0 end
    $lm add command -label Legend... -command "LegendDialog $graph"
    $lm add separator
    set names [$graph pen names]
    foreach n "$names" {
	$lm add command -label "$n..." -command "LineDialog $graph $n"
    }
}

proc LineDialog {graph line} {
    toplevel .ld
    table .ld
    wm title .ld "Options for $line"

    #title the dialog box
    label .ld.title -text $line 
    table .ld \
	.ld.title 0,0 -cspan 2 -anchor w	

    #allow choice of legend name
    set element [expr [lsearch [$graph element names] $line] != -1]
    if {$element} {
	set LegendName [$graph element cget $line -label]
	label .ld.l1 -text "Legend Name:"
	entry .ld.e1 
	.ld.e1 delete 0 end
	.ld.e1 insert 0 $LegendName

	table .ld \
	    .ld.l1 1,0 -anchor w		\
	    .ld.e1 1,1	
    }

    label .ld.lc -text "Color"
    label .ld.lf -text "Size"

    table .ld \
	.ld.lc 2,1	\
	.ld.lf 2,2

    #allow choice of line color
    label .ld.ll1 -text "Line: "
    global linecol
    set linecol [$graph pen cget $line -color]
    button .ld.b1 -text LineColor -bg $linecol -fg [InvertColor $linecol] -command {
	global graph
	set line [.ld.title cget -text]
	set linecol [$graph pen cget $line -color]
	set tmp [tk_chooseColor -initialcolor $linecol -title $line]
	if { $tmp != "" } {set linecol $tmp}
	.ld.b1 configure -bg $linecol -fg [InvertColor $linecol]
    }
    global linewidth
    tk_optionMenu .ld.om1 linewidth 0 1 2 3 4 5 6 7 8 9 10
    set linewidth [$graph pen cget $line -linewidth]
    table .ld \
	.ld.ll1 3,0 -fill x -padx 5 \
	.ld.b1 3,1 -fill x -padx 5 \
	.ld.om1 3,2 -fill x -padx 5

    #allow choice of symbol
    global symbol
    set om [tk_optionMenu .ld.om symbol square circle diamond plus cross splus scross triangle none]
    .ld.om configure -width 8
    set symbol [$graph pen cget $line -symbol]
    if {[string compare $symbol ""] == 0} {[set symbol none]}
    global symbcol
    set symbcol [$graph pen cget $line -fill]
    if { [string compare $symbcol defcolor] == 0 } {
	set symbcol [$graph pen cget $line -color]
    }
    button .ld.b2 -text SymbolColor -bg $symbcol -fg [InvertColor $symbcol] -command {
	global graph symbcol
	set line [.ld.title cget -text]
	set tmp [tk_chooseColor -initialcolor $symbcol -title $line]
	if { $tmp != "" } {set symbcol $tmp}
	.ld.b2 configure -bg $symbcol -fg [InvertColor $symbcol] 
    }
    global symbolsize
    tk_optionMenu .ld.om2 symbolsize 0 2 4 6 8 9 10 11 12 14 18 24
    set symbolsize [$graph pen cget $line -pixels]
    table .ld \
	.ld.om 4,0 -fill x -padx 5	\
	.ld.b2 4,1 -fill x -padx 5	\
	.ld.om2 4,2 -fill x -padx 5	

    frame .ld.sep2 -height 2 -relief sunken -bd 2
    table .ld .ld.sep2 5,0 -cspan 6 -pady 4 -fill x

    global xaxis yaxis
    if {$element} {
	set xaxis [$graph element cget $line -mapx]
	set yaxis [$graph element cget $line -mapy]
	label .ld.axl -text "Axes"
	ToggleButton .ld.tb1 xaxis x x2
	ToggleButton .ld.tb2 yaxis y y2
	table .ld \
	    .ld.axl 6,0	\
	    .ld.tb1 6,1	\
	    .ld.tb2 6,2	

	frame .ld.sep3 -height 2 -relief sunken -bd 2
	table .ld .ld.sep3 7,0 -cspan 6 -pady 4 -fill x
    }

    #display or hide the data
    if {$element} {
	global Hide
	label .ld.lh -text "Variable is:"
	if [$graph element cget $line -hide] {
	    set Hide Hidden
	} else {
	    set Hide Shown
	}
	ToggleButton .ld.b4 Hide Hidden Shown
	table .ld .ld.b4 8,1 -anchor w  .ld.lh 8,0 
    }

    button .ld.c1 -text Cancel -command {destroy .ld}
    button .ld.c2 -text Update -command {
	global graph symbol Hide linecol symbcol xaxis yaxis
	set line [.ld.title cget -text]
	if { [string compare $symbol none ] == 0 } {
	    $graph pen configure $line -symbol ""
	} else {
	    $graph pen configure $line -symbol $symbol
	}
	$graph pen configure $line -linewidth $linewidth
	$graph pen configure $line -pixels $symbolsize
	$graph pen configure $line -color $linecol
	$graph pen configure $line -fill $symbcol
	$graph pen configure $line -symbol $symbol
	set element [expr [lsearch [$graph element names] $line] != -1]
	if {$element} {
	    set Name [.ld.e1 get]
	    $graph element configure $line -label $Name
	    foreach l [$graph element names] {
		set p [$graph element cget $l -pen]
		if {[string compare $p $line]} {
		    $graph element configure $line -mapx $xaxis
		    $graph element configure $line -mapy $yaxis
		    if {[string compare $Hide Hidden] == 0} {
			$graph element configure $line -hide 1
		    } else {
			$graph element configure $line -hide 0
		    }
		}
	    }
	}
	destroy .ld
    }
    frame .ld.sep -relief sunken -width 2 -bd 2 

    table .ld \
	.ld.sep 0,3 -rspan 9 -fill y -padx 5	\
	.ld.c1 1,4 -anchor e	\
	.ld.c2 2,4 -anchor e \

    grab .ld
    tkwait window .ld
}

proc LegendDialog {graph} {
    toplevel .ld
    table .ld
    wm title .ld "Legend and Graph Options"

    #allow choice of graph title
    set GraphTitle [$graph cget -title]
    label .ld.l1 -text "Graph Title:"
    entry .ld.e1 
    .ld.e1 delete 0 end
    .ld.e1 insert 0 $GraphTitle

    global fontStr
    set fontStr [$graph cget -font]
    button .ld.b1 -text "Title Font" -command {
	set fontStr [SelectFont "Title Font" $fontStr]
	.ld.b1 configure -font $fontStr
    }

    table .ld \
	.ld.l1 1,0 -anchor w -padx 5		\
	.ld.e1 1,1 -padx 5	\
	.ld.b1 1,2 -padx 5

    frame .ld.sep1 -relief sunken -height 2 -bd 2 
    table .ld .ld.sep1 2,0 -cspan 3 -fill x -pady 4

    label .ld.l2 -text "Legend Options"
    global legendFont position spot
    set legendFont [$graph legend cget -font]
    button .ld.b2 -text "Legend Font" -command {
	set legendFont [SelectFont "Legend Font" $legendFont]
	.ld.b2 configure -font $legendFont
    }
    ToggleButton .ld.b3 position margin plotarea
    tk_optionMenu .ld.b4 spot nw n ne e se s sw w

    table .ld	\
	.ld.l2 3,0 -pady 4	\
	.ld.b2 4,0	\
	.ld.b3 4,1	\
	.ld.b4 4,2	

    button .ld.c1 -text Cancel -command {destroy .ld}
    button .ld.c2 -text Update -command {
	global graph fontStr legendFont position spot
	set Name [.ld.e1 get]
	$graph configure -title $Name
	$graph configure -font $fontStr
	$graph legend configure -font $legendFont
	if {[string compare $position margin]==0} {
	    switch $spot {
		nw {$graph legend configure -position top}
		n {$graph legend configure -position top}
		ne {$graph legend configure -position top}
		e {$graph legend configure -position right}
		se {$graph legend configure -position right}
		s {$graph legend configure -position bottom}
		sw {$graph legend configure -position left}
		w {$graph legend configure -position left}
	    }
	} else {
	    $graph legend configure -position plotarea
	    $graph legend configure -anchor $spot
	}
	destroy .ld
    }
    frame .ld.sep -relief sunken -width 2 -bd 2 

    table .ld \
	.ld.sep 0,3 -rspan 6 -fill y -padx 5	\
	.ld.c1 1,4 -anchor e	\
	.ld.c2 2,4 -anchor e \

    grab .ld
    tkwait window .ld
}

proc MarkerMenu {graph lm} {
    $lm delete 0 end
    $lm add command -label "New Marker..." -command "NewMarker $graph"
    $lm add separator
    set names [lsort [$graph marker names]]
    foreach n "$names" {
	if {! [string match "zoom*" $n]} {
	    switch [$graph marker type $n] {
		polygon {
		    $lm add command -label "$n..." -command "MPolyDialog $graph $n"
		}
		text {
		    $lm add command -label "$n..." -command "MTextDialog $graph $n"
		}
	    }
	}
    }
}

proc MPolyDialog {graph line} {
    toplevel .ld
    table .ld
    wm title .ld "Polygon Marker Dialog"

    #title the dialog box
    label .ld.title -text $line 
    table .ld \
	.ld.title 0,0 -cspan 2 -anchor w	

    #allow choice of fill color and option
    global fill fillcol
    set fillcol [lindex [$graph marker cget $line -fill] 0]
    set fill [expr [string compare $fillcol "{}"] != 0]
    if {! $fill} {set fillcol #000000}
    set fill [expr [string compare $fillcol ""] != 0]
    if {! $fill} {set fillcol #000000}
    checkbutton .ld.cb -text Fill -variable fill
    button .ld.b2 -text "Fill Color" -bg $fillcol -fg [InvertColor $fillcol] -command {
	global graph fillcol
	set line [.ld.title cget -text]
	set tmp [tk_chooseColor -initialcolor $fillcol -title $line]
	if { $tmp != "" } {set fillcol $tmp}
	.ld.b2 configure -bg $fillcol -fg [InvertColor $fillcol] 
    }
    table .ld \
	.ld.cb 2,0	\
	.ld.b2 2,1 -fill x -padx 5	\

    #display or hide the data
    global Hide
    label .ld.lh -text "Marker is:"
    if [$graph marker cget $line -hide] {
	set Hide Hidden
    } else {
	set Hide Shown
    }
    ToggleButton .ld.b4 Hide Hidden Shown
    table .ld .ld.b4 4,1 -anchor w -fill x -padx 5  .ld.lh 4,0 

    button .ld.c1 -text Cancel -command {destroy .ld}
    button .ld.c2 -text Update -command {
	global graph fill Hide fillcol linecol
	set line [.ld.title cget -text]
	if {$fill} {
	    $graph marker configure $line -fill $fillcol
	} else {
	    $graph marker configure $line -fill {}
	}
	if {[string compare $Hide Hidden] == 0} {
	    $graph marker configure $line -hide 1
	} else {
	    $graph marker configure $line -hide 0
	}
	destroy .ld
    }
    frame .ld.sep -relief sunken -width 2 -bd 2 

    table .ld \
	.ld.sep 0,3 -rspan 9 -fill y -padx 5	\
	.ld.c1 1,4 -anchor e	\
	.ld.c2 2,4 -anchor e \

    grab .ld
    tkwait window .ld
}


proc MTextDialog {graph line} {
    tk_dialog .err Error "Not Implemented Yet" error 0 Ok
}

proc NewMarker {graph} {
    tk_dialog .err Error "Not Implemented Yet" error 0 Ok
}


