#/************************************************************************
#*									 *
#*     Copyright c 1998 The board of regents of the University of 	 *
#*                      Florida.  All rights reserved.			 *
#*     This subroutine may not be used outside of the FLOO* computer	 *
#*     programs without the prior written consent of the University of 	 *
#*     Florida.								 *
#*									 *
#*************************************************************************/
#/*   sigma.tcl                Version 1.2     */
#/*   Last Modification : 06/11/98 10:20:43  */


proc sigma {Dopant chkmat backgrnd} {
    sel z = $Dopant
    set origlist [slice $chkmat]
    set origlist [lindex $origlist 0]
    set data [list]

    #build a new shorter list
    for {set j 0} {$j < [llength $origlist]/2} {incr j} {
        set dep [lindex $origlist [expr 2*$j]]
        set val [lindex $origlist [expr 2*$j+1]]
	if { $val > $backgrnd } {
	    lappend data $dep 
	    lappend data $val
	}
    }

    return [FitPearson $data]
}


proc FitPearson {origlist} {
    #build the oldformat list
    set data [list]
    for {set j 0} {$j < [llength $origlist]/2} {incr j} {
	set dep [lindex $origlist [expr 2*$j]]
	set val [lindex $origlist [expr 2*$j+1]]
	lappend data [list $dep $val]
    }

    #if the peak is at the surface...
    set peak [lindex [lindex $data 0] 1]
    set ip 1
    for {set j 1} {$j < [llength $data] && $ip} {incr j} {
	set pair [lindex $data $j]
	set val [lindex $pair 1]
	if {$val > 1.05 * $peak} {set ip 0}
    }

    #compute the projected range and dose
    set dose 0.0
    set integ 0.0
    for {set j 0} {$j < [llength $data]-1} {incr j} {
	set opair [lindex $data $j]
	set npair [lindex $data [expr $j + 1]]
	set d1 [lindex $opair 0]
	set v1 [lindex $opair 1]
	set d2 [lindex $npair 0]
	set v2 [lindex $npair 1]
	set dose [expr $dose + 0.5*($d2-$d1)*($v1+$v2)]
	set integ [expr $integ + 0.5*($d2-$d1)*($d1*$v1+$d2*$v2)]
	if {$ip} {
	    set integ [expr $integ - 0.5*($d2-$d1)*($d1*$v1+$d2*$v2)]
	}
    }
    set range [expr $integ / $dose]

    #compute the standard deviation
    set Istd 0.0
    set Iskew 0.0
    set Ikurt 0.0
    for {set j 0} {$j < [llength $data]-1} {incr j} {
	set opair [lindex $data $j]
	set npair [lindex $data [expr $j + 1]]
	set d0 [expr [lindex $opair 0] - $range]
	set dn [expr [lindex $npair 0] - $range]
	set v0 [lindex $opair 1]
	set vn [lindex $npair 1]
	set d0_2 [expr $d0 * $d0]
	set dn_2 [expr $dn * $dn]
	set d0_3 [expr $d0_2 * $d0]
	set dn_3 [expr $dn_2 * $dn]
	set d0_4 [expr $d0_3 * $d0]
	set dn_4 [expr $dn_3 * $dn]
	set Istd [expr $Istd + 0.5*($dn-$d0)*($d0_2*$v0+$dn_2*$vn)]
	set Iskew [expr $Iskew + 0.5*($dn-$d0)*($d0_3*$v0+$dn_3*$vn)]
	set Ikurt [expr $Ikurt + 0.5*($dn-$d0)*($d0_4*$v0+$dn_4*$vn)]
	if { $ip } {
	    set Istd [expr $Istd + 0.5*($dn-$d0)*($d0_2*$v0+$dn_2*$vn)]
	    set Iskew [expr $Iskew - 0.5*($dn-$d0)*($d0_3*$v0+$dn_3*$vn)]
	    set Ikurt [expr $Ikurt + 0.5*($dn-$d0)*($d0_4*$v0+$dn_4*$vn)]
	}
    }
    set std [expr sqrt( $Istd / $dose )]
    set skew [expr $Iskew / ($dose * $std * $std * $std)]
    set kurt [expr $Ikurt / ($dose * $std * $std * $std * $std)]
    return [list $dose $range $std $skew $kurt]
}


proc PearsonValue {depth Params} {
    #read the param list
    set dose [lindex $Params 0]
    set offset [lindex $Params 1]
    set Range [lindex $Params 2]
    set A0 [lindex $Params 3]
    set B0 [lindex $Params 4]
    set B2 [lindex $Params 5]
    set disc [expr $A0 * $A0 - 4 * $B0 * $B2]
    set f4 [expr sqrt( abs( $disc ) )]
    set f2 [expr $A0 / $B2 + 2.0 * $A0]
    set y1 [expr -0.5 * ($A0 - $f4) / $B2]
    set y2 [expr -0.5 * ($A0 + $f4) / $B2]
    if { $y1 < $y2 } {
	set ymin $y1
	set ymax $y2
    } else {
	set ymin $y2
	set ymax $y1
    }

    #three different Pearson functions are supported...
    set x [expr $depth - $Range]
    set f1 [expr log(abs($B0 + $A0*$x + $B2*$x*$x)) / (2.0 * $B2)]
    set f3 [expr 2.0 * $B2 * $x + $A0]
    if { $disc < 0.0 } {
	#Pearson IV distribution
	set f5 [expr atan2($f3, $f4)]
	set val [expr $f1 - $f2 * $f5 / $f4 ]
    } elseif {$disc == 0.0} {
	#Pearson V
	if { ($A0 < $ymin) && ($x > $ymin) } {
	    set val 0.0
	} elseif {($A0>$ymin)&&($A0<$ymax)&&(($x<$ymin)||($x>$ymax))} {
	    set val 0.0
	} elseif { ($A0 > $ymax) && ($x < $ymax) } {
	    set val 0.0
	} else {
	    set val [expr $f1 + $f2 / $f3 ]
	}
    } else {
	#Pearson VI
	if { ($A0 < $ymin) && ($x > $ymin) } {
	    set val 0.0
	} elseif {($A0>$ymin)&&($A0<$ymax)&&(($x<$ymin)||($x>$ymax))} {
	    set val 0.0
	} elseif { ($A0 > $ymax) && ($x < $ymax) } {
	    set val 0.0
	} else {
	    set f5 [expr log( abs(($f3 - $f4) / ($f3 + $f4)) ) ]
	    set val [expr ( $f1 - 0.5 * $f2 * $f5 / $f4 ) ]
	}
    }

    if {$dose == 0.0} {
	set ret $val
    } else {
	if { $val-$offset < -500.0 } {
	    set ret 0.0
	} elseif { $val == 0.0 } {
	    set ret 0.0
	} else {
	    set ret [expr $dose * exp( $val - $offset )]
	}
    }
    return $ret
}

proc BuildProfile { Name par } {
    #read off the profile information
    set Dose [lindex $par 0]
    set Range [lindex $par 1]
    set Std [lindex $par 2]
    set Skew [lindex $par 3]
    set Kurt [lindex $par 4]

    #build the Pearson parameters
    set gamsq [expr $Skew * $Skew]
    set ca [expr 10.0 * $Kurt - 12.0 * $Skew * $Skew - 18]
    set A0 [expr -$Skew * $Std * ($Kurt + 3.) / $ca ]
    set B0 [expr -$Std * $Std * (4. * $Kurt - 3. * $gamsq) / $ca ]
    set B2 [expr -(2. * $Kurt - 3. * $gamsq - 6.)  / $ca ]

    #store the Pearson Parameters
    set PearsParams [list 0.0 0.0 $Range $A0 $B0 $B2]
    set Peak [PearsonValue [expr $Range+$A0] $PearsParams]
    set scale [expr $Dose  / $Std]
    set PearsParams [list $Dose $Peak $Range $A0 $B0 $B2]

    #foreach depth
    tclsel z = {[PearsonValue $x $PearsParams]} name=Tmp
    sel z=Tmp

    set ll [layers]
    set sum 0.0
    for {set j 1} {$j < [llength $ll]} {incr j} {
	set sum [ expr $sum + [lindex [lindex $ll $j] 2] ]
    }
    set scale [expr 1.0e-4 * $Dose / $sum]

    sel z=$scale*Tmp name=$Name
}
